/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 202                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-24 17:29:41 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file bs_badwords.c
 *   @brief This is the Module file for BotServ BADWORDS Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the BotServ BADWORDS command.
 *
 *   The BotServ BADWORDS Command allows the user to manage<br/>
 *   the Badwords for target Channel.
 *
 *   Syntax is: /msg BotServ BADWORDS #channel { ADD | DEL | LIST | CLEAR }
 *
 *<table border="0">
 *    <tr><td> ADD		</td><td> Syntax is: /msg BotServ BADWORDS #channel ADD		</td><td> badword [SINGLE | START | END]	</td></tr>
 *    <tr><td> DEL		</td><td> Syntax is: /msg BotServ BADWORDS #channel DEL		</td><td> { badword | badword-num }			</td></tr>
 *    <tr><td> LIST		</td><td> Syntax is: /msg BotServ BADWORDS #channel LIST	</td><td> [*pattern*]						</td></tr>
 *    <tr><td> CLEAR	</td><td> Syntax is: /msg BotServ BADWORDS #channel CLEAR	</td><td>									</td></tr>
 *</table>
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"

#define SERVICE "bs"
#define MODUL "badwords"
#define MODUL_FILE "bs_badwords"
#define MODUL_VERSION "1.0"

MOD_BOOL bs_badwords_add(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL bs_badwords_del(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL bs_badwords_list(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL bs_badwords_clear(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);

MOD_BOOL bs_badwords_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL bs_badwords_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"badwords * option[1]","badwords");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,2,LOGINONLY,OPER_IF_DEFCON_BIT);
	addAccess (thread,&Module,BOTH);
	addMODType(thread,&Module,CORE);

	addLeastModuleLevels(thread,&Module,"badwords",is_services_user,is_services_user); /* just to add a displaylevel for /msg bs help
															  this should always be the lowest used Module level! */
	
	addCommand(thread,&Module,"badwords add",bs_badwords_add,is_services_user,is_services_user,6,7);
	addCommand(thread,&Module,"badwords del",bs_badwords_add,is_services_user,is_services_user,6,6);
	addCommand(thread,&Module,"badwords list",bs_badwords_list,is_services_user,is_services_user,5,6);
	addCommand(thread,&Module,"badwords clear",bs_badwords_clear,is_services_user,is_services_user,5,5);
	
	addCommand(thread,&Module,"help badwords",bs_badwords_help,is_services_user,is_services_user,4,4);
	addCommand(thread,&Module,"help",bs_badwords_help_listing,is_services_user,is_services_user,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}


/*  ************************************************************************************************  */

/**
 * @brief BS BADWORDS Add function for the BotServ BADWORDS Command.
 * 
 * The BotServ BADWORDS Add Command allows the user to Add Badwords for a Channel
 *  
 * Syntax is: /msg BotServ BADWORDS #channel ADD badword [SINGLE | START | END]<br/>
 *  If you leave the last parameter, the badword is matching wildcards.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL bs_badwords_add(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	//ChannelDB c;
	
	/* Initialise Variables to (NULL) */
	//bzero(&c,sizeof(c));
	
	/*
	 * Badwords Add Command
	 */

	//:shock PRIVMSG BotServ :badwords #channel add ROFL [SINGLE | START | END]
	//:shock PRIVMSG BotServ :badwords #channel del [ROFL | 1]
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"badwords add")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		//c = sql_chan_db(thread,param[4]); //Channel c - STRUCT
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT))	send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(thread->c.chan.bot[0]=='\0')									send_lang(thread,language,howtosend,snick,nick,"BS_NO_BOT_ASSIGNED_FOR_CHANNEL_X",thread->c.chan.name);
		
		else if(streqnc(thread->c.chan.serv,"chanserv") || streqnc(thread->c.chan.serv,"q")){
			if(stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.badwords || strcor(thread->c.chan.access.ownlevel,"mno") || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"badwords add")){
				if(CheckOption(thread,snick,nick,READONLY)){
					if(stoi(mysql_sel_var(thread,"SELECT count(1) FROM `orion_bs_badwords` WHERE `channel`='%s'",mysql_quote(thread,thread->c.chan.name)))<m->BSBadWordsMax){
						if(!streqnc(mysql_sel_var(thread,"SELECT `badword` FROM `orion_bs_badwords` WHERE `channel`='%s' AND `badword` LIKE '%c%s%c'",mysql_quote(thread,thread->c.chan.name),'%',mysql_quote(thread,param[6]),'%'),param[6])){
							if(!param[7]) domysql(thread,"INSERT INTO `orion_bs_badwords` (`art`,`channel`,`badword`) VALUES ('0','%s','%s')",mysql_quote(thread,thread->c.chan.name),mysql_quote(thread,param[6]));
							else if(streqnc(param[7],"single")) domysql(thread,"INSERT INTO `orion_bs_badwords` (`art`,`channel`,`badword`) VALUES ('1','%s','%s')",mysql_quote(thread,thread->c.chan.name),mysql_quote(thread,param[6]));
							else if(streqnc(param[7],"start")) domysql(thread,"INSERT INTO `orion_bs_badwords` (`art`,`channel`,`badword`) VALUES ('2','%s','%s')",mysql_quote(thread,thread->c.chan.name),mysql_quote(thread,param[6]));
							else if(streqnc(param[7],"end")) domysql(thread,"INSERT INTO `orion_bs_badwords` (`art`,`channel`,`badword`) VALUES ('3','%s','%s')",mysql_quote(thread,thread->c.chan.name),mysql_quote(thread,param[6]));
							send_lang(thread,language,howtosend,snick,nick,"BS_BADWORDS_ADDED_BADWORD_X_AS_X_FOR_CHANNEL_X",param[6],!param[7] ? "MATCH-ANY" : _uc(param[7]),thread->c.chan.name);
						}else send_lang(thread,language,howtosend,snick,nick,"BS_BADWORDS_BADWORD_X_FOR_CHANNEL_X_AREADY_IN",param[6],param[4]);
					}else send_lang(thread,language,howtosend,snick,nick,"BADWORDS_LIMIT_X_FOR_CHANNEL_X_REACHED",m->BSBadWordsMax,thread->c.chan.name);
				}else send_lang(thread,language,howtosend,snick,nick,"READONLY_MODE");
			}else send_lang(thread,language,howtosend,snick,nick,"NOT_LOGGED_IN_OR_NOT_ENOUGH_RIGHTS");
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief BS BADWORDS Del function for the BotServ BADWORDS Command.
 * 
 * The BotServ BADWORDS Del Command allows the user to delete a Badword for a Channel
 *  
 * Syntax is: /msg BotServ BADWORDS #channel DEL { badword | badword-number }
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL bs_badwords_del(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	//ChannelDB c;
	deepSQL selBadWords;
	
	/* Initialise Variables to (NULL) */
	//bzero(&c,sizeof(c));
	bzero(&selBadWords,sizeof(selBadWords));
	
	/*
	 * Badwords Del Command
	 */

	//:shock PRIVMSG BotServ :badwords #channel add ROFL [SINGLE | START | END]
	//:shock PRIVMSG BotServ :badwords #channel del [ROFL | 1]
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"badwords del")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		//c = sql_chan_db(thread,param[4]); //Channel c - STRUCT
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT))	send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(thread->c.chan.bot[0]=='\0')									send_lang(thread,language,howtosend,snick,nick,"BS_NO_BOT_ASSIGNED_FOR_CHANNEL_X",thread->c.chan.name);
		
		else if(streqnc(thread->c.chan.serv,"chanserv") || streqnc(thread->c.chan.serv,"q")){
			if(stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.badwords || strcor(thread->c.chan.access.ownlevel,"mno") || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"badwords del")){
				if(thread->c.chan.bot[0]!='\0' && CheckOption(thread,snick,nick,READONLY)){
					if(streqnc(mysql_sel_var(thread,"SELECT `badword` FROM `orion_bs_badwords` WHERE `channel`='%s' AND `badword` LIKE '%c%s%c'",mysql_quote(thread,thread->c.chan.name),'%',mysql_quote(thread,param[6]),'%'),param[6])){
						if(digitcol(param[6])){
							selBadWords.res = doSelectMySQL(thread,"SELECT `badword` FROM `orion_bs_badwords` WHERE `channel`='%s' ORDER BY `badword` ASC",mysql_quote(thread,thread->c.chan.name));
							domysql(thread,"DELETE FROM `orion_bs_badwords` WHERE `badword`='%s' AND `channel`='%s'",mysql_quote(thread,selBadWords.row[stoi(param[6])-1]),mysql_quote(thread,thread->c.chan.name));
							send_lang(thread,language,howtosend,snick,nick,"BS_BADWORDS_BADWORD_NR_X_DELETED",stoi(param[6]),thread->c.chan.name);

							mysql_free_result_myr(selBadWords.res);
						}else{
							domysql(thread,"DELETE FROM `orion_bs_badwords` WHERE `badword`='%s' AND `channel`='%s'",mysql_quote(thread,param[6]),mysql_quote(thread,thread->c.chan.name));
							send_lang(thread,language,howtosend,snick,nick,"BS_BADWORDS_BADWORD_X_DELETED",param[6],thread->c.chan.name);
						}
					}else send_lang(thread,language,howtosend,snick,nick,"BS_BADWORDS_NO_SUCH_BADWORD_X_FOR_CHANNEL_X",param[6],param[4]);
				}else send_lang(thread,language,howtosend,snick,nick,"READONLY_MODE");
			}else send_lang(thread,language,howtosend,snick,nick,"NOT_LOGGED_IN_OR_NOT_ENOUGH_RIGHTS");
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief BS BADWORDS List function for the BotServ BADWORDS Command.
 * 
 * The BotServ BADWORDS List Command allows the user to dump a List of Badwords
 *  
 * Syntax is: /msg BotServ BADWORDS #channel LIST [*pattern*]
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL bs_badwords_list(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	//ChannelDB c;
	deepSQL selBadWords;
	char pattern[BUFFER];
	int i=0;
	
	/* Initialise Variables to (NULL) */
	//bzero(&c,sizeof(c));
	bzero(&selBadWords,sizeof(selBadWords));
	bzero(&pattern,sizeof(pattern));
	
	/*
	 * Badwords List Command
	 */

	//:shock PRIVMSG BotServ :badwords #channel add ROFL [SINGLE | START | END]
	//:shock PRIVMSG BotServ :badwords #channel del [ROFL | 1]
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"badwords list")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		//c = sql_chan_db(thread,param[4]); //Channel c - STRUCT
		
		if(param[5] && strcor(param[5],"*")){
			dynncpy(pattern,param[5],sizeof(pattern));
			strrepc(pattern,'*','%');
		}else{
			pattern[0]='%';
			pattern[1]='\0';
		}
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT))	send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(thread->c.chan.bot[0]=='\0')									send_lang(thread,language,howtosend,snick,nick,"BS_NO_BOT_ASSIGNED_FOR_CHANNEL_X",thread->c.chan.name);
		
		else if(streqnc(thread->c.chan.serv,"chanserv") || streqnc(thread->c.chan.serv,"q")){
			if(stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.badwords || strcor(thread->c.chan.access.ownlevel,"mno") || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"badwords list")){
				i = 0;
				selBadWords.res    = doSelectMySQL(thread,"SELECT `badword`,`art` FROM `orion_bs_badwords` WHERE `channel`='%s' AND `badword` LIKE '%s' ORDER BY `badword` ASC",mysql_quote(thread,thread->c.chan.name),mysql_quote(thread,pattern));
				selBadWords.fields = mysql_num_rows(selBadWords.res);
				
				if(selBadWords.fields){
					send_lang(thread,language,howtosend,snick,nick,"BS_BADWORDS_LIST_HEADER",param[4]);
					for(i=0;i<selBadWords.fields;i++){
						selBadWords.row = mysql_fetch_row(selBadWords.res);
						
						if(stoi(selBadWords.row[1])==0) send_lang(thread,language,howtosend,snick,nick,"BS_BADWORDS_LIST",i+1,selBadWords.row[0],"MATCH-ANY");	
						if(stoi(selBadWords.row[1])==1) send_lang(thread,language,howtosend,snick,nick,"BS_BADWORDS_LIST",i+1,selBadWords.row[0],"SINGLE");
						else if(stoi(selBadWords.row[1])==2) send_lang(thread,language,howtosend,snick,nick,"BS_BADWORDS_LIST",i+1,selBadWords.row[0],"START");
						else if(stoi(selBadWords.row[1])==3) send_lang(thread,language,howtosend,snick,nick,"BS_BADWORDS_LIST",i+1,selBadWords.row[0],"END");
					}
					send_lang(thread,language,howtosend,snick,nick,"BS_BADWORDS_LIST_END",selBadWords.fields);
				}
				else send_lang(thread,language,howtosend,snick,nick,"BS_BADWORDS_LIST_NOENTRIES_END",param[4]);
				
				mysql_free_result_myr(selBadWords.res);
			}else send_lang(thread,language,howtosend,snick,nick,"NOT_LOGGED_IN_OR_NOT_ENOUGH_RIGHTS");
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief BS BADWORDS Clear function for the BotServ BADWORDS Command.
 * 
 * The BotServ BADWORDS Clear Command allows the user to clear all Badwords for a Channel
 *  
 * Syntax is: /msg BotServ BADWORDS #channel CLEAR
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL bs_badwords_clear(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	//ChannelDB c;
	
	/* Initialise Variables to (NULL) */
	//bzero(&c,sizeof(c));
	
	/*
	 * Badwords Clear Command
	 */

	//:shock PRIVMSG BotServ :badwords #channel add ROFL [SINGLE | START | END]
	//:shock PRIVMSG BotServ :badwords #channel del [ROFL | 1]
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"badwords clear")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		//c = sql_chan_db(thread,param[4]); //Channel c - STRUCT
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT))	send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(thread->c.chan.bot[0]=='\0')									send_lang(thread,language,howtosend,snick,nick,"BS_NO_BOT_ASSIGNED_FOR_CHANNEL_X",thread->c.chan.name);
		
		else if(streqnc(thread->c.chan.serv,"chanserv") || streqnc(thread->c.chan.serv,"q")){
			if(stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.badwords || strcor(thread->c.chan.access.ownlevel,"mno") || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"badwords clear")){
				if(CheckOption(thread,snick,nick,READONLY)){
					if(stoi(mysql_sel_var(thread,"SELECT count(*) FROM `orion_bs_badwords` WHERE `channel`='%s'",mysql_quote(thread,thread->c.chan.name)))){
						domysql(thread,"DELETE FROM `orion_bs_badwords` WHERE `channel`='%s'",mysql_quote(thread,param[6]),mysql_quote(thread,thread->c.chan.name));
						send_lang(thread,language,howtosend,snick,nick,"BS_BADWORDS_CLEARED_FOR_CHANNEL_X",param[4]);
					}else send_lang(thread,language,howtosend,snick,nick,"BS_BADWORDS_NO_BADWORDS_FOR_CHANNEL_X",param[4]);
				}else send_lang(thread,language,howtosend,snick,nick,"READONLY_MODE");
			}else send_lang(thread,language,howtosend,snick,nick,"NOT_LOGGED_IN_OR_NOT_ENOUGH_RIGHTS");
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the BotServ BADWORDS Command (/msg BotServ HELP BADWORDS)
 * 
 * When the user tries to get help for this command, using /msg BotServ HELP BADWORDS,<br/>
 * this function is executed.
 * 
 * Syntax: /msg BotServ HELP BADWORDS
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL bs_badwords_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Badwords Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help badwords"))
		send_lang(thread,language,howtosend,snick,nick,"BS_HELP_BADWORDS",snick);
	else
		send_lang(thread,language,howtosend,snick,nick,"BS_NO_HELP_AVAIL",snick);

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief BotServ Help List Command - BADWORDS part (/msg BotServ help)
 * 
 * When the user tries to get general BotServ help, using /msg BotServ help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the BADWORDS command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL bs_badwords_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_BS_Level=(thread->sql.aliases.active>0 ? thread->sql.access.rights : is_user);
	/*
	 * Badwords (Help Listing)
	 */
	if(Users_BS_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"BS_HELP_CMD_BADWORDS");

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
