/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 202                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-24 17:29:41 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file cs_dehalfop.c
 *   @brief This is the Module file for ChanServ DEHALFOP Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the ChanServ DEHALFOP command.
 *
 *   The ChanServ DEHALFOP Command allows the user to dehalfop ...<br/>
 *   1. Himself in all channels, he has the rights to take himself halfop (no parameters)<br/>
 *   2. Himself in a specific channel, if he has enough rights in that channel (the channel parameter)<br/>
 *   3. Another User in a specific channel, if he has enough rights to dehalfop target user in target channel (both parameters)
 *
 *   Syntax is: /msg ChanServ DEHALFOP [#channel] [nick]
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "services.h"
#include "global.h"

#define SERVICE "cs"
#define MODUL "dehalfop"
#define MODUL_FILE "cs_dehalfop"
#define MODUL_VERSION "1.0"

MOD_BOOL cs_dehalfop(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_dehalfop_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_dehalfop_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by <br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"dehalfop","dehalfop");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,2,LOGINONLY,OPER_IF_DEFCON);
	addAccess (thread,&Module,BOTH);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"dehalfop",cs_dehalfop,is_services_oper,is_services_user,3,5);
	addCommand(thread,&Module,"help dehalfop",cs_dehalfop_help,is_services_user,is_services_user,4,4);
	addCommand(thread,&Module,"help",cs_dehalfop_help_listing,is_services_user,is_services_user,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}


/*  ************************************************************************************************  */

/**
 * @brief Main function for the ChanServ DEHALFOP Command.
 * 
 * The ChanServ DEHALFOP Command allows the user to dehalfop ...<br/>
 *  1. Himself in all channels, he has the rights to take himself halfop (no parameters)<br/>
 *  2. Himself in a specific channel, if he has enough rights in that channel (the channel parameter)<br/>
 *  3. Another User in a specific channel, if he has enough rights to dehalfop target user in target channel (both parameters)
 *  
 * Syntax is: /msg ChanServ DEHALFOP [#channel] [nick]
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_dehalfop(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	Aliases aliases5;
	deepSQL dehalfop;
	int i = 0;

	/* Initialise Variables to (NULL) */
	bzero(&aliases5,sizeof(aliases5));
	bzero(&dehalfop,sizeof(dehalfop));

	/*
	 * Dehalfop Command
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"dehalfop")){
		if(param[4]){
			if(streqnc(param[5],nick) || !param[5]) // dehalfop myself in a certain channel
				dehalfop.res = doSelectMySQL(thread,"SELECT `level`,`orion_cs_core`.`cs_halfopme` AS `halfop`,`orion_cs_core`.`datecreate`,`orion_cs_core`.`botnick`,`orion_cs_core`.`bs_set` FROM `orion_cs_access` LEFT JOIN `orion_cs_core` ON `orion_cs_access`.`channel`=`orion_cs_core`.`channel` WHERE `orion_cs_access`.`serv`='ChanServ' AND `orion_cs_access`.`channel`='%s' AND `orion_cs_access`.`authnick`='%s' LIMIT 0,1",mysql_quote(thread,param[4]),mysql_quote(thread,thread->sql.aliases.authnick));
			else // dehalfop someone else voice in a certain channel
				dehalfop.res = doSelectMySQL(thread,"SELECT `level`,`orion_cs_core`.`cs_halfop`,`orion_cs_core`.`datecreate`,`orion_cs_core`.`settings`,`orion_cs_core`.`botnick`,`orion_cs_core`.`bs_set` FROM `orion_cs_access` LEFT JOIN `orion_cs_core` ON `orion_cs_access`.`channel`=`orion_cs_core`.`channel` WHERE `orion_cs_access`.`serv`='ChanServ' AND `orion_cs_access`.`channel`='%s' AND `orion_cs_access`.`authnick`='%s'  LIMIT 0,1",mysql_quote(thread,param[4]),mysql_quote(thread,thread->sql.aliases.authnick));

			dehalfop.fields = mysql_num_rows(dehalfop.res);
			if(dehalfop.fields){
				dehalfop.row = mysql_fetch_row(dehalfop.res);
				// dehalfop.row[0]: own chanaccess level, dehalfop.row[1]: halfopme/halfop level...
				if(stoi(dehalfop.row[0])>=stoi(dehalfop.row[1]) || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"dehalfop")){
					if(param[5] && !streqnc(param[5],nick)){
						aliases5 = sql_aliases(thread,param[5]);
						
						if(thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"dehalfop") || (~stol(dehalfop.row[3])&CS_PEACE_BIT || (stol(dehalfop.row[3])&CS_PEACE_BIT && stoi(dehalfop.row[0])>stoi(acclevel(thread,"ChanServ",param[4],aliases5.authnick))))){
							do_mode(thread,CSCheckBotInPlace(dehalfop.row[4],stol(dehalfop.row[5])),param[4],(time_t)stol(dehalfop.row[2]),"-h %s",param[5]);
							send_lang(thread,language,howtosend,snick,nick,"CS_COMMAND_DONE");
						}
						else send_lang(thread,language,howtosend,snick,nick,"CS_PERMISSION_DENIED");
					}else{
						do_mode(thread,CSCheckBotInPlace(dehalfop.row[3],stol(dehalfop.row[4])),param[4],(time_t)stol(dehalfop.row[2]),"-h %s",nick);
						send_lang(thread,language,howtosend,snick,nick,"CS_COMMAND_DONE");
					}
				}else send_lang(thread,language,howtosend,snick,nick,"ACCESS_IN_CHANNEL_LOW",param[4]);
			}
			mysql_free_result_myr(dehalfop.res);
		}else{
			dehalfop.res    = doSelectMySQL(thread,"SELECT `orion_cs_access`.`channel`,`orion_cs_access`.`level`,`orion_cs_core`.`cs_halfopme`,`orion_cs_core`.`datecreate`,`orion_cs_core`.`botnick`,`orion_cs_core`.`bs_set` FROM `orion_cs_access` LEFT JOIN (`orion_cs_core`,`orion_chan_users`) ON (`orion_cs_access`.`channel` = `orion_cs_core`.`channel` AND `orion_cs_access`.`channel`=`orion_chan_users`.`channel`) WHERE `orion_cs_access`.`authnick`='%s' AND `orion_chan_users`.`nick` ='%s' AND `orion_cs_access`.`serv`='ChanServ'",mysql_quote(thread,thread->sql.aliases.authnick),mysql_quote(thread,nick));
			dehalfop.fields = mysql_num_rows(dehalfop.res);
			
			for(i=0;i<dehalfop.fields;i++){
				dehalfop.row = mysql_fetch_row(dehalfop.res);
				// dehalfop.row[1]: own chanaccess level, dehalfop.row[2]: halfopme level
				if(stoi(dehalfop.row[1])>=stoi(dehalfop.row[2])) do_mode(thread,CSCheckBotInPlace(dehalfop.row[4],stol(dehalfop.row[5])),dehalfop.row[0],(time_t)stol(dehalfop.row[3]),"-h %s",nick);
			}
			send_lang(thread,language,howtosend,snick,nick,"CS_COMMAND_DONE");
			mysql_free_result_myr(dehalfop.res);
		}
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the ChanServ DEHALFOP Command (/msg ChanServ HELP DEHALFOP)
 * 
 * When the user tries to get help for this command, using /msg ChanServ HELP DEHALFOP,<br/>
 * this function is executed.
 * 
 * Syntax: /msg ChanServ HELP DEHALFOP
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_dehalfop_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Dehalfop Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help dehalfop"))
		send_lang(thread,language,howtosend,snick,nick,"CS_HELP_DEHALFOP");

	else send_lang(thread,language,howtosend,snick,nick,"CS_NO_HELP_AVAIL",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief ChanServ Help List Command - DEHALFOP part (/msg ChanServ help)
 * 
 * When the user tries to get general ChanServ help, using /msg ChanServ help, all loaded commands with <br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the DEHALFOP command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_dehalfop_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_CS_Level=(thread->sql.aliases.active>0 ? thread->sql.access.rights : is_user);
	/*
	 * Dehalfop (Help listing)
	 */
	if(Users_CS_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"CS_HELP_CMD_DEHALFOP");

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
