/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 280                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-02-15 17:29:45 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file o_idline.c
 *   @brief This is the Module file for O IDLINE Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the O IDLINE command.
 *
 *   The O IDLINE Command allows an admin to manage the Ident-Glines.<br/>
 *   (these Glines will be set normal for their realhosts,<br/>
 *    but they are completely processed and controlled by orion!)
 *
 *   Syntax is: /msg O IDLINE { ADD | DEL | LIST } [< OPTION >] [< ident[,ident] > [:< duration(i.e. 2d) > < reason >]]
 *
 *<table border="0">
 *    <tr><td> ADD	</td><td> Syntax is: /msg O IDLINE ADD	</td><td> { G | K }[,{ G | K }] ident[,ident] :< duration(i.e. 2d) > < reason >	</td></tr>
 *    <tr><td> DEL	</td><td> Syntax is: /msg O IDLINE DEL	</td><td> { G | K } ident														</td></tr>
 *    <tr><td> LIST	</td><td> Syntax is: /msg O IDLINE LIST	</td><td> [*ident*]																</td></tr>
 *</table>
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"

#define SERVICE "O"
#define MODUL "idline"
#define MODUL_FILE "o_idline"
#define MODUL_VERSION "1.0"

MOD_BOOL O_idline_add(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_idline_del(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_idline_list(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);

MOD_BOOL O_idline_add_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_idline_del_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_idline_list_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);

MOD_BOOL O_idline_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_idline_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"idline option[1]","idline helpoption[1]");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,4,LOGINONLY,OPER_IF_DEFCON,OPERONLY,READONLY);
	addAccess (thread,&Module,ADMIN);
	addMODType(thread,&Module,CORE);
	
	addLeastModuleLevels(thread,&Module,"idline",870,INFINIT); /* just to add a displaylevel for /msg o help
															  this should always be the lowest used Module level! */
	
	addCommand(thread,&Module,"idline add",O_idline_add,870,INFINIT,8,0);
	addCommand(thread,&Module,"idline del",O_idline_del,870,INFINIT,6,0);
	addCommand(thread,&Module,"idline list",O_idline_list,870,INFINIT,4,5);
	
	addCommand(thread,&Module,"help idline",O_idline_help,870,INFINIT,4,4);
	addCommand(thread,&Module,"help idline add",O_idline_add_help,870,INFINIT,5,5);
	addCommand(thread,&Module,"help idline del",O_idline_del_help,870,INFINIT,5,5);
	addCommand(thread,&Module,"help idline list",O_idline_list_help,870,INFINIT,5,5);
	
	addCommand(thread,&Module,"help",O_idline_help_listing,870,INFINIT,3,3); //100 - each Admin is able to see it.
	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}

/*  ************************************************************************************************  */

/**
 * @brief O IDLINE Add function for the O IDLINE Command.
 * 
 * The O IDLINE Add Command allows an admin to Add an Ident-Gline/Ident-Kill
 *  
 * Syntax is: /msg O IDLINE ADD { G | K }[,{ G | K }] ident[,ident] :< duration(i.e. 2d) > < reason >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_idline_add(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	deepSQL condata;
	int change=0, counted=0, last= 0;
	time_t expiretime=0;
	time_t date=0;
	size_t i = 0;
	char flag, reason1[500], reason[500];
	char *newparam[20], *reasons[ARRAY_SIZE];

	/* Initialise Variables to (NULL) */
	bzero(&condata,sizeof(condata));
	flag = '\0';
	bzero(&reason1,sizeof(reason1));
	bzero(&reason,sizeof(reason));
	
	bzero(newparam,sizeof(newparam));
	bzero(reasons,sizeof(reasons));

	/*
	 * Idline Add Command
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"idline add")){
		change=0;
		//:shock PRIVMSG O :idline gkkgkgkkg ident ident ident ident :expire reason
		for(i=6;param[i]!=NULL;i++){
			if(param[i][0]==':'){
				change=1;
				straddns(reason1,_dynadd("%s ",param[i]),sizeof(reason1));
			}
			else if(change!=1){
				newparam[i-6]=param[i];
				last++;
			}
			else if(change==1)
				straddns(reason1,_dynadd("%s ",param[i]),sizeof(reason1));

		}
		newparam[last+1]=NULL;
		
		if(dynlen(param[5])!=last){
			send_lang(thread,language,howtosend,snick,nick,"O_IDLINE_MORE_OR_LESS_FLAGS_THAN_EXPRESSIONS");
			return MOD_CONT;
		}
		//:3d reason
		
		substr(reason1,0);
		reason1[dynlen(reason1)-1]='\0'; // cut off last ' '
		
		bzero(reasons,sizeof(reasons));
		strsplit(reason1,reasons,' ');

		if(!check_expire(reasons[0])){
			send_lang(thread,language,howtosend,snick,nick,"INVALID_EXPIRESTRING",reasons[0]);
			return MOD_CONT;
		}
		
		expiretime=expire(reasons[0])+time(NULL);
		
		dynsplice(reasons,0,0);
		bzero(&reason,sizeof(reason));
		dynncpy(reason,_djoin(' ',reasons),sizeof(reason));

		date=time(NULL);
		if(stricor(param[5],"gk")){
			for(i=0;i<dynlen(param[5]) && newparam[i]!=NULL;i++){
				flag='\0';
				flag=param[5][i];
				counted=0;

				// nick!ident@realhost#ip=realname
				//(.*)!shock@(.*)

				if(lower(flag)=='g'){
					domysql(thread,"INSERT INTO `orion_core_glines` (`rng`,`line`,`gline`,`duration`,`addedby`,`reason`) VALUES ('3','G','(.*)!%s@(.*)','%ld','%s','%s') ON DUPLICATE KEY UPDATE `duration`=VALUES(`duration`),`addedby`=VALUES(`addedby`),`reason`=VALUES(`reason`)",mysql_quote(thread,newparam[i]),expiretime,mysql_quote(thread,nick),mysql_quote(thread,reason));
				}else if(lower(flag)=='k'){
					domysql(thread,"INSERT INTO `orion_core_glines` (`rng`,`line`,`gline`,`duration`,`addedby`,`reason`) VALUES ('4','G','(.*)!%s@(.*)','%ld','%s','%s') ON DUPLICATE KEY UPDATE `duration`=VALUES(`duration`),`addedby`=VALUES(`addedby`),`reason`=VALUES(`reason`)",mysql_quote(thread,newparam[i]),expiretime,mysql_quote(thread,nick),mysql_quote(thread,reason));
				}
				//printf("NEWPARAM[%d] = %s\n",i,newparam[i]);

				condata.res	   = doSelectMySQL(thread,"SELECT CONCAT('*@',`rhost`) AS `realhost`,`nick` FROM `orion_core_condata` WHERE `rident`='%s' AND `faketype`='0'",mysql_quote(thread,newparam[i]));
				condata.fields = mysql_num_rows(condata.res);

				if(condata.fields){
					for(i=0;i<condata.fields;i++){
						condata.row = mysql_fetch_row(condata.res);
						
						if(lower(flag)=='g' && condata.row[0]) do_gline(thread,m->servername,condata.row[0],_dynadd("%s!%s@%s",O->nick,O->ident,O->host),expiretime,date,reason);
						if(lower(flag)=='k' && condata.row[1]) do_kill(thread,snick,condata.row[1],reason);
					}
				}

				mysql_free_result_myr(condata.res); //free MySQL result
			}
			send_lang(thread,language,howtosend,snick,nick,"O_COMMAND_DONE");
		}else send_lang(thread,language,howtosend,snick,nick,"O_WRONG_PARAMETERS");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O IDLINE Del function for the O IDLINE Command.
 * 
 * The O IDLINE Del Command allows an admin to delete an Ident-Gline/Ident-Kill
 *  
 * Syntax is: /msg O IDLINE DEL { G | K }[{ G | K }..] < ident >[,< ident >]
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_idline_del(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	deepSQL rhost;
	char current, mode[50];
	size_t 	i    = 0;
	size_t  last = 0;

	/* Initialise Variables to (NULL) */
	bzero(&rhost,sizeof(rhost));
	bzero(&mode,sizeof(mode));

	/*
	 * Idline Del Command
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"idline del")){
		dynncpy(mode,param[5],sizeof(mode));
		
		while(param[last+6])
			last++;
		
		if(dynlen(param[5])!=last){
			send_lang(thread,language,howtosend,snick,nick,"O_IDLINE_MORE_OR_LESS_FLAGS_THAN_EXPRESSIONS");
			return MOD_CONT;
		}

		for(i=0;i<dynlen(mode);i++){
			current='\0';
			current=mode[i];
			if(lower(current)=='g'){
				domysql(thread,"DELETE FROM `orion_core_glines` WHERE `gline`='(.*)!%s@(.*)' AND `rng`='3'",mysql_quote(thread,param[i+6]));

				/*rhost.res    = doSelectMySQL(thread,"SELECT `rhost` FROM `orion_core_condata` WHERE `rident`='%s'",mysql_quote(thread,param[i+6]));
				rhost.fields = mysql_num_rows(rhost.res);

				if(rhost.fields){
					for(j=0;j<rhost.fields;j++){
						rhost.row = mysql_fetch_row(rhost.res);
						
						do_ungline(thread,m->servername,rhost.row[0],_dynadd("%s!%s@%s",O->nick,O->ident,O->host),time(NULL));
					}
				}

				mysql_free_result_myr(rhost.res); //free MySQL result*/
			}else if(lower(current)=='k') domysql(thread,"DELETE FROM `orion_core_glines` WHERE `gline`='(.*)!%s@(.*)' AND `rng`='4'",mysql_quote(thread,param[i+6]));
		}
		send_lang(thread,language,howtosend,snick,nick,"O_IDLINE_DEL_REMOVED_IDENTGLINE",_dstrjoin(' ',param,6));
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O IDLINE List function for the O IDLINE Command.
 * 
 * The O IDLINE List Command allows an admin to delete an Ident-Gline/Ident-Kill
 *  
 * Syntax is: /msg O IDLINE LIST [*ident*]
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_idline_list(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
    /* Global Variables */
    deepSQL MySQL;
	int i=0;
	char query[SQL_BUF];
	char time_return[150];
	char pattern[BUFSIZE];
	char ident[MAX_IDENT_LEN];
	time_t gettime;
	
	/* Initialise Variables to (NULL) */
	bzero(&MySQL,sizeof(MySQL));
	bzero(&query,sizeof(query));
	bzero(&time_return,sizeof(time_return));
	bzero(&pattern,sizeof(pattern));
	
	/*
	 * Idline List Command
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"idline list")){
		bzero(&query,sizeof(query));
		if(param[5] && strcor(param[5],"*")){
			dynncpy(pattern,param[5],sizeof(pattern));
			strrepc(pattern,'*','%');
			
			snprintf(query,sizeof(query),"SELECT `gline`,`duration`,`addedby`,`reason`,`rng` FROM `orion_core_glines` WHERE `rng` IN (3,4) AND `gline` LIKE '(.*)!%s@(.*)' ORDER BY `rng` ASC",mysql_quote(thread,pattern));
		}
		else
			snprintf(query,sizeof(query),"SELECT `gline`,`duration`,`addedby`,`reason`,`rng` FROM `orion_core_glines` WHERE `rng` IN (3,4) ORDER BY `rng` ASC");
		
		// in this case the query was built before using doSelectMySQL....
		MySQL.res    = doSelectMySQL(thread,query);
		MySQL.fields = mysql_num_rows(MySQL.res);
		
		if(MySQL.fields>0){
			for(i=0;i<MySQL.fields;i++){
				MySQL.row = mysql_fetch_row(MySQL.res);
				
				if(stoi(MySQL.row[1])==0) bzero(&time_return,sizeof(time_return));
				else{
					gettime=(time_t)stol(MySQL.row[1]);
					bzero(&time_return,sizeof(time_return));
					dynncpy(time_return,get_localtime(gettime),sizeof(time_return));
				}
			
				bzero(&ident,sizeof(ident));
				sscanf(MySQL.row[0], "(.*)!%[^@]", ident);
			
				if(i==0) send_lang(thread,language,howtosend,snick,nick,"O_IDLINE_LIST_HEADER_TIME_NOW",get_localtime(time(NULL)));
				if(stoi(MySQL.row[4])==3) send_lang(thread,language,howtosend,snick,nick,"O_IDLINE_LIST_LIST_IG_LINES",i+1,ident,MySQL.row[3],MySQL.row[2],time_return);
				if(stoi(MySQL.row[4])==4) send_lang(thread,language,howtosend,snick,nick,"O_IDLINE_LIST_LIST_IK_LINES",i+1,ident,MySQL.row[3],MySQL.row[2],time_return);
			}
		}
		
        mysql_free_result_myr(MySQL.res); //free MySQL result
		send_lang(thread,language,howtosend,snick,nick,"END_OF_LIST");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the O IDLINE Command (/msg O HELP IDLINE)
 * 
 * When the user tries to get help for this command, using /msg O HELP IDLINE,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP IDLINE
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_idline_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Idline Help
	 */ 
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help idline")) {
		send_lang(thread,language,howtosend,snick,nick,"O_HELP_IDLINE");
	}else send_lang(thread,language,howtosend,snick,nick,"O_UNKNOWN_COMMAND",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O IDLINE Add Help function for the O IDLINE Command (/msg O HELP IDLINE ADD)
 * 
 * When the user tries to get help for this command, using /msg O HELP IDLINE ADD,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP IDLINE ADD
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_idline_add_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Idline Add Help
	 */ 
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help idline add")) {
		send_lang(thread,language,howtosend,snick,nick,"REQUIRED_LEVEL",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"idline add"));
		send_lang(thread,language,howtosend,snick,nick,"O_HELP_IDLINE_ADD",snick,snick);
	}else send_lang(thread,language,howtosend,snick,nick,"O_UNKNOWN_COMMAND",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O IDLINE Del Help function for the O IDLINE Command (/msg O HELP IDLINE DEL)
 * 
 * When the user tries to get help for this command, using /msg O HELP IDLINE DEL,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP IDLINE DEL
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_idline_del_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Idline Del Help
	 */ 
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help idline del")) {
		send_lang(thread,language,howtosend,snick,nick,"REQUIRED_LEVEL",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"idline del"));
		send_lang(thread,language,howtosend,snick,nick,"O_HELP_IDLINE_DEL",snick);
	}else send_lang(thread,language,howtosend,snick,nick,"O_UNKNOWN_COMMAND",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O IDLINE List Help function for the O IDLINE Command (/msg O HELP IDLINE LIST)
 * 
 * When the user tries to get help for this command, using /msg O HELP IDLINE LIST,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP IDLINE LIST
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_idline_list_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Idline List Help
	 */ 
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help idline list")) {
		send_lang(thread,language,howtosend,snick,nick,"REQUIRED_LEVEL",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"idline list"));
		send_lang(thread,language,howtosend,snick,nick,"O_HELP_IDLINE_LIST");
	}else send_lang(thread,language,howtosend,snick,nick,"O_UNKNOWN_COMMAND",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief O Help List Command - IDLINE part (/msg O help)
 * 
 * When the user tries to get general O help, using /msg O help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the IDLINE command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_idline_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_O_Level=(thread->sql.aliases.active>0 ? thread->sql.access.olevel : is_user);
	/*
	 * Idline (Help listing)
	 */
	if(Users_O_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"O_SHOWCOMMANDS_IDLINE",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"idline"));

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
