/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 202                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-24 17:29:41 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file ms_check.c
 *   @brief This is the Module file for MemoServ CHECK Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the MemoServ CHECK command.
 *
 *   The MemoServ CHECK Command allows the user to check whether the _last_ memo he<br/>
 *   sent to < nick > has been read or not.<br/>
 *   NOTE: This is only working for nicks, not for channels.
 *
 *   Syntax is: /msg MemoServ CHECK < nick >
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "services.h"
#include "global.h"

#define SERVICE "ms"
#define MODUL "check"
#define MODUL_FILE "ms_check"
#define MODUL_VERSION "1.0"

MOD_BOOL ms_check(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ms_check_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ms_check_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"check","check");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,2,LOGINONLY,OPER_IF_DEFCON);
	addAccess (thread,&Module,BOTH);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"check",ms_check,is_services_admin,is_services_user,4,4);
	addCommand(thread,&Module,"help check",ms_check_help,is_services_admin,is_services_user,4,4);
	addCommand(thread,&Module,"help",ms_check_help_listing,is_services_admin,is_services_user,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}

//:shock PRIVMSG MemoServ :check shock
/*  ************************************************************************************************  */

/**
 * @brief Main function for the MemoServ CHECK Command.
 * 
 * The MemoServ CHECK Command allows the user to check whether the _last_ memo he<br/>
 *   sent to < nick > has been read or not.<br/>
 *   NOTE: This is only working for nicks, not for channels.
 *
 * Syntax is: /msg MemoServ CHECK < nick >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ms_check(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	deepSQL memos;
	Aliases aliases5;
	
	/* Initialise Variables to (NULL) */
	bzero(&memos,sizeof(memos));
	bzero(&aliases5,sizeof(aliases5));
	
	/*
	 * Check Command
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"check")){
		aliases5 = sql_aliases(thread,param[4]);
		
		if(aliases5.authnick[0]!='\0'){
			memos.res    = doSelectMySQL(thread,"SELECT `mode`,`time` FROM `orion_ms_core` WHERE `receiver`='%s' AND `sender`='%s' ORDER BY `time` DESC LIMIT 1",mysql_quote(thread,aliases5.authnick),mysql_quote(thread,thread->sql.aliases.authnick));
			memos.fields = mysql_num_rows(memos.res);
			if(memos.fields)
				memos.row    = mysql_fetch_row(memos.res);

			if(!memos.fields) send_lang(thread,language,howtosend,snick,nick,"MS_NO_MEMOS_FOR_TARGET",aliases5.authnick);
			else if(aliases5.authnick[0]!='\0' && memos.fields){
				if(stoi(memos.row[0])) send_lang(thread,language,howtosend,snick,nick,"MS_CHECK_LAST_MEMO_READ",aliases5.authnick);
				else send_lang(thread,language,howtosend,snick,nick,"MS_CHECK_LAST_MEMO_NOT_READ",aliases5.authnick);
			}else send_lang(thread,language,howtosend,snick,nick,"MEMO_LESS_0");
			
    		/* mysql_free_result_myr(); - memos */
    		mysql_free_result_myr(memos.res);
        }else send_lang(thread,language,howtosend,snick,nick,"NO_AUTHNICK");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the MemoServ CHECK Command (/msg MemoServ HELP CHECK)
 * 
 * When the user tries to get help for this command, using /msg MemoServ HELP CHECK,<br/>
 * this function is executed.
 * 
 * Syntax: /msg MemoServ HELP CHECK
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ms_check_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Check Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help check"))
		send_lang(thread,language,howtosend,snick,nick,"MS_HELP_CHECK");

	else send_lang(thread,language,howtosend,snick,nick,"MS_NO_HELP_AVAIL",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief MemoServ Help List Command - CHECK part (/msg MemoServ help)
 * 
 * When the user tries to get general MemoServ help, using /msg MemoServ help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the CHECK command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ms_check_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_MS_Level=(thread->sql.aliases.active>0 ? thread->sql.access.rights : is_user);
	/*
	 * Check (Help listing)
	 */
	if(Users_MS_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"MS_HELP_CMD_CHECK");

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
