/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 202                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-24 17:29:41 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file ms_info.c
 *   @brief This is the Module file for MemoServ INFO Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the MemoServ INFO command.
 *
 *   The MemoServ INFO Command allows the user to...<br/>
 *   1. Display information on the number of he/she has, how many of them are unread,<br/>
 *      and how many total memos he/she can receive. (No Parameter)<br/>
 *   2. Display the same information for a given channel. (The Channel Parameter)<br/>
 *   3. Displays the same information for the given nickname.<br/>
 *      (This use limited to Services Admins) - (The Nick Parameter)
 *
 *   Syntax is: /msg MemoServ INFO < nick / #channel >
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "services.h"
#include "global.h"

#define SERVICE "ms"
#define MODUL "info"
#define MODUL_FILE "ms_info"
#define MODUL_VERSION "1.0"

MOD_BOOL ms_info(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ms_info_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ms_info_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"info","info");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,2,LOGINONLY,OPER_IF_DEFCON);
	addAccess (thread,&Module,BOTH);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"info",ms_info,is_services_oper,is_services_user,3,4);
	addCommand(thread,&Module,"help info",ms_info_help,is_services_oper,is_services_user,4,4);
	addCommand(thread,&Module,"help",ms_info_help_listing,is_services_oper,is_services_user,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}

//:shock PRIVMSG MemoServ :cancel shock
/*  ************************************************************************************************  */

/**
 * @brief Main function for the MemoServ INFO Command.
 * 
 * The MemoServ INFO Command allows the user to...<br/>
 *   1. Display information on the number of he/she has, how many of them are unread,<br/>
 *      and how many total memos he/she can receive. (No Parameter)<br/>
 *   2. Display the same information for a given channel. (The Channel Parameter)<br/>
 *   3. Displays the same information for the given nickname.<br/>
 *      (This use limited to Services Admins) - (The Nick Parameter)
 *
 * Syntax is: /msg MemoServ INFO < nick / #channel >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ms_info(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	ChannelDB c;
	Aliases aliases5;
	int memos = 0;
	int unread = 0;
	
	/* Initialise Variables to (NULL) */
	bzero(&c,sizeof(c));
	bzero(&aliases5,sizeof(aliases5));
	
	/*
	 * Info Command
	 */
	 //:shock PRIVMSG MemoServ :info
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"info")){
		if(!param[4]){
			memos  = stoi(mysql_sel_var(thread,"SELECT count(*) FROM `orion_ms_core` WHERE `receiver`='%s'",mysql_quote(thread,thread->sql.aliases.authnick)));
			unread = stoi(mysql_sel_var(thread,"SELECT count(*) FROM `orion_ms_core` WHERE `receiver`='%s' AND `mode`='0'",mysql_quote(thread,thread->sql.aliases.authnick)));
  
			if(memos) //memos
                send_lang(thread,language,howtosend,snick,nick,"MS_INFO_ME_INFO",memos,unread,thread->sql.access.maxmemos);
			else
                send_lang(thread,language,howtosend,snick,nick,"MS_INFO_ME_NO_MEMOS",thread->sql.access.maxmemos);
        }else if(param[4][0]=='#'){ //Channel
			c     = sql_chan_db(thread,param[4]);
			memos = stoi(mysql_sel_var(thread,"SELECT count(*) FROM `orion_ms_core` WHERE `receiver`='%s'",mysql_quote(thread,c.name)));
  
			if(c.serv[0]=='\0') send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED");
			else if(thread->sql.access.rights>ModuleFile_RootCommandLevel(thread,MODUL_FILE,"info") || strcor(acclevel(thread,"Q",c.name,thread->sql.aliases.authnick),"o") || strcor(acclevel(thread,"L",c.name,thread->sql.aliases.authnick),"o") || stoi(acclevel(thread,"ChanServ",c.name,thread->sql.aliases.authnick))>=c.cs_memo){
				if(memos)
					send_lang(thread,language,howtosend,snick,nick,"MS_INFO_CHANNEL_INFO",c.name,memos,c.name,c.maxmemos);
				else
					send_lang(thread,language,howtosend,snick,nick,"MS_INFO_CHANNEL_NO_MEMOS",c.name,c.name,c.maxmemos);
			}else send_lang(thread,language,howtosend,snick,nick,"ACCESS_IN_CHANNEL_LOW",c.name);
		}else if(param[4] && thread->sql.access.rights>ModuleFile_RootCommandLevel(thread,MODUL_FILE,"info")){
			aliases5 = sql_aliases(thread,param[4]);
			memos    = stoi(mysql_sel_var(thread,"SELECT count(*) FROM `orion_ms_core` WHERE `receiver`='%s'",mysql_quote(thread,aliases5.authnick)));
  
			if(aliases5.authnick[0]=='\0') send_lang(thread,language,howtosend,snick,nick,"NO_AUTHNICK");
			else if(memos) //memos
                send_lang(thread,language,howtosend,snick,nick,"MS_INFO_NICK_INFO",aliases5.authnick,memos,aliases5.authnick,stoi(accessselect(thread,"maxmemos",aliases5.authnick)));
			else
                send_lang(thread,language,howtosend,snick,nick,"MS_INFO_NICK_NO_MEMOS",aliases5.authnick,aliases5.authnick,stoi(accessselect(thread,"maxmemos",aliases5.authnick)));
        }
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the MemoServ INFO Command (/msg MemoServ HELP INFO)
 * 
 * When the user tries to get help for this command, using /msg MemoServ HELP INFO,<br/>
 * this function is executed.
 * 
 * Syntax: /msg MemoServ HELP INFO
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ms_info_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Info Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help info"))
		send_lang(thread,language,howtosend,snick,nick,"MS_HELP_INFO");

	else send_lang(thread,language,howtosend,snick,nick,"MS_NO_HELP_AVAIL",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief MemoServ Help List Command - INFO part (/msg MemoServ help)
 * 
 * When the user tries to get general MemoServ help, using /msg MemoServ help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the INFO command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ms_info_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_MS_Level=(thread->sql.aliases.active>0 ? thread->sql.access.rights : is_user);
	/*
	 * Info (Help listing)
	 */
	if(Users_MS_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"MS_HELP_CMD_INFO");

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
