/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 202                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-24 17:29:41 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file ms_read.c
 *   @brief This is the Module file for MemoServ READ Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the MemoServ READ command.
 *
 *   The MemoServ READ Command allows the user to read his memos<br/>
 *   or one memo of target channel he has memo-manage access to.
 *
 *   Syntax is: /msg MemoServ READ < nick / #channel > { num | LAST | NEW }
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "services.h"
#include "global.h"

#define LOW 0
#define NORMAL 1
#define HIGH 2

#define SERVICE "ms"
#define MODUL "read"
#define MODUL_FILE "ms_read"
#define MODUL_FILE_MS_SEND "ms_send" //used to reply memos on read if rsend was used
#define MODUL_VERSION "1.0"

MOD_BOOL ms_read(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ms_read_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ms_read_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"read","read");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,2,LOGINONLY,OPER_IF_DEFCON);
	addAccess (thread,&Module,BOTH);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"read",ms_read,is_services_oper,is_services_user,4,5);
	addCommand(thread,&Module,"help read",ms_read_help,is_services_oper,is_services_user,4,4);
	addCommand(thread,&Module,"help",ms_read_help_listing,is_services_oper,is_services_user,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}

/*  ************************************************************************************************  */

/**
 * @brief Main function for the MemoServ READ Command.
 * 
 * The MemoServ READ Command allows the user to read his memos<br/>
 *   or one memo of target channel he has memo-manage access to.
 *
 * Syntax is: /msg MemoServ READ < nick / #channel > { num | LAST | NEW }
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ms_read(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	ChannelDB c;
	deepSQL memo;
	deepSQL reply_memo;
	int memonr = 0;
	char *acclevel = NULL;
	char From[80];
	char Subject[80];

	/* Initialise Variables to (NULL) */
	bzero(&c,sizeof(c));
	bzero(&memo,sizeof(memo));
	bzero(&reply_memo,sizeof(reply_memo));
	bzero(&From,sizeof(From));
	bzero(&Subject,sizeof(Subject));
	
	/*
	 * Read Command
	 */
	// :shock PRIVMSG memoserv :read [channel] memonr
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"read")){
		if(param[4][0]=='#' && param[5]){
			
			if(streqnc(param[5],"last")){
				
				memo.res    = doSelectMySQL(thread,"SELECT `memo`,`sender`,`rsend`,`priority`,`time`,`receiver` FROM `orion_ms_core` WHERE `receiver`='%s' ORDER BY `time` ASC",mysql_quote(thread,param[4]));
				memonr      = mysql_num_rows(memo.res);
				
				mysql_free_result_myr(memo.res);
			}
			else if(digitcol(param[5]))
				memonr = stoi(param[5]);
				
			if(memonr<=0){
				send_lang(thread,language,howtosend,snick,nick,"MS_NO_MEMOS_FOR_TARGET",param[4]);
				return MOD_CONT;
			}
				
			c      = sql_chan_db(thread,param[4]);
			
			if(c.serv[0]!='\0'){ // Channel memo...
				acclevel = mysql_sel_var(thread,"SELECT `level` FROM `orion_cs_access` WHERE `authnick`='%s' AND `channel`='%s' LIMIT 0,1",mysql_quote(thread,thread->sql.aliases.authnick),mysql_quote(thread,c.name));
				
				if((digitcol(acclevel) && stoi(acclevel)>=c.cs_memo) || strcor(acclevel,"o") || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"read")){
					memo.res = doSelectMySQL(thread,"SELECT `memo`,`sender`,`rsend`,`priority`,`time` FROM `orion_ms_core` WHERE `receiver`='%s' ORDER BY `time` ASC LIMIT %d,1",mysql_quote(thread,c.name),(memonr-1));
					if(mysql_num_rows(memo.res)>0){
						memo.row = mysql_fetch_row(memo.res);
						
						send_lang(thread,language,howtosend,snick,thread->sql.aliases.nick,"MS_READ_CHANNEL_MEMO",memonr,memo.row[1],get_localtime((time_t)stol(memo.row[4])),snick,c.name,memonr,memo.row[0]);
						domysql(thread,"UPDATE `orion_ms_core` SET `mode`='1' WHERE `time`='%ld' AND `receiver`='%s' LIMIT 1",(time_t)stol(memo.row[4]),mysql_quote(thread,c.name));
						if(stoi(memo.row[2])){  // sends confirmation to the sender
							reply_memo.res = doSelectMySQL(thread,"SELECT `language`,`settings`,`maxmemos`,`email`,COUNT(`orion_ms_core`.`receiver`) AS `count_memos`,`orion_ns_aliases`.`nick`,`orion_ns_aliases`.`active` FROM `orion_ns_core` LEFT OUTER JOIN `orion_ms_core` ON `orion_ns_core`.`authnick`=`orion_ms_core`.`receiver` LEFT OUTER JOIN `orion_ns_aliases` ON `orion_ns_core`.`authnick`=`orion_ns_aliases`.`display` WHERE `orion_ns_core`.`authnick`='%s' GROUP BY `orion_ms_core`.`receiver`",memo.row[1]);
							if(mysql_num_rows(reply_memo.res)>0){
								reply_memo.row = mysql_fetch_row(reply_memo.res);
								
								if(reply_memo.row[5]!=NULL && (stoi(reply_memo.row[4])<stoi(reply_memo.row[2]) || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE_MS_SEND,"send"))){
									domysql(thread,"INSERT INTO `orion_ms_core` (`sender`,`receiver`,`priority`,`rsend`,`mode`,`time`,`memo`) VALUES ('%s','%s','%d','0','0','%ld','%s')",mysql_quote(thread,thread->sql.aliases.authnick),mysql_quote(thread,memo.row[1]),stoi(memo.row[3]),time(NULL),mysql_quote(thread,_dynadd("MEMO REPLY - receipent: %s (%s) has read your memo: %s",c.name,thread->sql.aliases.authnick,memo.row[0])));
									if(stoi(reply_memo.row[6])==1 && (stoi(memo.row[3])==LOW || !m->MSUsePriorities))
										send_lang(thread,languages[stoi(reply_memo.row[0])],stoi(reply_memo.row[1]) & NS_SET_MSG ? 1 : 0,snick,memo.row[1],"MS_SEND_RECEIVED_NEW_MEMO_FROM_X",thread->sql.aliases.authnick,snick,(stoi(reply_memo.row[4])+1),stoi(memo.row[3]),snick);

									else if(m->MSUsePriorities){
										if(stoi(memo.row[3])==NORMAL){
											if(stoi(reply_memo.row[6])==1)
												send_lang(thread,languages[stoi(reply_memo.row[0])],1,snick,memo.row[1],"MS_SEND_RECEIVED_NEW_MEMO_FROM_X",thread->sql.aliases.authnick,snick,(stoi(reply_memo.row[4])+1),stoi(memo.row[3]),snick);
										}
										else if(stoi(memo.row[3])==HIGH){
											if(stoi(reply_memo.row[6])==1)
												send_lang(thread,languages[stoi(reply_memo.row[0])],1,snick,memo.row[1],"MS_SEND_RECEIVED_NEW_MEMO_FROM_X",thread->sql.aliases.authnick,snick,(stoi(reply_memo.row[4])+1),stoi(memo.row[3]),snick);
											
											if (reply_memo.row[3][0]!='\0' && m->UseMail){
												//bzero(&From,sizeof(From));
												//bzero(&Subject,sizeof(Subject));
												
												dynncpy(From,_dynadd("%s Network <%s>",m->SendFrom),sizeof(From));
												dynncpy(Subject,_dynadd("New Memo for Group %s",memo.row[1]),sizeof(Subject));
												
												MailInfo *mail = o_email(thread,From,"",c.email,Subject);
												//Email					
												fprintf(mail->pipe,"There is a new memo for Group: %s\n"
												"Sent by MainNick / Nick: %s / %s\n"
												"Memo Nr.: %d\n"
												"Issue: MEMO REPLY - receipent: %s has read your memo: %s\n",
												memo.row[1],thread->sql.aliases.authnick,nick,(stoi(reply_memo.row[4])+1),memo.row[1],memo.row[0]);
												c_email(thread,mail);
											}
										}
									}
								}
							}
							mysql_free_result_myr(reply_memo.res);
						}
					}else send_lang(thread,language,howtosend,snick,nick,"MEMO_UNKNOWN");
					mysql_free_result_myr(memo.res);
				}else send_lang(thread,language,howtosend,snick,thread->sql.aliases.authnick,"ACCESS_IN_CHANNEL_LOW",c.name);
			}else send_lang(thread,language,howtosend,snick,thread->sql.aliases.authnick,"CHANNEL_NOT_REGISTERED");
		}else if(param[4] && !param[5]){ // normal memo to a user
			if(streqnc(param[4],"last")){
				
				memo.res    = doSelectMySQL(thread,"SELECT `memo`,`sender`,`rsend`,`priority`,`time`,`receiver` FROM `orion_ms_core` WHERE `receiver`='%s' ORDER BY `time` ASC",mysql_quote(thread,thread->sql.aliases.authnick));
				memonr      = mysql_num_rows(memo.res);
				
				mysql_free_result_myr(memo.res);
			}
			else if(digitcol(param[4]))
				memonr = stoi(param[4]);
				
			if(memonr<=0){
				send_lang(thread,language,howtosend,snick,nick,"MEMO_NO_MEMOS");
				return MOD_CONT;
			}
		
			memo.res = doSelectMySQL(thread,"SELECT `memo`,`sender`,`rsend`,`priority`,`time` FROM `orion_ms_core` WHERE `receiver`='%s' ORDER BY `time` ASC LIMIT %d,1",mysql_quote(thread,thread->sql.aliases.authnick),(memonr-1));
			if(mysql_num_rows(memo.res)>0){
				memo.row = mysql_fetch_row(memo.res);
				
				send_lang(thread,language,howtosend,snick,thread->sql.aliases.nick,"MS_READ_USER_MEMO",memonr,memo.row[1],get_localtime((time_t)stol(memo.row[4])),snick,memonr,memo.row[0]);
				domysql(thread,"UPDATE `orion_ms_core` SET `mode`='1' WHERE `time`='%ld' AND `receiver`='%s'",(time_t)stol(memo.row[4]),mysql_quote(thread,thread->sql.aliases.authnick));
				
				if(stoi(memo.row[2])){ // sends confirmation to the sender
					reply_memo.res = doSelectMySQL(thread,"SELECT `language`,`settings`,`maxmemos`,`email`,COUNT(`orion_ms_core`.`receiver`) AS `count_memos`,`orion_ns_aliases`.`nick`,`orion_ns_aliases`.`active` FROM `orion_ns_core` LEFT OUTER JOIN `orion_ms_core` ON `orion_ns_core`.`authnick`=`orion_ms_core`.`receiver` LEFT OUTER JOIN `orion_ns_aliases` ON `orion_ns_core`.`authnick`=`orion_ns_aliases`.`display` WHERE `orion_ns_core`.`authnick`='%s' GROUP BY `orion_ms_core`.`receiver`",memo.row[1]);
					if(mysql_num_rows(reply_memo.res)>0){
						reply_memo.row = mysql_fetch_row(reply_memo.res);
						if(reply_memo.row[5]!=NULL && (stoi(reply_memo.row[4])<stoi(reply_memo.row[2]) || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE_MS_SEND,"send"))){
							domysql(thread,"INSERT INTO `orion_ms_core` (`sender`,`receiver`,`priority`,`rsend`,`mode`,`time`,`memo`) VALUES ('%s','%s','%d','0','0','%ld','%s')",mysql_quote(thread,thread->sql.aliases.authnick),mysql_quote(thread,memo.row[1]),stoi(memo.row[3]),time(NULL),mysql_quote(thread,_dynadd("MEMO REPLY - receipent: %s has read your memo: %s",thread->sql.aliases.authnick,memo.row[0])));
							
							if(stoi(reply_memo.row[6])==1 && (stoi(memo.row[3])==LOW || !m->MSUsePriorities))
								send_lang(thread,languages[stoi(reply_memo.row[0])],stoi(reply_memo.row[1]) & NS_SET_MSG ? 1 : 0,snick,reply_memo.row[5],"MS_SEND_RECEIVED_NEW_MEMO_FROM_X",thread->sql.aliases.authnick,snick,(stoi(reply_memo.row[4])+1),stoi(memo.row[3]),snick);
							
							else if(m->MSUsePriorities){
								if(stoi(memo.row[3])==NORMAL){
									if(stoi(reply_memo.row[6])==1)
										send_lang(thread,languages[stoi(reply_memo.row[0])],1,snick,reply_memo.row[5],"MS_SEND_RECEIVED_NEW_MEMO_FROM_X",thread->sql.aliases.authnick,snick,(stoi(reply_memo.row[4])+1),stoi(memo.row[3]),snick);
								
								}else if(stoi(memo.row[3])==HIGH){
									if(stoi(reply_memo.row[6])==1)
										send_lang(thread,languages[stoi(reply_memo.row[0])],1,snick,reply_memo.row[5],"MS_SEND_RECEIVED_NEW_MEMO_FROM_X",thread->sql.aliases.authnick,snick,(stoi(reply_memo.row[4])+1),stoi(memo.row[3]),snick);
									
									if(reply_memo.row[3][0]!='\0' && m->UseMail){
										//bzero(&From,sizeof(From));
										//bzero(&Subject,sizeof(Subject));
										
										dynncpy(From,_dynadd("%s Network <%s>",m->SendFrom),sizeof(From));
										dynncpy(Subject,_dynadd("New Memo for Group %s",memo.row[1]),sizeof(Subject));
										
										MailInfo *mail = o_email(thread,From,"",c.email,Subject);
										//Email					
										fprintf(mail->pipe,"There is a new memo for Group: %s\n"
										"Sent by MainNick / Nick: %s / %s\n"
										"Memo Nr.: %d\n"
										"Issue: MEMO REPLY - receipent: %s has read your memo: %s\n",
										memo.row[1],thread->sql.aliases.authnick,nick,(stoi(reply_memo.row[4])+1),memo.row[1],memo.row[0]);
										c_email(thread,mail);
									}
								}
							}
						}
					}
					mysql_free_result_myr(reply_memo.res);
				}
			}else send_lang(thread,language,howtosend,snick,nick,"MEMO_NO_MEMOS");
			mysql_free_result_myr(memo.res);
		}
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the MemoServ READ Command (/msg MemoServ HELP READ)
 * 
 * When the user tries to get help for this command, using /msg MemoServ HELP READ,<br/>
 * this function is executed.
 * 
 * Syntax: /msg MemoServ HELP READ
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ms_read_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Read Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help read"))
		send_lang(thread,language,howtosend,snick,nick,"MS_HELP_READ");

	else send_lang(thread,language,howtosend,snick,nick,"MS_NO_HELP_AVAIL",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief MemoServ Help List Command - READ part (/msg MemoServ help)
 * 
 * When the user tries to get general MemoServ help, using /msg MemoServ help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the READ command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ms_read_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_MS_Level=(thread->sql.aliases.active>0 ? thread->sql.access.rights : is_user);
	/*
	 * Read (Help listing)
	 */
	if(Users_MS_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"MS_HELP_CMD_READ");

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
