/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 202                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-24 17:29:41 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file ns_glist.c
 *   @brief This is the Module file for NickServ GLIST Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the NickServ GLIST command.
 *
 *   The NickServ GLIST Command allows the user to dump all nicknames<br/>
 *   in his group or all nicks in the group of [nick].
 *
 *   Syntax is: /msg NickServ GLIST [nick]
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "services.h"
#include "global.h"

#define SERVICE "ns"
#define MODUL "glist"
#define MODUL_FILE "ns_glist"
#define MODUL_VERSION "1.0"

MOD_BOOL ns_glist(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_glist_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_glist_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"glist","glist");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,2,LOGINONLY,OPER_IF_DEFCON);
	addAccess (thread,&Module,BOTH);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"glist",ns_glist,is_services_admin,is_services_user,3,4);
	addCommand(thread,&Module,"help glist",ns_glist_help,is_services_user,is_services_user,4,4);
	addCommand(thread,&Module,"help",ns_glist_help_listing,is_services_user,is_services_user,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}


/*  ************************************************************************************************  */

/**
 * @brief Main function for the NickServ GLIST Command.
 * 
 * The NickServ GLIST Command allows the user to dump all nicknames<br/>
 *  in his group or all nicks in the group of [nick].
 *
 * Syntax is: /msg NickServ GLIST [nick]
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_glist(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
    deepSQL groups;
    char authnick[MAX_NICK_LEN];
    int i = 0;
    
	/* Initialise Variables to (NULL) */
	bzero(&groups,sizeof(groups));
	bzero(&authnick,sizeof(authnick));
	
	/*
	 * Glist Command
	 */
	if(param[4] && thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"glist")){
		dynncpy(authnick,mysql_sel_var(thread,"SELECT `display` FROM `orion_ns_aliases` WHERE `nick`='%s'",mysql_quote(thread,param[4])),sizeof(authnick));
		if(authnick[0]!='\0'){
			groups.res    = doSelectMySQL(thread,"SELECT `nick` FROM `orion_ns_aliases` WHERE `display`='%s'",mysql_quote(thread,authnick));
			groups.fields = mysql_num_rows(groups.res);
			
			send_lang(thread,language,howtosend,snick,nick,"NS_GLIST_LISTHEADER",authnick);
			
			for(i=0;i<groups.fields;i++){
				groups.row = mysql_fetch_row(groups.res);
				
				send_lang(thread,language,howtosend,snick,nick,"NS_GLIST_LIST",i+1,groups.row[0]);
			}
			
	   		/* mysql_free_result_myr(); - groups */
	   		mysql_free_result_myr(groups.res);
			send_lang(thread,language,howtosend,snick,nick,"NS_GLIST_END_HOW_MANY",groups.fields);
		}else send_lang(thread,language,howtosend,snick,nick,"NICK_NOT_REGISTERED",param[4]);
	}else if(!param[4] && thread->sql.access.rights>=ModuleFile_UserCommandLevel(thread,MODUL_FILE,"glist")){
		groups.res    = doSelectMySQL(thread,"SELECT `nick` FROM `orion_ns_aliases` WHERE `display`='%s'",mysql_quote(thread,thread->sql.aliases.authnick));
		groups.fields = mysql_num_rows(groups.res);
		
		if(groups.fields){
			//send_lang(thread,language,howtosend,snick,nick,"NS_GLIST_LISTHEADER",thread->sql.aliases.authnick);
			send_lang(thread,language,howtosend,snick,nick,"NS_GLIST_LISTHEADER_MINE");
			
			for(i=0;i<groups.fields;i++){
				groups.row = mysql_fetch_row(groups.res);
				
				send_lang(thread,language,howtosend,snick,nick,"NS_GLIST_LIST",i+1,groups.row[0]);
			}
			
			send_lang(thread,language,howtosend,snick,nick,"NS_GLIST_END_HOW_MANY",groups.fields);
		}else send_lang(thread,language,howtosend,snick,nick,"NICK_NOT_REGISTERED",nick);
		
	   	/* mysql_free_result_myr(); - groups */
	   	mysql_free_result_myr(groups.res);
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NOT_LOGGED_IN_OR_NOT_ENOUGH_RIGHTS");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the NickServ GLIST Command (/msg NickServ HELP GLIST)
 * 
 * When the user tries to get help for this command, using /msg NickServ HELP GLIST,<br/>
 * this function is executed.
 * 
 * Syntax: /msg NickServ HELP GLIST
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_glist_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Glist Help
	 */
	if(thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"help glist"))
		send_lang(thread,language,howtosend,snick,nick,"NS_SERVADMIN_HELP_GLIST");
	else if(thread->sql.access.rights>=ModuleFile_UserCommandLevel(thread,MODUL_FILE,"help glist"))
		send_lang(thread,language,howtosend,snick,nick,"NS_HELP_GLIST");

	else send_lang(thread,language,howtosend,snick,nick,"NS_NO_HELP_AVAIL",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief NickServ Help List Command - GLIST part (/msg NickServ help)
 * 
 * When the user tries to get general NickServ help, using /msg NickServ help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the GLIST command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_glist_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_NS_Level=(thread->sql.aliases.active>0 ? thread->sql.access.rights : is_user);
 	/*
	 * Glist (Help listing)
	 */
	if(Users_NS_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"NS_HELP_CMD_GLIST");

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
