/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 202                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-24 17:29:41 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file ns_info.c
 *   @brief This is the Module file for NickServ INFO Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the NickServ INFO command.
 *
 *   The NickServ INFO Command allows the user to dump some<br/>
 *   information about target user. (last usermask, last seen, ... )<br/>
 *   If no nick was given, some Information about you will be returned.
 *
 *   Syntax is: /msg NickServ INFO < nick >
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "services.h"
#include "global.h"

#define SERVICE "ns"
#define MODUL "info"
#define MODUL_FILE "ns_info"
#define MODUL_VERSION "1.0"

MOD_BOOL ns_info(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_info_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_info_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"info","info");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,2,LOGINONLY,OPER_IF_DEFCON);
	addAccess (thread,&Module,BOTH);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"info",ns_info,is_services_admin,is_services_user,4,4);
	addCommand(thread,&Module,"help info",ns_info_help,is_services_admin,is_services_user,4,4);
	addCommand(thread,&Module,"help",ns_info_help_listing,is_services_user,is_services_user,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}


/*  ************************************************************************************************  */

/**
 * @brief Main function for the NickServ INFO Command.
 * 
 * The NickServ INFO Command allows the user to dump some<br/>
 *   information about target user. (last usermask, last seen, ... )<br/>
 *   If no nick was given, some Information about you will be returned.
 *
 * Syntax is: /msg NickServ INFO < nick >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_info(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	deepSQL chanaccess;
    Aliases aliases5;
    Access  access5;
    char options[150];
	char escaped[MAX_NICK_LEN*2];
	int i          = 0;
	int old_fields = 0;

	/* Initialise Variables to (NULL) */
	bzero(&chanaccess,sizeof(chanaccess));
	bzero(&aliases5,sizeof(aliases5));
	bzero(&access5,sizeof(access5));
	bzero(&options,sizeof(options));
	bzero(&escaped,sizeof(escaped));

	/*
	 * Info Command
	 */
 	aliases5=sql_aliases(thread,param[4]); //thread->sql.aliases.authnick (Q-authnick - referencenick)
 	if(aliases5.fields) access5=sql_access(thread,aliases5.authnick); //Data about the referencenick
 	/*
 	(23:39) -NickServ- NetSpLit is tuka
    (23:39) -NickServ- NetSpLit is a services root administrator.
    (23:39) -NickServ- Is online from: NetSpLit@netadmin.deepirc.net
    (23:39) -NickServ- Time registered: Jan 28 10:35:40 2003 CET
    (23:39) -NickServ- Last quit message: Ping timeout
    (23:39) -NickServ- URL: http://www.deepirc.net
    (23:39) -NickServ- E-mail address: netsplit@deepirc.net
    (23:39) -NickServ- ICQ #: 272057995
    (23:39) -NickServ- Greet message: .
    (23:39) -NickServ- Options: Protection, Security
    (23:39) -NickServ- Expires on: Jan 15 23:23:25 2007 CET
    */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"info")){
        if(aliases5.forbid) send_lang(thread,language,howtosend,snick,nick,"NS_GROUP_NICK_FORBIDDEN_WITH_REASON",aliases5.last_quit);
		else if(streqnc(param[4],aliases5.nick) && !aliases5.forbid){
    		if(access5.settings&NS_SUSPENDED){
    			send_lang(thread,language,howtosend,snick,nick,"LOGIN_SUSPENDED");
    			send_lang(thread,language,howtosend,snick,nick,"LOGIN_SUSPEND_REASON",access5.reason);
            }
		    if(aliases5.active){
				if(~access5.settings&NS_SET_HIDE_STATUS){
					send_lang(thread,language,howtosend,snick,nick,"NS_INFO_USER_ONLINE_NOW",param[4]);
					send_lang(thread,language,howtosend,snick,nick,"NS_INFO_USER_IS_AUTHED_AS_SINCE",param[4],aliases5.authnick,get_localtime(aliases5.last_login));
				}
		    }else
				send_lang(thread,language,howtosend,snick,nick,"NS_INFO_USER_OFFLINE_NOW",param[4]);
		    
		    send_lang(thread,language,howtosend,snick,nick,"NS_INFO_REGISTERED_SINCE",get_localtime(aliases5.time_linked));
		    
		    if(~access5.settings&NS_SET_HIDE_STATUS)
				send_lang(thread,language,howtosend,snick,nick,"NS_INFO_GLOBAL_LEVEL",param[4],access5.qlevel,access5.olevel,access5.llevel);
            
            if(~access5.settings&NS_SET_HIDE_STATUS){
    		    if(access5.rights == is_services_user) send_lang(thread,language,howtosend,snick,nick,"NS_INFO_IS_SERVICES_USER",param[4]);
    		    else if(access5.rights == is_services_oper) send_lang(thread,language,howtosend,snick,nick,"NS_INFO_IS_SERVICES_OPER",param[4]);
    		    else if(access5.rights == is_services_admin) send_lang(thread,language,howtosend,snick,nick,"NS_INFO_IS_SERVICES_ADMIN",param[4]);
    		    else if(access5.rights == is_services_root) send_lang(thread,language,howtosend,snick,nick,"NS_INFO_IS_SERVICES_ROOT",param[4]);
		    }
			if(thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"info") || streqnc(thread->sql.aliases.nick,aliases5.nick))
				send_lang(thread,language,howtosend,snick,nick,"NS_INFO_LAST_LOGIN",get_localtime(aliases5.last_login));
            
			if(!aliases5.forbid && aliases5.last_quit[0]!='\0' && (~access5.settings&NS_SET_HIDE_QUIT || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"info") || streqnc(thread->sql.aliases.nick,aliases5.nick)))
                send_lang(thread,language,howtosend,snick,nick,"NS_INFO_LAST_QUITMSG",aliases5.last_quit);
			
            if(aliases5.last_vhost[0]!='\0' && (~access5.settings&NS_SET_HIDE_MASK || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"info") || streqnc(thread->sql.aliases.nick,aliases5.nick)))
    		    send_lang(thread,language,howtosend,snick,nick,"NS_INFO_USERMASK",aliases5.last_vhost);

            if(access5.email[0]!='\0' && (~access5.settings&NS_SET_HIDE_EMAIL || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"info") || streqnc(thread->sql.aliases.nick,aliases5.nick)))
    		    send_lang(thread,language,howtosend,snick,nick,"NS_INFO_EMAIL",access5.email);
    		    
		    if(access5.url[0]!='\0') send_lang(thread,language,howtosend,snick,nick,"NS_INFO_URL",access5.url);	
            if(access5.icq) send_lang(thread,language,howtosend,snick,nick,"NS_INFO_ICQ",access5.icq);
		    if(access5.greet[0]!='\0') send_lang(thread,language,howtosend,snick,nick,"NS_INFO_GREET",access5.greet);

			if(thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"info") || streqnc(thread->sql.aliases.nick,aliases5.nick)){
	    		options[0]='\0';
	    		if(access5.settings&NS_SET_SECURE) straddns(options,"Secure, ",sizeof(options));
	    		if(access5.settings&NS_SET_KILLPROTECT) straddns(options,"Protection, ",sizeof(options));
	    		if(access5.settings&NS_SET_PRIVATE) straddns(options,"Private, ",sizeof(options));
	    		if(access5.settings&NS_SET_HIDE_EMAIL || access5.settings&NS_SET_HIDE_MASK || access5.settings&NS_SET_HIDE_STATUS || access5.settings&NS_SET_HIDE_QUIT) straddns(options,"Hide: ",sizeof(options));
	    		if(access5.settings&NS_SET_HIDE_STATUS) straddns(options,"Status, ",sizeof(options));
				if(access5.settings&NS_SET_HIDE_EMAIL) straddns(options,"Email, ",sizeof(options));
	    		if(access5.settings&NS_SET_HIDE_QUIT) straddns(options,"Quit, ",sizeof(options));
	    		if(access5.settings&NS_SET_HIDE_MASK) straddns(options,"Usermask, ",sizeof(options));
			    if(options[0]!='\0'){
	                options[strlen(options)-2]='\0';
	                send_lang(thread,language,howtosend,snick,nick,"NS_INFO_OPTIONS",options);
	            }
			}

		    if(aliases5.expire) send_lang(thread,language,howtosend,snick,nick,"NS_INFO_EXPIRE",get_localtime(aliases5.last_used+(expire(m->NSExpire))));
		    else send_lang(thread,language,howtosend,snick,nick,"NS_INFO_NOEXPIRE");
			
			if(thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"info") || streqnc(thread->sql.aliases.nick,aliases5.nick)){
			    bzero(&escaped,sizeof(escaped)); //variables for MySQL
			    mysql_real_escape_string(MYSQL_CONNECTION_HANDLE, escaped, aliases5.authnick, strlen(aliases5.authnick));
	            
	            chanaccess.res    = doSelectMySQL(thread,"SELECT `channel`,`level` FROM `orion_cs_access` WHERE `authnick`='%s' AND `serv` IN ('Q','L')",escaped);
	            chanaccess.fields = mysql_num_rows(chanaccess.res);
	
			    if(chanaccess.fields){
	    		    msend(thread,howtosend,snick,nick,"\r\n");
	    		    send_lang(thread,language,howtosend,snick,nick,"NS_ALIST_SAY_QL_CHANNELS",aliases5.authnick);
	    		    while((chanaccess.row = mysql_fetch_row(chanaccess.res))){
	    				i++;
	    				if(chanaccess.row) send_lang(thread,language,howtosend,snick,nick,"NS_ALIST_QL_ACCESS_LISTINGS",i,chanaccess.row[0],chanaccess.row[1]);
	    				else send_lang(thread,language,howtosend,snick,nick,"END_OF_LIST");
	    		    }
	    		    i=0;
			    }
			    mysql_free_result_myr(chanaccess.res); //free result
	        		
	        	old_fields        = chanaccess.fields;
			    chanaccess.res    = doSelectMySQL(thread,"SELECT `channel`,`level` FROM `orion_cs_access` WHERE `authnick`='%s' AND (`serv`='ChanServ')",escaped);
	            chanaccess.fields = mysql_num_rows(chanaccess.res);
	
				if(old_fields && chanaccess.fields)
					msend(thread,howtosend,snick,nick,"---------------------");
	
			    if(chanaccess.fields){
					msend(thread,howtosend,snick,nick,"\r\n");
	    		    send_lang(thread,language,howtosend,snick,nick,"NS_ALIST_SAY_CHANSERV_CHANNELS",aliases5.authnick);
	    		    while((chanaccess.row = mysql_fetch_row(chanaccess.res))){
	    				i++;
	    				if(chanaccess.row) send_lang(thread,language,howtosend,snick,nick,"NS_ALIST_CHANSERV_ACCESS_LISTINGS",i,chanaccess.row[0],chanaccess.row[1]);
	    				else send_lang(thread,language,howtosend,snick,nick,"END_OF_LIST");
	    		    }
			    }
			    
	            mysql_free_result_myr(chanaccess.res); //free result
			}
		}else send_lang(thread,language,howtosend,snick,nick,"NOT_KNOWN_USER",param[4]);
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the NickServ INFO Command (/msg NickServ HELP INFO)
 * 
 * When the user tries to get help for this command, using /msg NickServ HELP INFO,<br/>
 * this function is executed.
 * 
 * Syntax: /msg NickServ HELP INFO
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_info_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Info Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help info"))
		send_lang(thread,language,howtosend,snick,nick,"NS_HELP_INFO");

	else send_lang(thread,language,howtosend,snick,nick,"NS_NO_HELP_AVAIL",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief NickServ Help List Command - INFO part (/msg NickServ help)
 * 
 * When the user tries to get general NickServ help, using /msg NickServ help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the INFO command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_info_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_NS_Level=(thread->sql.aliases.active>0 ? thread->sql.access.rights : is_user);
	/*
	 * Info (Help listing)
	 */
	if(Users_NS_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"NS_HELP_CMD_INFO");

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
