/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 202                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-24 17:29:41 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file ns_list.c
 *   @brief This is the Module file for NickServ LIST Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the NickServ LIST command.
 *
 *   The NickServ LIST Command allows the user to search for a mask in the database.<br/>
 *   Lists all registered nicknames which match the given<br/>
 *   pattern, in *nick* format. Nicks with the PRIVATE<br/>
 *   option set will only be displayed to Services admins.
 *
 *   (This Command may be limited to Services Admins)
 *
 *   Syntax is: /msg NickServ LIST [*nick-pattern*] [FORBIDDEN] [NOEXPIRE] [SUSPEND]
 *
 *<table border="0">
 *    <tr><td> FORBIDDEN	</td><td> Syntax is: /msg NickServ LIST [*nick-pattern*] FORBIDDEN	</td><td> (will list only forbidden nicks)						</td></tr>
 *    <tr><td> SUSPEND		</td><td> Syntax is: /msg NickServ LIST [*nick-pattern*] SUSPEND	</td><td> (will list only suspended nicks)						</td></tr>
 *    <tr><td> NOEXPIRE		</td><td> Syntax is: /msg NickServ LIST [*nick-pattern*] NOEXPIRE	</td><td> (will list only nicks with the NO-Expire Option set)	</td></tr>
 *</table>
 *
 *   Combinations like i.e - /msg NickServ LIST [*nick-pattern*] FORBIDDEN NOEXPIRE - ARE POSSIBLE!
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "services.h"
#include "global.h"

#define SERVICE "ns"
#define MODUL "list"
#define MODUL_FILE "ns_list"
#define MODUL_VERSION "1.0"

MOD_BOOL ns_list(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_list_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_list_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"list","list");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	
	if(m->NSListOpersOnly){
		addOptions(thread,&Module,3,LOGINONLY,OPER_IF_DEFCON,OPERONLY);
		addAccess (thread,&Module,ADMIN);
	}else{
		addOptions(thread,&Module,2,LOGINONLY,OPER_IF_DEFCON);
		addAccess (thread,&Module,BOTH);
	}
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"list",ns_list,is_services_oper,is_services_user,4,7);
	addCommand(thread,&Module,"help list",ns_list_help,is_services_user,is_services_user,4,4);
	addCommand(thread,&Module,"help",ns_list_help_listing,is_services_user,is_services_user,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}


/*  ************************************************************************************************  */

/**
 * @brief Main function for the NickServ LIST Command.
 * 
 * The NickServ LIST Command allows the user to search for a mask in the database.<br/>
 *   Lists all registered nicknames which match the given<br/>
 *   pattern, in *nick* format. Nicks with the PRIVATE<br/>
 *   option set will only be displayed to Services admins.<br/>
 *   (This Command may be limited to Services Admins)
 *
 * Syntax is: /msg NickServ LIST [*nick-pattern*] [FORBIDDEN] [NOEXPIRE] [SUSPEND]
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_list(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	deepSQL nicklist;
	char wildcard[MAX_CHANNELNAME_LEN];
	int counter=0;
	int i=5;
	int suspended=0, forbidden=0, noexpire=0;

	/* Initialise Variables to (NULL) */
	bzero(&nicklist,sizeof(nicklist));
	bzero(&wildcard,sizeof(wildcard));
	
	/*
	 * List Command
	 */	
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"list")){

		if(strcor(param[4],"*")){
			while(param[i]){
				if(streqnc(param[i],"forbidden")) forbidden=1;
				else if(streqnc(param[i],"suspend")) suspended=1;
				else if(streqnc(param[i],"noexpire")) noexpire=1;
				i++;
			}
			dynncpy(wildcard,param[4],sizeof(wildcard));
			strdelc(wildcard,'*');

			nicklist.res    = doSelectMySQL(thread,"SELECT `orion_ns_aliases`.`nick`,`orion_ns_aliases`.`display`,`orion_ns_aliases`.`active`,`orion_ns_aliases`.`forbid`,`orion_ns_aliases`.`expire`,`orion_ns_core`.`settings` FROM `orion_ns_aliases` LEFT OUTER JOIN `orion_ns_core` ON (`orion_ns_core`.`authnick`=`orion_ns_aliases`.`display`) WHERE `orion_ns_aliases`.`nick` LIKE '%c%s%c' LIMIT %d",'%',mysql_quote(thread,wildcard),'%',m->NSListMax);
			nicklist.fields = mysql_num_rows(nicklist.res);
			
			for(i=0;i<nicklist.fields;i++){
				nicklist.row = mysql_fetch_row(nicklist.res);
				
				if((~stoi(nicklist.row[5]) & NS_SET_PRIVATE) || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"list")){
					if(stoi(nicklist.row[2])==1){
						if(stoi(nicklist.row[3])==1 && forbidden)					send_lang(thread,language,howtosend,snick,nick,"NS_LIST_LIST_LOGGED_IN_FLAG",++counter,nicklist.row[0],nicklist.row[1],"Forbidden");
						else if(stoi(nicklist.row[5])&NS_SUSPENDED && suspended)	send_lang(thread,language,howtosend,snick,nick,"NS_LIST_LIST_LOGGED_IN_FLAG",++counter,nicklist.row[0],nicklist.row[1],"Suspended");
						else if(stoi(nicklist.row[4])==0 && noexpire)				send_lang(thread,language,howtosend,snick,nick,"NS_LIST_LIST_LOGGED_IN",++counter,_dynadd("!%s",nicklist.row[0]),nicklist.row[1]);
						else if(!forbidden && !suspended && !noexpire){
							if(stoi(nicklist.row[4])==0)	send_lang(thread,language,howtosend,snick,nick,"NS_LIST_LIST_LOGGED_IN",++counter,_dynadd("!%s",nicklist.row[0]),nicklist.row[1]);
							else							send_lang(thread,language,howtosend,snick,nick,"NS_LIST_LIST_LOGGED_IN",++counter,nicklist.row[0],nicklist.row[1]);
						}
					}else{
						if(stoi(nicklist.row[3])==1 && forbidden)					send_lang(thread,language,howtosend,snick,nick,"NS_LIST_LIST_LOGGED_OUT_FLAG",++counter,nicklist.row[0],nicklist.row[1],"Forbidden");
						else if(stoi(nicklist.row[5])&NS_SUSPENDED && suspended)	send_lang(thread,language,howtosend,snick,nick,"NS_LIST_LIST_LOGGED_OUT_FLAG",++counter,nicklist.row[0],nicklist.row[1],"Suspended");
						else if(stoi(nicklist.row[4])==0 && noexpire)				send_lang(thread,language,howtosend,snick,nick,"NS_LIST_LIST_LOGGED_OUT",++counter,_dynadd("!%s",nicklist.row[0]),nicklist.row[1]);
						else if(!forbidden && !suspended && !noexpire){
							if(stoi(nicklist.row[4])==0)	send_lang(thread,language,howtosend,snick,nick,"NS_LIST_LIST_LOGGED_OUT",++counter,_dynadd("!%s",nicklist.row[0]),nicklist.row[1]);
							else							send_lang(thread,language,howtosend,snick,nick,"NS_LIST_LIST_LOGGED_OUT",++counter,nicklist.row[0],nicklist.row[1]);
						}
					}
				}
			}
			//strrepc(param[4],'%','*');
			//printf("param4: %s\n",param[4]);
			strdelc(param[4],'*');
			if(counter!=0){
				send_lang(thread,language,howtosend,snick,nick,"NS_LIST_FOUND_X_MATCHING",counter,param[4]);
				send_lang(thread,language,howtosend,snick,nick,"NS_LIST_USE_WHOIS_FOR_INFO",ns->nick);
			}else send_lang(thread,language,howtosend,snick,nick,"NS_LIST_FOUND_NOTHING_MATCHING",param[4]);

			/* MySQL Free Result */
			mysql_free_result_myr(nicklist.res);
		}else send_lang(thread,language,howtosend,snick,nick,"NS_LIST_FAILED_HAVE_TO_SEARCH_WITH_WILDCARD");
	}else{
	/*
	 *  Error Messages
	 */
		if(m->NSListOpersOnly)  send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
		else					send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the NickServ LIST Command (/msg NickServ HELP LIST)
 * 
 * When the user tries to get help for this command, using /msg NickServ HELP LIST,<br/>
 * this function is executed.
 * 
 * Syntax: /msg NickServ HELP LIST
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_list_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * List Help
	 */
	if(thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"help list"))
		send_lang(thread,language,howtosend,snick,nick,"NS_SERVADMIN_HELP_LIST");
	else if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help list"))
		send_lang(thread,language,howtosend,snick,nick,"NS_HELP_LIST");

	else send_lang(thread,language,howtosend,snick,nick,"NS_NO_HELP_AVAIL",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief NickServ Help List Command - LIST part (/msg NickServ help)
 * 
 * When the user tries to get general NickServ help, using /msg NickServ help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the LIST command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_list_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_NS_Level=(thread->sql.aliases.active>0 ? thread->sql.access.rights : is_user);
	/*
	 * List (Help listing)
	 */
	if(Users_NS_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"NS_HELP_CMD_LIST");

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
