/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 202                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-24 17:29:41 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file ns_logout.c
 *   @brief This is the Module file for NickServ LOGOUT Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the NickServ LOGOUT command.
 *
 *   The NickServ LOGOUT Command allows the user to log hisself / an other user out.<br/>
 *   Logging out other users is limited to Services Admins! (< nick >)<br/>
 *   Logging out other users and forcing them to reidentfy is limited to Services Admins! (REVALIDATE)
 *
 *   Syntax is: /msg NickServ LOGOUT [< nick > [REVALIDATE]]
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "services.h"
#include "global.h"

#define SERVICE "ns"
#define MODUL "logout"
#define MODUL_FILE "ns_logout"
#define MODUL_VERSION "1.0"

MOD_BOOL ns_logout(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_logout_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_logout_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"logout","logout");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,2,LOGINONLY,OPER_IF_DEFCON);
	addAccess (thread,&Module,BOTH);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"logout",ns_logout,is_services_admin,is_services_user,3,5);
	addCommand(thread,&Module,"help logout",ns_logout_help,is_services_user,is_services_user,4,4);
	addCommand(thread,&Module,"help",ns_logout_help_listing,is_services_user,is_services_user,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}


/*  ************************************************************************************************  */

/**
 * @brief Main function for the NickServ LOGOUT Command.
 * 
 * The NickServ LOGOUT Command allows the user to log hisself / an other user out.<br/>
 *   Logging out other users is limited to Services Admins! (< nick >)<br/>
 *   Logging out other users and forcing them to reidentfy is limited to Services Admins! (REVALIDATE)
 *
 * Syntax is: /msg NickServ LOGOUT [< nick > [REVALIDATE]]
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_logout(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	Aliases aliases5;
	Access access5;
	Users *user = NULL;

	/* Initialise Variables to (NULL) */
	bzero(&aliases5,sizeof(aliases5));
	bzero(&access5,sizeof(access5));
	
	/*
	 * Logout Command
	 */
	 //   0       1       2       3     4       5
	 //:shock PRIVMSG NickServ :logout shock REVALIDATE

	aliases5 = sql_aliases(thread,param[4]);
	access5  = sql_access(thread,aliases5.authnick);
	
	if(access5.rights<=is_services_admin && thread->sql.access.rights>=ModuleFile_UserCommandLevel(thread,MODUL_FILE,"logout")){ //cmd logout
		if(param[4] && streqnc(param[5],"revalidate") && thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"logout")){
			if(aliases5.active){
				user 		 = GetUser(param[4]);
				user->rights = is_user;
				
				if(IRC.NickRegMode)
					do_svs2mode(thread,ns->nick,param[4],IRC.ModeOnNickDrop);
					
				if(IRC.p10protocol){
					if(do_del_account(thread,GetUser(param[4])->uid) > 0)
						return MOD_CONT;
				}

				domysql(thread,"UPDATE `orion_ns_aliases` SET `active`='0' WHERE `nick`='%s'",mysql_quote(thread,param[4]));
				send_lang(thread,language,howtosend,snick,nick,"NS_LOGOUT_LOGGED_OUT_GIVEN_USER",param[4]);
					
				do_logchan(thread,snick,nick,thread->sql.Cdata.ident,thread->sql.Cdata.rhost,"logged out nickname %s",param[4]);
				send_lang(thread,languages[access5.language],access5.settings&NS_SET_MSG ? 1 : 0,snick,param[4],"NS_WARN_NICK_REGISTERED_BY_ANOTHER_USER");//Keine Rechte - Warnen mit NickServ dieser nick ist nicht in der selben gruppe - geh�t wem anderen (nswarntime setzen)
				domysql(thread,"UPDATE `orion_core_condata` SET `ns_warntime`='%d' WHERE `nick`='%s'",time(NULL),mysql_quote(thread,param[4]));
			}else send_lang(thread,language,howtosend,snick,nick,"NS_LOGOUT_NICK_NOT_IN_USE",param[4]);
		}else if(param[4] && !param[5] && thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"logout")){
			if(aliases5.active){
				user 		 = GetUser(param[4]);
				user->rights = is_user;
				
				if(IRC.NickRegMode)
					do_svs2mode(thread,ns->nick,param[4],IRC.ModeOnNickDrop);
					
				if(IRC.p10protocol){
					if(do_del_account(thread,GetUser(param[4])->uid) > 0)
						return MOD_CONT;
				}
				
				domysql(thread,"UPDATE `orion_ns_aliases` SET `active`='0' WHERE `nick`='%s'",mysql_quote(thread,param[4]));
				send_lang(thread,language,howtosend,snick,nick,"NS_LOGOUT_LOGGED_OUT_GIVEN_USER",param[4]);
					
				do_logchan(thread,snick,nick,thread->sql.Cdata.ident,thread->sql.Cdata.rhost,"logged out nickname %s",param[4]);
			}else send_lang(thread,language,howtosend,snick,nick,"NS_LOGOUT_NICK_NOT_IN_USE",param[4]);
		}else if(!param[4] && !param[5] && thread->sql.aliases.active){
			user 		 = GetUser(nick);
			user->rights = is_user;
			
			if(IRC.NickRegMode)
				do_svs2mode(thread,ns->nick,nick,IRC.ModeOnNickDrop);
				
			if(IRC.p10protocol){
				if(do_del_account(thread,GetUser(nick)->uid) > 0)
					return MOD_CONT;
			}
			
			domysql(thread,"UPDATE `orion_ns_aliases` SET `active`='0' WHERE `nick`='%s'",mysql_quote(thread,nick));
			send_lang(thread,language,howtosend,snick,nick,"NS_LOGOUT_LOGGED_YOU_OUT");
					
			do_logchan(thread,snick,nick,thread->sql.Cdata.ident,thread->sql.Cdata.rhost,"logged out nickname %s",nick);
		}else send_lang(thread,language,howtosend,snick,nick,"MISSING_PARAMETER");
	}else{
	/*
	 *  Error Messages
	 */
		if(access5.rights>=is_services_admin)	send_lang(thread,language,howtosend,snick,nick,"NS_LOGOUT_CANT_LOGOUT_X_IS_ADMIN",param[4]);
		else 									send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the NickServ LOGOUT Command (/msg NickServ HELP LOGOUT)
 * 
 * When the user tries to get help for this command, using /msg NickServ HELP LOGOUT,<br/>
 * this function is executed.
 * 
 * Syntax: /msg NickServ HELP LOGOUT
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_logout_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Logout Help
	 */
	if(thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"help logout"))
		send_lang(thread,language,howtosend,snick,nick,"NS_SERVADMIN_HELP_LOGOUT");
	else if(thread->sql.access.rights>=ModuleFile_UserCommandLevel(thread,MODUL_FILE,"help logout"))
		send_lang(thread,language,howtosend,snick,nick,"NS_HELP_LOGOUT");
		
	else send_lang(thread,language,howtosend,snick,nick,"NS_NO_HELP_AVAIL",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief NickServ Help List Command - LOGOUT part (/msg NickServ help)
 * 
 * When the user tries to get general NickServ help, using /msg NickServ help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the LOGOUT command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_logout_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_NS_Level=(thread->sql.aliases.active>0 ? thread->sql.access.rights : is_user);
	/*
	 * Logout (Help listing)
	 */
	if(Users_NS_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"NS_HELP_CMD_LOGOUT");

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
