/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 202                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-24 17:29:41 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file ns_register.c
 *   @brief This is the Module file for NickServ REGISTER Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the NickServ REGISTER command.
 *
 *   The NickServ REGISTER Command allows the user to register a new user account.<br/>
 *   If EAS (Email Activation System) is enabled, you will find a passcode in you mailbox,<br/>
 *   don't answer that email - follow the Instructions there to verify your Email-Address!
 *
 *   Syntax is: /msg NickServ REGISTER < password > < email >
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "services.h"
#include "global.h"

#define SERVICE "ns"
#define MODUL "register"
#define MODUL_FILE "ns_register"
#define MODUL_VERSION "1.0"

MOD_BOOL ns_register(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_register_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_register_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"register","register");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,2,OPER_IF_DEFCON,READONLY);
	addAccess (thread,&Module,USER);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"register",ns_register,is_user,is_user,4,5);
	addCommand(thread,&Module,"help register",ns_register_help,is_user,is_user,4,4);
	addCommand(thread,&Module,"help",ns_register_help_listing,is_user,is_user,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}


/*  ************************************************************************************************  */

/**
 * @brief Main function for the NickServ REGISTER Command.
 * 
 * The NickServ REGISTER Command allows the user to register a new user account.<br/>
 *   If EAS (Email Activation System) is enabled, you will find a passcode in you mailbox,<br/>
 *   don't answer that email - follow the Instructions there to verify your Email-Address!
 *
 * Syntax is: /msg NickServ REGISTER < password > < email >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_register(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
    /* Global Variables */
	char tmppasswd[MAX_PASSWORD_LEN + 1];
	char passcode[PASSWORD_LEN];
	char From[80];
	char Subject[80];
	time_t now;
	Users *user = NULL;
	
	/* Initialise Variables to (NULL) */
	bzero(&tmppasswd,sizeof(tmppasswd));
	bzero(&passcode,sizeof(passcode));
	bzero(&From,sizeof(From));
	bzero(&Subject,sizeof(Subject));

	/*
	 * Register Command
	 */
	 //   0     1     2    3           4                  5
	 //:shock PRIVMSG Q :hello Manuel-Moser@aon.at Manuel-Moser@aon.at
	 //   0       1       2         3        4           5
	 //:shock PRIVMSG NickServ :register 34154424 Manuel-Moser@aon.at
	 
	if(thread->sql.Cdata.localtime+m->NSWaitRegister>time(NULL)){
		send_lang(thread,language,howtosend,snick,nick,"NS_REGISTER_NOT_LONG_ENOUGH_CONNECTED",m->NSWaitRegister);
		return MOD_CONT;
	}
	
	else if(m->NSRegDelay){
		user = GetUser(&param[0][1]);
		now 	    									 = time(NULL);
		if(user->LastNSRegister==0) user->LastNSRegister = time(NULL);
			
		if(user->rights < is_services_oper){

			if (now - user->LastNSRegister > m->NSRegDelay) {
				user->LastNSRegister 	= time(NULL);
				user->NSRegisterMsgs 	= 0;
			}
			
			if (user->NSRegisterMsgs) {
				msend(thread,0,m->servername,user->nick,"Due to a Floodattack services are not available at the moment, please be patient");
				return MOD_CONT;
			}
		}
	}
	
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"register") && dynlen(param[4])<=MAX_PASSWORD_LEN){ //cmd register
		if((!m->StrictPasswords && dynlen(param[4])>=ns->min_passwordlen) || (m->StrictPasswords && !strins(nick,param[4]) && dynlen(param[4])>=ns->min_passwordlen)){
			if(param[5]!=NULL){  //password longer than 5 charakters
			
				if(MailValidate(param[5])){ //email ok and accepted
					if(thread->sql.access.settings&NS_SUSPENDED || thread->sql.aliases.forbid) send_lang(thread,language,howtosend,snick,nick,"NICK_SUSPENDED_OR_FORBIDDEN",nick);
					else if(thread->sql.aliases.active && thread->sql.aliases.authnick[0]!='\0') send_lang(thread,language,howtosend,snick,nick,"REGISTERED_ALREADY");
					
					else if(stoi(mysql_sel_var(thread,"SELECT count(1) FROM `orion_ns_core` WHERE `email`='%s'",mysql_quote(thread,param[5])))==m->NSMaxGroups){
						send_lang(thread,language,howtosend,snick,nick,"EMAIL_USED_TOO_OFTEN_LIMIT_X",m->NSMaxGroups);
						return MOD_CONT;
					}
					
					else if(thread->sql.aliases.nick[0]=='\0'){
						if(is_valid_chars(param[4],"0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ+-.:<>|^!\"$%&/()=?*'#,;~{[]}\\`")){
							send_lang(thread,language,howtosend,snick,nick,"INVALID_PASSWORD_CHARS","0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ+-.:<>|^!\"$%&/()=?*'#,;~{[]}\\`");
							return MOD_CONT;
						}

						dynncpy(tmppasswd,param[4],sizeof(tmppasswd));
						randomstring(passcode,sizeof(passcode)-1);
						//Crypt ON/OFF
						
						char enc_password[ENC_PASSMAX+1];
						bzero(&enc_password,sizeof(enc_password));
						if (orion_encrypt(tmppasswd, strlen(tmppasswd), enc_password, ENC_PASSMAX) < 0) {
							//memset(tmppasswd, 0, PASSWORD_LEN);
							alog(thread,"%s: Failed to encrypt password for %s (register)\n",
								snick, nick);
							send_lang(thread,language,howtosend,snick,nick,"REGISTRATION_FAILED");
							return MOD_CONT;
						}
						
						alog(thread,"\nenc (register): hashed password to [%s]\n",enc_password); 

						if(m->NSEmailReg){
							if(stoi(mysql_sel_var(thread,"SELECT count(1) FROM `orion_ns_request` WHERE `display`='%s'",mysql_quote(thread,nick)))==0){
								domysql(thread,"INSERT INTO `orion_ns_request` (`display`,`passcode`,`password`,"
															  	"`email`,`reqtime`,`reqsent`,`emailchange`)"
															"VALUES ('%s','%s','%s','%s','%ld','%ld','0')",
											mysql_quote(thread,nick),mysql_quote(thread,passcode),mysql_quote(thread,enc_password),param[5],time(NULL),time(NULL));
								
								if(param[5][0]!='\0' && nick[0]!='\0' && param[4][0]!='\0' && m->UseMail){
									dynncpy(From,_dynadd("%s Network (%s) <%s>",m->network,ns->nick,m->SendFrom),sizeof(From));
									dynncpy(Subject,_dynadd("Your %s account",snick),sizeof(Subject));
					
									MailInfo *mail = o_email(thread,From,nick,param[5],Subject);
									//Email					
									s_email(thread,language,mail->pipe,"NS_REGISTER_REQUEST_NICK_EMAIL",nick,tmppasswd,passcode,ns->nick,0xA0,snick,ns->host,passcode,ns->nick,0xA0,snick,ns->host,tmppasswd,0xA0,snick,nick,nick,Q->nick,0xA0,snick,m->NSExpire,m->homepage);
									c_email(thread,mail);
								}
								
								do_logchan(thread,snick,NULL,NULL,NULL,"Nick %s has been requested",nick);
								send_lang(thread,language,howtosend,snick,nick,"NS_REGISTER_REQUESTED_NICK",nick,param[5]);
								send_lang(thread,language,howtosend,snick,nick,"NS_REGISTER_NICK_PASSWORD_IS",param[4]);
							}else send_lang(thread,language,howtosend,snick,nick,"NICK_REGISTERED_BY_AN_OTHER_USER");
						}
						else{
							domysql(thread,"INSERT INTO `orion_ns_core` (`authnick`,`password`,`qlevel`,"
															  "`olevel`,`slevel`,`llevel`,`email`,"
															  "`realname`,`language`,`rights`,"
															  "`maxmemos`,`settings`)"
															"VALUES ('%s','%s','1','0','0','1','%s','%s','%d','%d','%d','%d')",
										mysql_quote(thread,nick),mysql_quote(thread,enc_password),param[5],mysql_quote(thread,thread->sql.Cdata.realname),m->language,is_services_user,m->maxmemos,NSCheckSettings());
							domysql(thread,"INSERT INTO `orion_ns_aliases` (`nick`,`display`,`last_usermask`,`last_realname`,"
													"`time_linked`,`last_login`,`last_used`,"
													"`expire`,`linked`,`active`,`forbid`)"
													"VALUES ('%s','%s','%s','%s','%ld','%ld','%ld','1','0','1','0')",
										mysql_quote(thread,nick),mysql_quote(thread,nick),mysql_quote(thread,thread->sql.Cdata.vhost),mysql_quote(thread,thread->sql.Cdata.realname),time(NULL),time(NULL),time(NULL));
						}

						if(!m->NSEmailReg){
							send_lang(thread,language,howtosend,snick,nick,"NS_REGISTER_NICK_REGISTERED",nick,thread->sql.Cdata.vhost);
							send_lang(thread,language,howtosend,snick,nick,"NS_REGISTER_NICK_PASSWORD_IS",param[4]);
							if(IRC.NickRegMode){
								do_svs2mode(thread,ns->nick,nick,IRC.ModeOnNickReg);
							}
								
							if(IRC.p10protocol){
								do_add_account(thread,GetUser(nick)->uid,nick);
							}

							do_logchan(thread,snick,NULL,NULL,NULL,"Nick %s has been requested",nick);
							do_logchan(thread,snick,NULL,NULL,NULL,"group %s has been created",nick);
							do_logchan(thread,snick,NULL,NULL,NULL,"\'%s\' registered by %s@%s (email: %s)",nick,thread->sql.Cdata.ident,thread->sql.Cdata.rhost,param[5]);
						
							if(param[5][0]!='\0' && nick[0]!='\0' && param[4][0]!='\0' && m->UseMail){
								dynncpy(From,_dynadd("%s Network (%s) <%s>",m->network,ns->nick,m->SendFrom),sizeof(From));
								dynncpy(Subject,_dynadd("Your %s account",snick),sizeof(Subject));
					
								MailInfo *mail = o_email(thread,From,nick,param[5],Subject);
								//Email					
								s_email(thread,language,mail->pipe,"NS_REGISTER_EMAIL",nick,tmppasswd,0xA0,snick,ns->host,tmppasswd,0xA0,snick,nick,nick,Q->nick,0xA0,snick,m->NSExpire,m->homepage);
								c_email(thread,mail);
							}
						}
						
						user->NSRegisterMsgs++; // Registered a nickname!
						
					}else send_lang(thread,language,howtosend,snick,nick,"NICK_REGISTERED_BY_AN_OTHER_USER");
				}else send_lang(thread,language,howtosend,snick,nick,"NS_REGISTER_INVALID_EMAIL");
			}else send_lang(thread,language,howtosend,snick,nick,"NS_REGISTER_FORGOT_TYPE_EMAIL");
		}else{
			send_lang(thread,language,howtosend,snick,nick,"PASSWORD_TOO_SHORT",ns->min_passwordlen);
			send_lang(thread,language,howtosend,snick,nick,"NS_REGISTER_SYNTAX",snick);
		}
	}else{
	/*
	 *  Error Messages
	 */
	 	if(dynlen(param[4])>MAX_PASSWORD_LEN)	send_lang(thread,language,howtosend,snick,nick,"PASSWORD_TOO_LONG",MAX_PASSWORD_LEN);
		else									send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the NickServ REGISTER Command (/msg NickServ HELP REGISTER)
 * 
 * When the user tries to get help for this command, using /msg NickServ HELP REGISTER,<br/>
 * this function is executed.
 * 
 * Syntax: /msg NickServ HELP REGISTER
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_register_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Register Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help register"))
		send_lang(thread,language,howtosend,snick,nick,"NS_HELP_REGISTER",snick,snick,snick);

	else send_lang(thread,language,howtosend,snick,nick,"NS_NO_HELP_AVAIL",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief NickServ Help List Command - REGISTER part (/msg NickServ help)
 * 
 * When the user tries to get general NickServ help, using /msg NickServ help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the REGISTER command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_register_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_NS_Level=(thread->sql.aliases.active>0 ? thread->sql.access.rights : is_user);
	/*
	 * Register (Help listing)
	 */
	if(Users_NS_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"NS_HELP_CMD_REGISTER");

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
