/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 202                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-24 17:29:41 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file ns_suspend.c
 *   @brief This is the Module file for NickServ SUSPEND Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the NickServ SUSPEND command.
 *
 *   The NickServ SUSPEND Command allows an admin to disallow anyone from<br/>
 *   registering or using the given < nick >.
 *
 *   May be cancelled by using the UNSUSPEND command to preserve all<br/>
 *   previous nick data/settings.<br/>
 *   (Reason may be required on certain networks.)<br/>
 *     NickServ suspends a nick and loggs it out (admins only).
 *
 *   Syntax is: /msg NickServ SUSPEND < nick > < reason >
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"

#define SERVICE "ns"
#define MODUL "suspend"
#define MODUL_FILE "ns_suspend"
#define MODUL_VERSION "1.0"

MOD_BOOL ns_suspend(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_suspend_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_suspend_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"suspend","suspend");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,4,LOGINONLY,OPER_IF_DEFCON,OPERONLY,READONLY);
	addAccess (thread,&Module,ADMIN);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"suspend",ns_suspend,is_services_admin,INFINIT,4,0);
	addCommand(thread,&Module,"help suspend",ns_suspend_help,is_services_admin,INFINIT,4,4);
	addCommand(thread,&Module,"help",ns_suspend_help_listing,is_services_admin,INFINIT,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}


/*  ************************************************************************************************  */

/**
 * @brief Main function for the NickServ SUSPEND Command.
 * 
 * The NickServ SUSPEND Command allows an admin to disallow anyone from<br/>
 *   registering or using the given < nick >.
 *
 *   May be cancelled by using the UNSUSPEND command to preserve all<br/>
 *   previous nick data/settings.<br/>
 *   (Reason may be required on certain networks.)<br/>
 *     NickServ suspends a nick and loggs it out (admins only).
 *
 * Syntax is: /msg NickServ SUSPEND < nick > < reason >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_suspend(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
    /* Global Variables */
    Aliases aliases5;
    Access  access5;
    
	/* Initialise Variables to (NULL) */
	bzero(&aliases5,sizeof(aliases5));
	bzero(&access5,sizeof(access5));
	
	/*
	 * Suspend Command
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"suspend")){
		aliases5 = sql_aliases(thread,param[4]);
		    
		if(aliases5.authnick[0]!='\0'){
    		if(param[5]){
				access5  = sql_access (thread,aliases5.authnick);
				
				if(~access5.settings&NS_SUSPENDED){
					access5.settings |= NS_SUSPENDED;
					
	        		domysql(thread,"UPDATE `orion_ns_core` LEFT OUTER JOIN `orion_ns_aliases` ON `orion_ns_core`.`authnick`=`orion_ns_aliases`.`display` SET `orion_ns_core`.`reason`='%s',`orion_ns_core`.`settings`='%d',`orion_ns_aliases`.`active`='0' WHERE `orion_ns_core`.`authnick`='%s'",mysql_quote(thread,_dstrjoin(' ',param,5)),access5.settings,mysql_quote(thread,aliases5.authnick));
					
					if(IRC.NickRegMode && GetUser(aliases5.nick)!=NULL)
						do_svs2mode(thread,ns->nick,aliases5.nick,IRC.ModeOnNickDrop);
					
	        		send_lang(thread,language,howtosend,snick,nick,"NS_COMMAND_DONE");
				}else send_lang(thread,language,howtosend,snick,nick,"LOGIN_X_SUSPENDED",access5.authnick);
        	}else send_lang(thread,language,howtosend,snick,nick,"NO_REASON");
        }else send_lang(thread,language,howtosend,snick,nick,"NO_AUTHNICK");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the NickServ SUSPEND Command (/msg NickServ HELP SUSPEND)
 * 
 * When the user tries to get help for this command, using /msg NickServ HELP SUSPEND,<br/>
 * this function is executed.
 * 
 * Syntax: /msg NickServ HELP SUSPEND
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_suspend_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Suspend Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help suspend"))
		send_lang(thread,language,howtosend,snick,nick,"NS_SERVADMIN_HELP_SUSPEND");

	else send_lang(thread,language,howtosend,snick,nick,"NS_NO_HELP_AVAIL",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief NickServ Help List Command - SUSPEND part (/msg NickServ help)
 * 
 * When the user tries to get general NickServ help, using /msg NickServ help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the SUSPEND command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_suspend_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_NS_Level=(thread->sql.aliases.active>0 ? thread->sql.access.rights : is_user);
	/*
	 * Suspend (Help Listing)
	 */
	if(Users_NS_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"NS_HELP_CMD_SUSPEND");

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
