/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 202                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-24 17:29:41 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file orion_ircops.c
 *   @brief This is the Module file for Orion IRCOPS Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the Orion IRCOPS command.
 *
 *   The Orion IRCOPS Command allows an admin to dump<br/>
 *   a list of all known ServicesOps (or higher).
 *
 *   Syntax is: /msg Orion IRCOPS
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"

#define SERVICE "orion"
#define MODUL "ircops"
#define MODUL_FILE "orion_ircops"
#define MODUL_VERSION "1.0"

MOD_BOOL orion_ircops(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_ircops_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_ircops_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"ircops","ircops");
	
	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,3,LOGINONLY,OPER_IF_DEFCON,OPERONLY);
	addAccess (thread,&Module,ADMIN);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"ircops",orion_ircops,1000,INFINIT,3,3);
	addCommand(thread,&Module,"help ircops",orion_ircops_help,1000,INFINIT,4,4);
	addCommand(thread,&Module,"help",orion_ircops_help_listing,1000,INFINIT,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}


/*  ************************************************************************************************  */

/**
 * @brief Main function for the Orion IRCOPS Command.
 * 
 * The Orion IRCOPS Command allows an admin to dump<br/>
 *   a list of all known ServicesOps (or higher).
 *
 * Syntax is: /msg Orion IRCOPS
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL orion_ircops(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	deepSQL select;
	int i = 0;
	
	/* Initialise Variables to (NULL) */
	bzero(&select,sizeof(select));
	
	/*
	 * Ircops Command
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"ircops")){
		select.res    = doSelectMySQL(thread,"SELECT `authnick`,`qlevel`,`olevel`,`slevel`,`llevel` FROM `orion_ns_core` WHERE `olevel`>='100'");
		select.fields = mysql_num_rows(select.res);

		for(i=0;i<select.fields;i++){
			select.row = mysql_fetch_row(select.res);
			
			do_privmsg(thread,m->nick,nick,"NICK: %s, Q:%d, O:%d, S:%d, L:%d",select.row[0],stoi(select.row[1]),stoi(select.row[2]),stoi(select.row[3]),stoi(select.row[4]));
		}
		do_privmsg(thread,m->nick,nick,"End of List");

		/* MySQL Free Result */
		mysql_free_result_myr(select.res);
	}else{
		/*
		 *  Error Messages
		 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the Orion IRCOPS Command (/msg Orion HELP IRCOPS)
 * 
 * When the user tries to get help for this command, using /msg Orion HELP IRCOPS,<br/>
 * this function is executed.
 * 
 * Syntax: /msg Orion HELP IRCOPS
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL orion_ircops_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Ircops Help
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help ircops")){
		send_lang(thread,language,1,snick,nick,"REQUIRED_LEVEL",ModuleFile_RootCommandLevel(thread,MODUL_FILE,"ircops"));
		send_lang(thread,language,1,snick,nick,"ORION_HELP_IRCOPS",snick);
	}else send_lang(thread,language,howtosend,snick,nick,"ORION_NO_HELP_AVAIL");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief Orion Help List Command - IRCOPS part (/msg Orion help)
 * 
 * When the user tries to get general Orion help, using /msg Orion help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the IRCOPS command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL orion_ircops_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_ORION_Level=(thread->sql.aliases.active>0 ? thread->sql.access.olevel : is_user);
	/*
	 * Ircops (Help listing)
	 */
	if(Users_ORION_Level>=command_level)
		send_lang(thread,language,1,snick,nick,"ORION_HELP_CMD_IRCOPS",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"ircops"));

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
