/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 202                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-24 17:29:41 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file q_chanstat.c
 *   @brief This is the Module file for Q CHANSTAT Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the Q CHANSTAT command.
 *
 *   The Q CHANSTAT Command allows an admin view<br/>
 *   target channels statistics.
 *
 *   Syntax is: /msg Q CHANSTAT #channel
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"

#define SERVICE "Q"
#define MODUL "chanstat"
#define MODUL_FILE "q_chanstat"
#define MODUL_VERSION "1.0"

MOD_BOOL Q_chanstat(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL Q_chanstat_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL Q_chanstat_showcommands(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"chanstat","chanstat");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,3,LOGINONLY,OPER_IF_DEFCON,OPERONLY);
	addAccess (thread,&Module,ADMIN);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"chanstat",Q_chanstat,600,INFINIT,4,4);
	addCommand(thread,&Module,"help chanstat",Q_chanstat_help,600,INFINIT,4,4);
	addCommand(thread,&Module,"showcommands",Q_chanstat_showcommands,600,INFINIT,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}

/*  ************************************************************************************************  */

/**
 * @brief Main function for the Q CHANSTAT Command.
 * 
 * The Q CHANSTAT Command allows an admin view<br/>
 *   target channels statistics.
 *
 * Syntax is: /msg Q CHANSTAT #channel
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL Q_chanstat(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global variables */

	/* Initialise Variables to (NULL) */

	/*
	 * Chanstat Command
	 */
	if(thread->sql.access.qlevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"chanstat")){
		/* Updates the Threads Channel-Management */
		UpdateChannelInformation(thread,param[4],NULL);
		
		if(thread->c.chan.fields) {
			send_lang(thread,language,howtosend,snick,nick,"Q_CHANSTAT_HEADER",thread->c.chan.name);
			if(streqnc(thread->c.chan.serv,"q")){
				send_lang(thread,language,howtosend,snick,nick,"Q_CHANSTAT_REGISTERED_SINCE",get_localtime(thread->c.chan.regdate));
				send_lang(thread,language,howtosend,snick,nick,"Q_CHANSTAT_HOMEPAGE",thread->c.chan.url);
				send_lang(thread,language,howtosend,snick,nick,"Q_CHANSTAT_CHANNELTYPE",thread->c.chan.description);
				send_lang(thread,language,howtosend,snick,nick,"Q_CHANSTAT_ADDEDBY",thread->c.chan.founder);
				if(thread->c.chan.fields && thread->c.chan.email[0]!='\0') 
					send_lang(thread,language,howtosend,snick,nick,"Q_CHANSTAT_EMAIL",thread->c.chan.email);
					
				msend(thread,howtosend,snick,nick,"--------------------");
				if(thread->c.chan.fields && thread->c.chan.mode[0]!='\0') send_lang(thread,language,howtosend,snick,nick,"Q_CHANSTAT_MODE",thread->c.chan.mode);
				if(thread->c.chan.fields && thread->c.chan.link[0]!='\0') send_lang(thread,language,howtosend,snick,nick,"Q_CHANSTAT_LINK",thread->c.chan.link);
				if(thread->c.chan.fields && thread->c.chan.limit!=0) send_lang(thread,language,howtosend,snick,nick,"Q_CHANSTAT_LIMIT",thread->c.chan.limit);
				if(thread->c.chan.fields && thread->c.chan.flood[0]!='\0') send_lang(thread,language,howtosend,snick,nick,"Q_CHANSTAT_FLOOD",thread->c.chan.flood);
				if(thread->c.chan.fields && thread->c.chan.key[0]!='\0') send_lang(thread,language,howtosend,snick,nick,"Q_CHANSTAT_CHANKEY",thread->c.chan.key);
				if(thread->c.chan.fields && thread->c.chan.last_topic[0]!='\0') send_lang(thread,language,howtosend,snick,nick,"Q_CHANSTAT_TOPIC",thread->c.chan.last_topic);
			}else{
				if(thread->c.chan.fields && thread->c.chan.mode[0]!='\0') send_lang(thread,language,howtosend,snick,nick,"Q_CHANSTAT_MODE",thread->c.chan.mode);
				if(thread->c.chan.fields && thread->c.chan.link[0]!='\0') send_lang(thread,language,howtosend,snick,nick,"Q_CHANSTAT_LINK",thread->c.chan.link);
				if(thread->c.chan.fields && thread->c.chan.limit!=0) send_lang(thread,language,howtosend,snick,nick,"Q_CHANSTAT_LIMIT",thread->c.chan.limit);
				if(thread->c.chan.fields && thread->c.chan.flood[0]!='\0') send_lang(thread,language,howtosend,snick,nick,"Q_CHANSTAT_FLOOD",thread->c.chan.flood);
				if(thread->c.chan.fields && thread->c.chan.key[0]!='\0') send_lang(thread,language,howtosend,snick,nick,"Q_CHANSTAT_CHANKEY",thread->c.chan.key);
				if(thread->c.chan.fields && thread->c.chan.last_topic[0]!='\0') send_lang(thread,language,howtosend,snick,nick,"Q_CHANSTAT_TOPIC",thread->c.chan.last_topic);
			}
			send_lang(thread,language,howtosend,snick,nick,"Q_COMMAND_DONE");
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the Q CHANSTAT Command (/msg Q HELP CHANSTAT)
 * 
 * When the user tries to get help for this command, using /msg Q HELP CHANSTAT,<br/>
 * this function is executed.
 * 
 * Syntax: /msg Q HELP CHANSTAT
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL Q_chanstat_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Chanstat Help
	 */
	if(thread->sql.access.qlevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help chanstat"))
		send_lang(thread,language,howtosend,snick,nick,"Q_HELP_CHANSTAT");

	else send_lang(thread,language,howtosend,snick,nick,"Q_NO_HELP_AVAIL",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Q SHOWCOMMANDS Command - CHANSTAT part (/msg Q showcommands)
 * 
 * When the user tries to get general Q help, using /msg Q showcommands, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to<br/>
 * include the information for the CHANSTAT command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL Q_chanstat_showcommands(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=0, Users_Q_Level=0;
	command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"showcommands");
	Users_Q_Level=(thread->sql.aliases.active>0 ? thread->sql.access.qlevel : is_user);
	/*
	 * Chanstat (Showcommands)
	 */
	if(Users_Q_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"Q_SHOWCOMMANDS_CHANSTAT",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"chanstat"));

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
