/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  Please read COPYING and README for further details.
 *  Based on the original code of Services by Andy Church. 
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 194                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-16 18:33:19 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file dynamic.c
 *   @brief This is the Module file for dynamic string manipulations.
 *
 *   It contains all functions which are neccessary to
 *   manipulate strings, pointer arrays. (static vars or malloc/calloc)
 *
 *   dynamic lib, dynamic functions - working with static vars or malloc/calloc
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "services.h"

#ifndef USE_THREADS

/**
 * @brief dynlen:  Reads a string len, it's like a synonym for strlen();
 *
 * @param string String to check for it's length
 * @return String length
 **/
size_t dynlen(char *string){
	size_t i = 0;

	if(string == NULL) return (size_t)0;

	while( string[i] ){
		i++;
	}
	return i;
}

/*  ************************************************************************************************  */

/**
 * @brief dyncat:  Adds a String to another String, by writing to the source String and
 *           adds a null terminator after the last character copied. (synonym strncat();)
 *
 * @param source Buffer where data-string should be added to
 * @param data Data-string to add to the source-string
 * @return void
 **/
void dyncat(char *source,char *data){
	size_t i = dynlen(source);
	size_t b = 0;

	if(source == NULL || data == NULL) return;

	while( data[b] ){
		source[i] = data[b];
		i++;
		b++;
	}
	source[i++] = '\0';
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief dynncat:  Adds x chars from String-A to String-B, by writing to the source String and
 *           adds a null terminator after the last character copied. (synonym strnncat();)
 *
 * @param source Buffer where data-string should be added to
 * @param data Data-string to add to the source-string
 * @param len Length of Data to be written to the source-string
 * @return void
 **/
void dynncat(char *source, char *data, size_t len){
	size_t i = strlen(source)
	size_t b = 0;

	if(source == NULL || data == NULL) return;

	while(b <= len){
		source[i] = data[b];
		i++;
		b++;
	}
	source[i] = '\0';
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief dyncpy:  Copy characters from a string to a buffer, and
 *           add a null terminator after the last character copied.
 *
 * @param target Buffer to copy into
 * @param source Data to be copied
 * @return void
 **/
void dyncpy(char *target, char *source){
	size_t i = 0;

	if(source == NULL) return;

	while(source[i] != '\0'){
		target[i] = source[i];
		i++;
	}
	target[i++] = '\0';
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief dynncpy:  Copy characters from a string to a buffer, and
 *           add a null terminator after the last character copied.
 *
 * @param target Buffer to copy into
 * @param source Data to be copied
 * @param length the size of target (avoid overflow)
 * @return void
 **/
void dynncpy(char *target, char *source, size_t length){
	size_t i = 0;

	if(source == NULL) return;

	while(source[i] != '\0' && i<length-1){
		target[i] = source[i];
		i++;
	}
	target[i++] = '\0';
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief dyndup:  Copy characters from a string to a static buffer, and
 *           add a null terminator after the last character copied.
 *
 * @param source Data to be copied
 * @return The Pointer to the static buffer (string) (char*)
 **/
char* dyndup(char *source){
	static char copy[BUFFER];
	char *dup = NULL;
	size_t i = 0;

	if(source == NULL) return NULL;

	bzero(copy,sizeof(copy));
	while(source[i] != '\0'){
		copy[i] = source[i];
		i++;
	}
	
	copy[i++] = '\0';
	dup = copy;
	return dup;
}

/*  ************************************************************************************************  */

/**
 * @brief dynadd:  Works like sprintf(), but the buffer which is normally given by snprintf
 *           is a static buffer in this case, this function also
 *           adds a null terminator after the last character copied.
 *
 * @param text Data to be throughn together by using vsnprintf();
 * @return The Pointer to the static buffer (string) (char*)
 **/
char* dynadd(const char* text, ...){
	static char tmp[BUFFER + 1];
	char* string = NULL;

	if(text == NULL) return NULL;

	bzero(tmp,sizeof(tmp));
	
	va_list va;
	va_start(va, text);
	vsnprintf(tmp, sizeof(tmp)-1, text, va);
	va_end(va);
	
	string = tmp;
	return string;
}

/*  ************************************************************************************************  */

/**
 * @brief dynsplice:  Deletes a range of elements from a PointerArray (x to y)
 *           where x and y are deleted as well.
 *           Example: splice(a,0,2), will delete a[0] a[1] a[2]
 *           In this case a[0] is set to be the old a[3] (if it exists before)
 *
 * @param value PointerArray from which the parameters should be deleted
 * @param x First element to delete
 * @param y Last element to delete
 * @return void
 **/
void dynsplice(char *value[], size_t x, size_t y){
	size_t i = 0;
	size_t z = 1;

	if(value[0]==NULL) return;

	for(i = 0; value[i] != NULL; i++){
		while(value[x] != NULL){
			if      (value[y+z]==NULL)   value[x] = NULL;
			else if (value[y+z]!=NULL)	 value[x] = value[y+z];
			else                     	 value[x] = NULL;
			z++;
			x++;
			i++;
		}
		value[i] = NULL;
	}
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief dynunshift:  Adds an array element anywhere to the array
 *
 * @param value PointerArray to which we like to add an element
 * @param string The element we like to add
 * @param x The Place where we want to add out new element
 * @param size The maximum size in target string (check for: is it really possible to add the element?)
 * @return int On Success 1, on Error 0
 **/
bool dynunshift(char *value[], char *string, size_t x, size_t size){
	size_t i = 0;
	size_t j = 0;
	size_t last_value = 1;
	size_t overwrite = 0;
	size_t length = 0;

	if(value[0]==NULL)
		return 0;
	
	for(i = 0; value[i]; i++){
		length += dynlen(value[i])+1;
	}
	last_value = i;
	
	if(length+dynlen(string)+1<=size){
		value[last_value+1] = '\0';
		
		if(x>=last_value){
			overwrite 			= dynlen(value[last_value-1])+1;
			value[last_value] 	= &value[last_value-1][overwrite];
				
			for(i=0;string[i];i++,overwrite++)
				value[last_value-1][overwrite] = string[i];

			value[last_value-1][overwrite++] = '\0';
			return 1;
		}
		else{
			for(j = last_value-1; value[j]; j--){
				value[j+1] = value[j];
				if(j==x) break;
			}
			
			overwrite 			= dynlen(value[last_value])+1;
			value[x] 			= &value[last_value][overwrite];
				
			for(i=0;string[i];i++,overwrite++)
				value[last_value][overwrite] = string[i];

			value[last_value][overwrite++] = '\0';
			return 1;
		}
	}else return 0;
}

/*  ************************************************************************************************  */

/**
 * @brief reverse:  revereses a string "source" and saves the result in "target"
 *
 * @param target The buffer where the result should be stored
 * @param source The buffer to reverse
 * @return void
 **/
void reverse(char *target, char *source){
	size_t a = 0;
	size_t b = 0;

	if(source == NULL || target == NULL) return;

	a = dynlen(source)-1;
	while(a != 0){
		target[b] = source[a];
		b++;
		a--;
	}
	target[dynlen(source)-1] = source[0];
	target[dynlen(source)] = '\0';
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief join:  Joins arguments from an array to a string by using a character
 *           between the array parameters,
 *           adds a null terminator after the last character copied.
 *
 * @param buffer The buffer where the result should be stored
 * @param param The array where the arguments are read out
 * @param inputchar The character for spacefilling between the arguments
 * @return void
 **/
void join(char *buffer, char *param[], char inputchar){
	size_t i = 0;
	char *target = NULL;

	if(buffer == NULL || param[0] == NULL || inputchar == NULL) return;

	target = buffer;
	for(i = 0; param[i] != NULL; i++){
		target += sprintf(target, "%s%c", param[i], inputchar);
	}
	*(target-1) = '\0';
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief djoin:  Joins arguments from an array to a string by using a character
 *           between the array parameters,
 *           adds a null terminator after the last character copied.
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param param The array where the arguments are read out
 * @param inputchar The character for spacefilling between the arguments
 * @return Pointer to the static buffer
 **/
char* djoin(char inputchar, char *param[]){
	static char receive[BUFFER];
	size_t i = 1;
	size_t length = 0;
	char *djoin = NULL;

	if(param[0] == NULL || inputchar == NULL) return NULL;

	bzero(receive,sizeof(receive));
	receive[0] = '\0';
	
	dyncpy(receive, param[0]);
	for(i = 1; param[i] != NULL; i++){
		length += dynlen(param[i])+1;
		if(length<sizeof(receive)){
			straddnc(receive,inputchar,sizeof(receive));
			straddns(receive,param[i],sizeof(receive));
		}else break;
	}
	djoin = receive;
	return djoin;
}

/*  ************************************************************************************************  */

/**
 * @brief dstrjoin:  Joins arguments from an array to a string by using a character
 *               between the array parameters,
 *               adds a null terminator after the last character copied.
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param inputchar The character for spacefilling between the arguments
 * @param param The array where the arguments are read out
 * @param begin The arrayposition at which the join should begin
 * @return Pointer to the static buffer
 **/
char* dstrjoin(char inputchar, char *param[], size_t begin){
	static char receive[PUTSERVER_BUF_LEN];
	size_t i = 0;
	size_t length = 0;
	char *dstrjoin = NULL;

	if(param[0] == NULL || inputchar == '\0') return NULL;

	bzero(receive,sizeof(receive));
	receive[0] = '\0';
	dynncpy(receive,param[begin],sizeof(receive));
	for(i = begin+1; param[i] != NULL; i++){
		length += dynlen(param[i])+1;
		if(length<sizeof(receive)){
			straddnc(receive,inputchar,sizeof(receive));
			straddns(receive,param[i],sizeof(receive));
		}else break;
	}
	dstrjoin = receive;
	return dstrjoin;
}


/*  ************************************************************************************************  */

/**
 * @brief dstrjoin_det:  Joins arguments from an array to a string by using a character
 *               between the array parameters,
 *               adds a null terminator after the last character copied.
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param inputchar The character for spacefilling between the arguments
 * @param param The array where the arguments are read out
 * @param begin The arrayposition at which the join should begin
 * @return Pointer to the static buffer
 **/
char* dstrjoin_det(char inputchar, char *param[], size_t begin, size_t end){
	static char receive[PUTSERVER_BUF_LEN];
	size_t i = 0;
	size_t length = 0;
	char *dstrjoin = NULL;

	if(param[0] == NULL || inputchar == '\0') return NULL;

	bzero(receive,sizeof(receive));
	receive[0] = '\0';
	dynncpy(receive,param[begin],sizeof(receive));
	for(i = begin+1; param[i] != NULL && i<end; i++){
		length += dynlen(param[i])+1;
		if(length<sizeof(receive)){
			straddnc(receive,inputchar,sizeof(receive));
			straddns(receive,param[i],sizeof(receive));
		}else break;
	}
	dstrjoin = receive;
	return dstrjoin;
}

/*  ************************************************************************************************  */

#endif
