/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  Please read COPYING and README for further details.
 *  Based on the original code of Services by Andy Church. 
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 207                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-02-05 12:32:05 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file join.c
 *   @brief This is the Module file for JOIN (Server Side) Command.
 *
 *   It contains all functions which are neccessary to process and use 
 *   the JOIN command.
 *
 *   The JOIN File is used to process JOINS sent from Servers
 *   and to Servers. (Protocol-Process-File -> PPF)
 *   (Join functions for all clients and the server - manages the DB Status.)
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "global.h"
#include "services.h"
#include "modules.h"
#include "osocket.h"

/**
 * @brief do_sajoin:  Sends out a JOIN (depending on the Loaded Protocol) - using SAJOIN
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param fake The faketype (So we are able to send the on_join EVENT if it's not a fake)
 * @param snick The ServicesNick which sends the JOIN
 * @param nick The Nick we want to join to target channel
 * @param chan The Channel where we want to join the nick
 * @param date The Channels creation date
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL do_sajoin(Threads *thread, int fake, char *snick, char *nick, char *chan, time_t date){
	PROTOCOL_BOOL analyzer = CMD_ERROR;

	/* Let'z check out inputs - are they valid? */
	if(snick==NULL || nick==NULL || chan==NULL)
		return CMD_ERROR;

	/* Analyze Command - is it supported from the IRCd !? */
	if(chan[0]=='#' || chan[0]=='&') {
		if(orion_WRCtranslate_valid_chan(thread, chan) == false)
			return CMD_ERROR;
			
		if(orion_WRCtranslate_valid_nick(thread, nick) == false)
			return CMD_ERROR;
			
		if(GetUser(nick)==NULL)
			return CMD_ERROR;

		/* Analyze Command - is it supported from the IRCd !? */
		analyzer = orion_WRCtranslate_cmd_sajoin(thread,snick,nick,chan,date ? date : time(NULL));
		if(analyzer >= CMD_NOT_SUPPORTED)
			return analyzer;
			
		/* Command supported .. we start analyzing it */
		if(!IRC.DoesGlobalChangeAfterSA)
			_join(thread,fake,nick,chan,date ? date : time(NULL));
	}
	return analyzer;
}

/*  ************************************************************************************************  */

/**
 * @brief do_svsjoin:  Sends out a JOIN (depending on the Loaded Protocol) - using SVSJOIN
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param fake The faketype (So we are able to send the on_join EVENT if it's not a fake)
 * @param snick The ServicesNick which sends the JOIN
 * @param nick The Nick we want to join to target channel
 * @param chan The Channel where we want to join the nick
 * @param date The Channels creation date
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL do_svsjoin(Threads *thread, int fake, char *snick, char *nick, char *chan, time_t date){
	PROTOCOL_BOOL analyzer = CMD_ERROR;

	/* Let'z check out inputs - are they valid? */
	if(snick==NULL || nick==NULL || chan==NULL)
		return CMD_ERROR;

	/* Analyze Command - is it supported from the IRCd !? */
	if(chan[0]=='#' || chan[0]=='&'){
		if(orion_WRCtranslate_valid_chan(thread, chan) == false)
			return CMD_ERROR;
			
		if(orion_WRCtranslate_valid_nick(thread, nick) == false)
			return CMD_ERROR;
			
		if(GetUser(nick)==NULL)
			return CMD_ERROR;
		
		/* Analyze Command - is it supported from the IRCd !? */
		analyzer = orion_WRCtranslate_cmd_svsjoin(thread,snick,nick,chan,date ? date : time(NULL));
		if(analyzer >= CMD_NOT_SUPPORTED)
			return analyzer;
			
		/* Command supported .. we start analyzing it */
		if(!IRC.DoesGlobalChangeAfterSVS)
			_join(thread,fake,nick,chan,date ? date : time(NULL));
	}
	return analyzer;
}

/*  ************************************************************************************************  */

/**
 * @brief do_services_join:  Sends out an SJOIN (depending on the Loaded Protocol) - ONLY FOR SERVICES!!
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param snick The ServicesNick which sends the SJOIN and receives the channel modes
 * @param channel The Channel which should be SJOIN'ed
 * @param time_in The Channels creation date
 * @param fake The faketype (So we are able to send the on_join EVENT if it's not a fake)
 * @return void
 **/
PROTOCOL_BOOL do_services_join(Threads *thread, char *snick, char *channel, time_t time_in, int fake){
    Users *user = GetUser(snick);

	/* Let'z check out inputs - are they valid? */
	if(snick==NULL || channel==NULL)
		return CMD_ERROR;

	if(orion_WRCtranslate_valid_chan(thread, channel) == false)
		return CMD_ERROR;
			
	if(orion_WRCtranslate_valid_nick(thread, snick) == false)
		return CMD_ERROR;

	if(channel[0]=='#' || channel[0]=='&'){
		if(user==NULL)
		    return CMD_ERROR;
		    
		/* Add the user it's new channel */
		addUserChannel(user, channel);
		    
		return orion_WRCtranslate_cmd_services_join(thread,snick,channel,time_in,fake);
	}
		
	return CMD_ERROR;
}

/*  ************************************************************************************************  */

/**
 * @brief do_join:  Sends out a JOIN (depending on the Loaded Protocol) - using JOIN
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param fake The faketype (So we are able to send the on_join EVENT if it's not a fake)
 * @param nick The ServicesNick which sends the JOIN
 * @param channel The Channel the ServicesNick likes to JOIN
 * @param date The Channels creation date
 * @return void
 **/
PROTOCOL_BOOL do_join(Threads *thread, int fake, char *nick, char *channel, time_t date){
	PROTOCOL_BOOL analyzer = CMD_ERROR;
	
/* JOIN Function */
	if(orion_WRCtranslate_valid_chan(thread, channel) == false)
		return CMD_ERROR;
			
	if(orion_WRCtranslate_valid_nick(thread, nick) == false)
		return CMD_ERROR;
		
	if(GetUser(nick)==NULL)
		return CMD_ERROR;

	analyzer = orion_WRCtranslate_cmd_join(thread,nick, channel, date ? date : time(NULL));
	if(analyzer >= CMD_NOT_SUPPORTED)
		return analyzer;
		
	_join(thread,fake>0 ? fake : 4,nick,channel,date ? date : time(NULL));
	
	return analyzer;
}

/*  ************************************************************************************************  */

/**
 * @brief _join: Processes the X-JOIN-Statements for adding Information to SQL (NO OUTPUT!)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param fake The faketype (So we are able to send the on_join EVENT if it's not a fake)
 * @param nick The Nick that joins a channel
 * @param chanbuf Holds the Join Information (possibly seperated by ',')
 * @param time_in The Channels creation date
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL _join(Threads *thread, int fake, char *nick, char *chanbuf, time_t time_in){
	//:O SVSJOIN shock #channel
	//:shock JOIN #channel
	char receive[BUFSIZE];
	char *channel[ARRAY_SIZE];
	char *eventarray[3];
	char odate[50];
	
	size_t channels = 0;
	size_t i        = 0;
	
	bzero(&receive,sizeof(receive));
	bzero(&odate,sizeof(odate));
	
	dynncpy(receive,chanbuf,sizeof(receive));
	channels = strsplit(receive,channel,',');
	//receive -> *bits, receive2 -> *param
	// BEGIN on_join
	//
	
	MySQL_FlushInserts(thread,SQL_BUFFER_CONDATA,SQL_BUFFER_CONDATA_NO_MODES);
	
	for(i=0;i<channels && !digitcol(channel[i]);i++){
		Users *user = GetUser(nick);
		if(user!=NULL){
			/* Updates the Threads Channel Management */
			UpdateChannelInformation3(thread,channel[i],NULL);
			
			if(!thread->c.chan.fields && thread->c.chan.name[0]=='\0'){
				dynncpy(thread->c.chan.name,channel[i],sizeof(thread->c.chan.name));
				thread->c.chan.datecreate = time(NULL);
				domysql(thread,"INSERT IGNORE INTO `orion_cs_core` (`datecreate`,`channel`) VALUES ('%ld','%s')",thread->c.chan.datecreate,mysql_quote(thread,thread->c.chan.name));
			}
			
			/* Add the user it's new channel */
			domysql(thread,"INSERT INTO `orion_chan_users` (`nick`,`channel`,`faketype`) VALUES ('%s','%s','%d') ON DUPLICATE KEY UPDATE `nick`=VALUES(`nick`),`channel`=VALUES(`channel`)",mysql_quote(thread,nick),mysql_quote(thread,channel[i]),fake);
			addUserChannel(user, channel[i]);
			
			thread->c.chan.countusers++;
		}
		
		if(!fake){
			bzero(eventarray,sizeof(eventarray));
			bzero(&odate,sizeof(odate));
			sprintf(odate,"%ld",time_in);
			
			eventarray[0] = nick;
			eventarray[1] = channel[i];
			eventarray[2] = odate;
			
			/**************************** SEND on_join to the eventhandler ****************************/
			_ExecuteEvent(nick,eventarray,"on_join");
			/******************************************************************************************/
		}
	}
	return CMD_EXECUTED;
}

/*  ************************************************************************************************  */
