/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  Please read COPYING and README for further details.
 *  Based on the original code of Services by Andy Church. 
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 207                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-02-05 12:32:05 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file kick.c
 *   @brief This is the Module file for KICK (Server Side) Command.
 *
 *   It contains all functions which are neccessary to process and use 
 *   the KICK command.
 *
 *   The KICK File is used to process KICKS sent from Servers
 *   and to Servers. (Protocol-Process-File -> PPF)
 *   (Kick functions for all clients and the server - manages the DB Status.)
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "global.h"
#include "services.h"
#include "modules.h"
#include "osocket.h"

/**
 * @brief do_BSkick:  Sends out a KICK (depending on the Loaded Protocol) - using SAJOIN
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param nick The (Services)Nick which should execute the Kick
 * @param channel The Channel where we want to kick the nick (knick)
 * @param knick The Nick we want to kick
 * @param reason The Reason why we kick that user
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL do_BSkick(Threads *thread, char *nick, char *channel, char *knick, const char *reason, ...){
	PROTOCOL_BOOL analyzer = CMD_ERROR;
	Users *user 	= GetUser(knick);
	
	/* Let'z check out inputs - are they valid? */
	if(nick==NULL || channel==NULL || knick==NULL || reason==NULL)
		return CMD_ERROR;
		
	if(user==NULL)
		return CMD_ERROR;
		
	/* Declare Variables */
	va_list va;
	char tmp[BUFFER+1];
	
	/* Initialise Variables to (NULL) */	
	bzero(&tmp,sizeof(tmp));

	/* Join reason string */
	va_start(va, reason);
	vsnprintf(tmp, sizeof(tmp)-1, reason, va);
	va_end(va);

	/* Analyze Command - is it supported from the IRCd !? */
	analyzer = orion_WRCtranslate_cmd_kick(thread, nick, channel, knick, tmp);
	if(analyzer >= CMD_NOT_SUPPORTED)
		return analyzer;

	/* part channel - change user channel data (we keep data but set that the user isn't in that chan anymore) */
    PartChannel(user,channel);

	/* Command supported .. we start analyzing it */
	kick(thread,knick,channel);
	return analyzer;
}

/*  ************************************************************************************************  */

/**
 * @brief do_kick:  Sends out a KICK (depending on the Loaded Protocol) - using KICK
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param nick The (Services)Nick which should execute the Kick
 * @param channel The Channel where we want to kick the nick (knick)
 * @param knick The Nick we want to kick
 * @param reason The Reason why we kick that user
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL do_kick(Threads *thread, char *nick, char *channel, char *knick, const char *reason, ...){
	PROTOCOL_BOOL analyzer = CMD_ERROR;
	Users *user 	= GetUser(knick);
	
	/* Let'z check out inputs - are they valid? */
	if(nick==NULL || channel==NULL || knick==NULL || reason==NULL)
		return CMD_ERROR;

	if(user==NULL)
		return CMD_ERROR;

	/* Declare Variables */
	va_list va;
	char tmp[BUFFER+1];
	
	/* Initialise Variables to (NULL) */	
	bzero(&tmp,sizeof(tmp));

	/* Join reason string */
	va_start(va, reason);
	vsnprintf(tmp, sizeof(tmp)-1, reason, va);
	va_end(va);
	
	/* Analyze Command - is it supported from the IRCd !? */
	analyzer = orion_WRCtranslate_cmd_kick(thread, nick, channel, knick, tmp);
	if(analyzer >= CMD_NOT_SUPPORTED)
		return analyzer;

	/* part channel - change user channel data (we keep data but set that the user isn't in that chan anymore) */
    PartChannel(user,channel);

	/* Command supported .. we start analyzing it */
	kick(thread,knick,channel);
	return analyzer;
}

/*  ************************************************************************************************  */

/**
 * @brief do_kick_nopart:  Sends out a KICK (depending on the Loaded Protocol),
 *                       using KICK (THE BOT DOES NOT PART IN THIS FUNCTION!)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param nick The (Services)Nick which should execute the Kick
 * @param channel The Channel where we want to kick the nick (knick)
 * @param knick The Nick we want to kick
 * @param reason The Reason why we kick that user
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL do_kick_nopart(Threads *thread, char *nick, char *channel, char *knick, const char *reason, ...){
	PROTOCOL_BOOL analyzer = CMD_ERROR;
	Users *user 	= GetUser(knick);
	
	/* Let'z check out inputs - are they valid? */
	if(nick==NULL || channel==NULL || knick==NULL || reason==NULL)
		return CMD_ERROR;
		
	if(user==NULL)
		return CMD_ERROR;
	
	/* Declare Variables */
	va_list va;
	char tmp[BUFFER+1];
	
	/* Initialise Variables to (NULL) */
	bzero(&tmp,sizeof(tmp));

	/* Join reason string */
	va_start(va, reason);
	vsnprintf(tmp, sizeof(tmp)-1, reason, va);
	va_end(va);

	/* Analyze Command - is it supported from the IRCd !? */
	analyzer = orion_WRCtranslate_cmd_kick(thread, nick, channel, knick, tmp);
	if(analyzer >= CMD_NOT_SUPPORTED)
		return analyzer;

	/* Command supported .. we go on running it */
	/* part channel - change user channel data (we keep data but set that the user isn't in that chan anymore) */
    PartChannel(user,channel);
	
	/* Updates the Threads Channel Management */
	UpdateChannelInformation3(thread,channel,NULL);
	
	domysql(thread,"DELETE FROM `orion_chan_users` WHERE `nick`='%s' AND `channel`='%s'",mysql_quote(thread,nick),mysql_quote(thread,channel));
	thread->c.chan.countusers--;
	
	if(thread->c.chan.countusers==0){
		if(thread->c.chan.serv[0]!='\0'){	// channel empty, but registered by a bot (Q/L/ChanServ/etc...)
		
			/* Update memory for caching */
#ifndef USE_THREADS
			thread->c.chan.datecreate 	= (time_t)0;
			thread->c.chan.limit 		= 0;
			bzero(&thread->c.chan.mode,sizeof(thread->c.chan.mode));
			bzero(&thread->c.chan.link,sizeof(thread->c.chan.link));
			bzero(&thread->c.chan.flood,sizeof(thread->c.chan.flood));
			bzero(&thread->c.chan.key,sizeof(thread->c.chan.key));
#endif
			
			if((streqnc(thread->c.chan.serv,"q") && !strcor(thread->c.chan.flags,"t")) || (streqnc(thread->c.chan.serv,"chanserv") && ~thread->c.chan.settings&CS_KEEPTOPIC_BIT))
				bzero(&thread->c.chan.last_topic,sizeof(thread->c.chan.last_topic));
		
			/* Update SQL as live update */
			if((streqnc(thread->c.chan.serv,"q") && strcor(thread->c.chan.flags,"t") && ~thread->c.chan.settings&CS_SUSPEND_BIT) || (streqnc(thread->c.chan.serv,"chanserv") && thread->c.chan.settings&CS_KEEPTOPIC_BIT && ~thread->c.chan.settings&CS_SUSPEND_BIT)){
				domysql(thread,"UPDATE `orion_cs_core` SET "
				"`datecreate`='',`mode`='',`link`='',`limit`='',`flood`='',`key`=''"
				" WHERE `channel`='%s'",mysql_quote(thread,channel));
			}else{
				domysql(thread,"UPDATE `orion_cs_core` SET "
				"`datecreate`='',`mode`='',`link`='',`limit`='',`flood`='',`key`='',`last_topic`=''"
				" WHERE `channel`='%s'",mysql_quote(thread,channel));
			}
				
			domysql(thread,"DELETE `orion_chan_bans`,`orion_chan_expbans` FROM `orion_chan_bans` LEFT OUTER JOIN `orion_chan_expbans` ON `orion_chan_expbans`.`channel`=`orion_chan_bans`.`channel` WHERE `orion_chan_bans`.`channel`='%s'",mysql_quote(thread,channel));
		}else{	// channel empty - no bots -> delete it...
			/* Update memory for caching */
#ifndef USE_THREADS
			bzero(&thread->c,sizeof(thread->c));
#endif
			
			/* Update SQL as live update */
			domysql(thread,"DELETE FROM `orion_cs_core` WHERE `channel`='%s'",mysql_quote(thread,channel));
		}
	}
	
	else if(thread->c.chan.fields && !streqnc(nick,Q->nick) && thread->c.chan.name && strcor("c",thread->c.chan.flags) && streqnc(thread->c.chan.serv,"q"))
		addLimit(thread, channel, thread->c.chan.datecreate, 1, thread->c.chan.serv, thread->c.chan.alimit);

	return analyzer;
}

/*  ************************************************************************************************  */

/**
 * @brief on_kick:  Receives a KICK (depending on the Loaded Protocol)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param sender The Sender who sent that kick execution
 * @param nick The Nick which was kicked from Channel
 * @param channel The Channel where that nick was kicked
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL on_kick(Threads *thread, char *sender, char *nick, char *channel){
	Users *user 	= GetUser(nick);

	/* Let'z check out inputs - are they valid? */
	if(user==NULL)
		return CMD_ERROR;

	/* Declare variables */
	char *eventarray[4];
	
	/* Processing the Command .. */
	/* part channel - change user channel data (we keep data but set that the user isn't in that chan anymore) */
    PartChannel(user,channel);
		
	kick(thread,nick,channel);
	
	bzero(eventarray,sizeof(eventarray));
	eventarray[0] = sender;
	eventarray[1] = channel;
	eventarray[2] = nick;
	eventarray[3] = NULL;
	/**************************** SEND kick to the eventhandler ****************************/
	_ExecuteEvent(sender,eventarray,"on_kick");
	/***************************************************************************************/
	return CMD_EXECUTED;
}

/*  ************************************************************************************************  */

/**
 * @brief kick: Processes the X-KICK-Statements for deleting Information from SQL (NO OUTPUT!)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param nick The Nick that got kicked before
 * @param channel The Channel in which it happened
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL kick(Threads *thread, char *nick, char *channel){
	
	/* Updates the Threads Channel Management */
	UpdateChannelInformation3(thread,channel,NULL);

	domysql(thread,"DELETE FROM `orion_chan_users` WHERE `nick`='%s' AND `channel`='%s'",mysql_quote(thread,nick),mysql_quote(thread,channel));
	thread->c.chan.countusers--;
	
	if(thread->c.chan.countusers==0){
		if(thread->c.chan.serv[0]!='\0'){	// channel empty, but registered by a bot (Q/L/ChanServ/etc...)
		
			/* Update memory for caching */
#ifndef USE_THREADS
			thread->c.chan.datecreate 	= (time_t)0;
			thread->c.chan.limit 		= 0;
			bzero(&thread->c.chan.mode,sizeof(thread->c.chan.mode));
			bzero(&thread->c.chan.link,sizeof(thread->c.chan.link));
			bzero(&thread->c.chan.flood,sizeof(thread->c.chan.flood));
			bzero(&thread->c.chan.key,sizeof(thread->c.chan.key));
#endif
			
			if((streqnc(thread->c.chan.serv,"q") && !strcor(thread->c.chan.flags,"t")) || (streqnc(thread->c.chan.serv,"chanserv") && ~thread->c.chan.settings&CS_KEEPTOPIC_BIT))
				bzero(&thread->c.chan.last_topic,sizeof(thread->c.chan.last_topic));
		
			/* Update SQL as live update */
			if((streqnc(thread->c.chan.serv,"q") && strcor(thread->c.chan.flags,"t") && ~thread->c.chan.settings&CS_SUSPEND_BIT) || (streqnc(thread->c.chan.serv,"chanserv") && thread->c.chan.settings&CS_KEEPTOPIC_BIT && ~thread->c.chan.settings&CS_SUSPEND_BIT)){
				domysql(thread,"UPDATE `orion_cs_core` SET "
				"`datecreate`='',`mode`='',`link`='',`limit`='',`flood`='',`key`=''"
				" WHERE `channel`='%s'",mysql_quote(thread,channel));
			}else{
				domysql(thread,"UPDATE `orion_cs_core` SET "
				"`datecreate`='',`mode`='',`link`='',`limit`='',`flood`='',`key`='',`last_topic`=''"
				" WHERE `channel`='%s'",mysql_quote(thread,channel));
			}
				
			domysql(thread,"DELETE `orion_chan_bans`,`orion_chan_expbans` FROM `orion_chan_bans` LEFT OUTER JOIN `orion_chan_expbans` ON `orion_chan_expbans`.`channel`=`orion_chan_bans`.`channel` WHERE `orion_chan_bans`.`channel`='%s'",mysql_quote(thread,channel));
		}else{	// channel empty - no bots -> delete it...
			/* Update memory for caching */
#ifndef USE_THREADS
			bzero(&thread->c,sizeof(thread->c));
#endif
			
			/* Update SQL as live update */
			domysql(thread,"DELETE FROM `orion_cs_core` WHERE `channel`='%s'",mysql_quote(thread,channel));
		}
	}
	else if(((~thread->c.chan.settings&CS_INHABIT_BIT && thread->c.chan.countusers==1) || (thread->c.chan.settings&CS_INHABIT_BIT && thread->c.chan.countusers==2)) && thread->c.chan.bot[0]!='\0' && !streqnc(nick,Q->nick) && !streqnc(nick,cs->nick) && bs->loaded)
		do_part(thread,thread->c.chan.bot,channel,NULL);
	
	else if(thread->c.chan.fields && !streqnc(nick,Q->nick) && thread->c.chan.name && strcor("c",thread->c.chan.flags) && streqnc(thread->c.chan.serv,"q"))
		addLimit(thread, channel, thread->c.chan.datecreate, 1, thread->c.chan.serv, thread->c.chan.alimit);

	return CMD_EXECUTED;
}

/*  ************************************************************************************************  */
