/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  Please read COPYING and README for further details.
 *  Based on the original code of Services by Andy Church. 
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 207                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-02-05 12:32:05 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file kill.c
 *   @brief This is the Module file for KILL (Server Side) Command.
 *
 *   It contains all functions which are neccessary to process and use 
 *   the KILL command.
 *
 *   The KILL File is used to process KILLS sent from Servers
 *   and to Servers. (Protocol-Process-File -> PPF)
 *   (Kill functions for all clients and the server - manages the DB Status.)
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "global.h"
#include "services.h"
#include "modules.h"
#include "osocket.h"

/**
 * @brief do_svskill:  Sends out a KILL (depending on the Loaded Protocol) - using SVSKILL
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param nick The Nick that sends the SVS-kill
 * @param target The Nick which we want to SVS-kill
 * @param message The Kill-message we want to provide to the public
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL do_svskill(Threads *thread, char *nick, char *target, const char *message, ...){
	PROTOCOL_BOOL analyzer = CMD_ERROR;
	
	/* Let'z check out inputs - are they valid? */
	if(nick==NULL || target==NULL || message==NULL)
		return CMD_ERROR;
			
	if(orion_WRCtranslate_valid_nick(thread, target) == false)
		return CMD_ERROR;
		
	if(GetUser(target)==NULL)
		return CMD_ERROR;
		
	/* Deklarate Variables */
	va_list va;
	char tmp[BUFFER+1];
	
	/* Initialise Variables to (NULL) */	
	bzero(&tmp,sizeof(tmp));

	/* Join reason string */
	va_start	(va,  message);
	vsnprintf	(tmp, sizeof(tmp)-1, message, va);
	va_end		(va);
	
	/* Analyze Command - is it supported from the IRCd !? */
	analyzer = orion_WRCtranslate_cmd_svskill(thread,nick,target,tmp);
	if(analyzer >= CMD_NOT_SUPPORTED)
		return analyzer;
		
	/* Command supported .. we start analyzing it */
	if(!IRC.DoesGlobalChangeAfterSVS || !IRC.DoesQuitAfterKill)
		_kill(thread,target);

	return analyzer;
}

/*  ************************************************************************************************  */

/**
 * @brief do_kill:  Sends out a KILL (depending on the Loaded Protocol) - using KILL
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param nick The Nick that sends the KILL
 * @param target The Nick which we want to KILL
 * @param message The Kill-message we want to provide to the public
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL do_kill(Threads *thread, char *nick, char *target, const char *message, ...){
	PROTOCOL_BOOL analyzer = CMD_ERROR;
	
	/* Let'z check out inputs - are they valid? */
	if(nick==NULL || target==NULL || message==NULL)
		return CMD_ERROR;
		
	if(orion_WRCtranslate_valid_nick(thread, target) == false)
		return CMD_ERROR;
		
	if(GetUser(target)==NULL)
		return CMD_ERROR;
		
	/* Deklarate Variables */
	va_list va;
	char tmp[BUFFER+1];
	
	/* Initialise Variables to (NULL) */	
	bzero(&tmp,sizeof(tmp));

	/* Join reason string */
	va_start	(va,  message);
	vsnprintf	(tmp, sizeof(tmp)-1, message, va);
	va_end		(va);
	
	/* Analyze Command - is it supported from the IRCd !? */
	analyzer = orion_WRCtranslate_cmd_kill(thread,nick,target,tmp);
	if(analyzer >= CMD_NOT_SUPPORTED)
		return analyzer;
		
	/* Command supported .. we start analyzing it */
	if(!IRC.DoesQuitAfterKill)
		_kill(thread,target);

	return analyzer;
}

/*  ************************************************************************************************  */

/**
 * @brief _kill: Processes the X-KILL-Statements for deleting Information from SQL (NO OUTPUT!)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param nick The Nick that got killed before
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL _kill(Threads *thread, char *nick){
	PROTOCOL_BOOL analyzer = CMD_ERROR;
	
	/* Global variables */
	deepSQL channels;
	char chan_update[(MAX_CHANNELNAME_LEN+3)*MAX_CHANS];
	char chan_delete[(MAX_CHANNELNAME_LEN+3)*MAX_CHANS];
	char chan_bot   [(MAX_CHANNELNAME_LEN+3)*MAX_CHANS];
	int i = 0;

	/* Initialise Variables to (NULL) */
	bzero(&channels,sizeof(channels));
	bzero(&chan_bot,sizeof(chan_bot));
	bzero(&chan_update,sizeof(chan_update));
	bzero(&chan_delete,sizeof(chan_delete));
	
	if(nick==NULL)
		return CMD_ERROR;
		
	if(DuringServerlink && m->UseWhowas)
		domysql(thread,"DELETE `orion_core_condata`,`orion_core_condata_tmp`,`orion_chan_users`,`orion_ns_aliases` FROM `orion_core_condata` LEFT OUTER JOIN `orion_core_condata_tmp` ON `orion_core_condata_tmp`.`nick`=`orion_core_condata`.`nick` LEFT OUTER JOIN `orion_chan_users` ON `orion_chan_users`.`nick`=`orion_core_condata`.`nick` LEFT OUTER JOIN `orion_ns_aliases` ON (`orion_ns_aliases`.`nick`=`orion_core_condata`.`nick` AND `orion_ns_aliases`.`linked`=1) WHERE `orion_core_condata`.`nick`='%s'",mysql_quote(thread,nick));
	else
		domysql(thread,"DELETE `orion_core_condata`,`orion_chan_users`,`orion_ns_aliases` FROM `orion_core_condata` LEFT OUTER JOIN `orion_chan_users` ON `orion_chan_users`.`nick`=`orion_core_condata`.`nick` LEFT OUTER JOIN `orion_ns_aliases`   ON (`orion_ns_aliases`.`nick`=`orion_core_condata`.`nick` AND `orion_ns_aliases`.`linked`=1) WHERE `orion_core_condata`.`nick`='%s'",mysql_quote(thread,nick));
	
	domysql(thread,"UPDATE `orion_ns_aliases` SET `active`='0' WHERE `nick`='%s' AND `linked`='0'",mysql_quote(thread,nick)); //perm-linked & logout
	/** CASCADE (MySQL - InnoDB) does the other statements, because of referencing - chanusers, chanop, ... **/

	DelUserNick(nick);
	
	/* 	Update our Cache	*/
	if(streqnc(thread->c.Cdata.nick,nick)){
		bzero(&thread->c.Cdata,sizeof(thread->c.Cdata));
		thread->c.aliases.active 	= 0;
		
		unknown_access(thread->c.chan.access.targetlevel,sizeof(thread->c.chan.access.targetlevel));
	}else if(streqnc(thread->sql.Cdata.nick,nick)){
		bzero(&thread->sql.Cdata,sizeof(thread->sql.Cdata));
		thread->sql.aliases.active 	= 0;
		
		unknown_access(thread->c.chan.access.ownlevel,sizeof(thread->c.chan.access.ownlevel));
	}
	/**			***			**/
	
	channels.res    = doSelectMySQL(thread,"SELECT `orion_cs_core`.`serv`,`orion_cs_core`.`channel`,`orion_cs_core`.`botnick`, SUM(`orion_chan_users`.`nick`=`orion_cs_core`.`botnick`) as `botinchan`, COUNT(`orion_chan_users`.`nick`) as `count_users`,`orion_cs_core`.`settings` FROM `orion_cs_core` LEFT JOIN `orion_chan_users` ON (`orion_cs_core`.`channel` = `orion_chan_users`.`channel`) WHERE `orion_cs_core`.`datecreate`!='' GROUP BY `orion_cs_core`.`channel`");
	channels.fields = mysql_num_rows(channels.res);
		
	for(i=0;i<channels.fields;i++){
		channels.row = mysql_fetch_row(channels.res);
		int settings = stoi(channels.row[5]);

		if(
			(
				( ~settings&CS_INHABIT_BIT && stoi(channels.row[4])==1 && stoi(channels.row[3])==1 ) ||
				( settings&CS_INHABIT_BIT && stoi(channels.row[4])==2 && stoi(channels.row[3])==1)
			) ||
			stoi(channels.row[4])==0
		){
			if(streqnc(channels.row[1],thread->c.chan.name))
				thread->c.chan.countusers--;
			
			if(channels.row[0][0]!='\0'){	// channel empty, but registered by a bot (Q/L/ChanServ/etc...)
			
				straddnc(chan_update,toascii(39),sizeof(chan_update));
				straddns(chan_update,mysql_quote(thread,channels.row[1]),sizeof(chan_update));
				straddnc(chan_update,toascii(39),sizeof(chan_update));
				straddnc(chan_update,',',sizeof(chan_update));
			
				if(channels.row[2]!=NULL && stoi(channels.row[3])==1 && bs->loaded){
					analyzer = orion_WRCtranslate_cmd_part(thread,channels.row[2],channels.row[1],NULL);
					
					straddnc(chan_bot,toascii(39),sizeof(chan_bot));
					straddns(chan_bot,mysql_quote(thread,channels.row[1]),sizeof(chan_bot));
					straddnc(chan_bot,toascii(39),sizeof(chan_bot));
					straddnc(chan_bot,',',sizeof(chan_bot));
					
				}
	
			}else{
				straddnc(chan_delete,toascii(39),sizeof(chan_delete));
				straddns(chan_delete,mysql_quote(thread,channels.row[1]),sizeof(chan_delete));
				straddnc(chan_delete,toascii(39),sizeof(chan_delete));
				straddnc(chan_delete,',',sizeof(chan_delete));
			}
		}
	}
	
	/** Update MySQL (Update channels which are empty & registered, Delete all bans (cauze channel is empty), 
		Delete and the Bot (it was parted in the loop), Delete unregistered Channels that are empty **/
	if(chan_update[0]!='\0'){
		chan_update[dynlen(chan_update)-1]='\0';
		
		domysql(thread,"UPDATE `orion_cs_core` SET "
			"`datecreate`='',`mode`='',`link`='',`limit`='',`flood`='',`key`=''"
			" WHERE `channel` IN(%s)",chan_update);
		
		domysql(thread,"DELETE `orion_chan_bans`,`orion_chan_expbans` FROM `orion_chan_bans` LEFT OUTER JOIN `orion_chan_expbans` ON `orion_chan_expbans`.`channel`=`orion_chan_bans`.`channel` WHERE `orion_chan_bans`.`channel` IN(%s)",chan_update);
	}
	
	if(chan_delete[0]!='\0'){
		chan_delete[dynlen(chan_delete)-1]='\0';
		domysql(thread,"DELETE FROM `orion_cs_core` WHERE `channel`='%s'",chan_delete);
	}
	
	if(chan_bot[0]!='\0'){
		chan_bot[dynlen(chan_bot)-1]='\0';
		domysql(thread,"DELETE `orion_chan_users` FROM `orion_chan_users` LEFT OUTER JOIN `orion_cs_core` ON (`orion_cs_core`.`botnick`=`orion_chan_users`.`nick` AND `orion_cs_core`.`channel`=`orion_chan_users`.`channel`) WHERE `orion_cs_core`.`botnick` IS NOT NULL AND `orion_chan_users`.`channel` IN(%s)",chan_bot);
	}
	
	mysql_free_result_myr(channels.res); //free result

	/* 
	InnoDB: using ON UPDATE/DELETE CASCADE -> Update/Delete not neccessary anymore
	NOTE: mode system changed, old tables not neccessary anylonger...
	*/
	return CMD_EXECUTED;
}

/*  ************************************************************************************************  */
