/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  Please read COPYING and README for further details.
 *  Based on the original code of Services by Andy Church. 
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 194                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-16 18:33:19 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file limits.c
 *   @brief This is the Module file for handling/performing Limits.
 *
 *   It contains all functions which are neccessary to
 *   handle Limits. (Channel limits - is used for autolimit controlling)
 *
 *	 Limit structs and Limitmanagement (for autolimit performing)
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "global.h"
#include "services.h"
#include "modules.h"
#include "osocket.h"

static Limits *limits = NULL;

/**
 * @brief addLimit:  Adds a Limit to the list
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param channel The Channel which's Limit we should keep up to date
 * @param date The Channel's creation date
 * @param flag it 0 - No Autolimit for that channel, 1 - It's a Autolimit channel
 * @param serv The Services holding the Channel
 * @param alimit The Limit we should set (alimit+users_on_channel) == Autolimit
 * @return void
 **/
void addLimit(Threads *thread, char *channel, time_t date, bool flag, char* serv, long alimit)
{
	Limits *limit = NULL;
	
#ifdef USE_THREADS
	my_pthread_mutex_lock(&limit_lock);
#endif
	limit = limits;
	
	for (; limit; limit = limit->next) {
		if(streqnc(limit->channel,channel)){
#ifdef USE_THREADS
			my_pthread_mutex_unlock(&limit_lock);
#endif
			return;
		}
	}
	
	limit = (Limits*)malloc(sizeof(Limits));
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&limit_lock);
#endif
	bzero(limit,sizeof(Limits));

	dynncpy(limit->channel,channel,sizeof(limit->channel));
	limit->date=date;
	dynncpy(limit->serv,serv,sizeof(limit->serv));
	
	limit->flag   = flag;
	limit->alimit = alimit;

#ifdef USE_THREADS
	my_pthread_mutex_lock(&limit_lock);
#endif
	limit->next = limits;
	limit->prev = NULL;
	if (limits)
		limits->prev = limit;
	limits = limit;
	
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&limit_lock);
#endif
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief delLimit:  Deletes a Limit-Updater for a Channel
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param limit The Limit-struct, the one with the Channel where we want to drop the Autolimit
 * @return int Returns 1 if limit was deleted and 0 if there was an error in deleting
 **/
bool delLimit(Threads *thread, Limits *limit)
{
#ifdef USE_THREADS
	my_pthread_mutex_lock(&limit_lock);
#endif
	Limits *ptr = NULL;

	for (ptr = limits; ptr; ptr = ptr->next) {
		if (ptr == limit)
			break;
	}
	if (!ptr){
#ifdef USE_THREADS
		my_pthread_mutex_unlock(&limit_lock);
#endif
		return 0;
	}
	
	if (limit->prev)
		limit->prev->next = limit->next;
	else
		limits = limit->next;
	if (limit->next)
		limit->next->prev = limit->prev;
	free(limit);
	
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&limit_lock);
#endif
	return 1;
}

/*  ************************************************************************************************  */

/**
 * @brief PerformLimits:  Performs all Limits (updates them for all AUTOLIMIT Channels)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return void
 **/
void PerformLimits(Threads *thread){
	Limits *limit = NULL;
	long nlimit    = 0;
	
#ifdef USE_THREADS
	my_pthread_mutex_lock(&limit_lock);
#endif
	for (limit = limits; limit; limit = limit->next) {
		if(limit->flag && streqnc(limit->serv,"q")){
			nlimit  = stol(mysql_sel_var(thread,"SELECT count(*) FROM `orion_chan_users` WHERE `channel`='%s'",mysql_quote(thread,limit->channel)));
			nlimit += limit->alimit;
			do_mode(thread,Q->nick,limit->channel,limit->date,_dynadd("+%c %ld",IRC.LimitCMode,nlimit));
		}
	}
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&limit_lock);
#endif
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief delLimits:  Deletes all Limits from the List
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return void
 **/
void delLimits(Threads *thread){
	Limits *limit = NULL;
	
#ifdef USE_THREADS
	my_pthread_mutex_lock(&limit_lock);
#endif
	for (limit = limits; limit; limit = limit->next) {
		if(!delLimit(thread,limit)) break;
	}
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&limit_lock);
#endif
	return;
}

/*  ************************************************************************************************  */
