/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  Please read COPYING and README for further details.
 *  Based on the original code of Services by Andy Church. 
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 194                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-16 18:33:19 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file mail.c
 *   @brief This is the Module file for sending email.
 *
 *   It contains all functions which are neccessary to
 *   send emails. (ONLY SEND - NO RECEIVING)
 *
 *	 Mail Functions (using sendmail!)
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "global.h"
#include "services.h"
#include "modules.h"

/**
 * @brief o_email:  Opens a pipe to sendmail and sends an email with the given parameters
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param sender The email address / Name from the sender (services@your-irc.net)
 * @param recipient Name of the receiver - the nick in most cases
 * @param email_address email-address (the target where the mail should be sent)
 * @param subject Subject for the email
 * @return MailInfo* (makes the pipe useable anywhere)
 **/
MailInfo *o_email(Threads *thread, char *sender,char *recipient,char *email_address,char *subject){
	static MailInfo mail;
	mail.sender=sender;
	mail.recipient=recipient;
	mail.email=email_address;
	mail.subject=subject;

#ifdef USE_THREADS
	my_pthread_mutex_lock(&mail_lock);
	my_pthread_mutex_lock(&globvar_lock);
#endif
	if (!(mail.pipe = popen(_dynadd("%s %s",m->sendmailpath,mail.email), "w"))) {
#ifdef USE_THREADS
		my_pthread_mutex_unlock(&globvar_lock);
		my_pthread_mutex_unlock(&mail_lock);
#endif
		return NULL;
	}
	//fprintf(mail.pipe, "MIME-Version: 1.0\n");
	//fprintf(mail.pipe, "Content-type: text/html; charset=utf-8\n");
	fprintf(mail.pipe, "From: %s\n", mail.sender);
	if (m->DontQuoteAddresses) {
		fprintf(mail.pipe, "To: %s <%s>\n", mail.recipient, mail.email);
	}else{
		fprintf(mail.pipe, "To: \"%s\" <%s>\n", mail.recipient,mail.email);
	}
	fprintf(mail.pipe, "Subject: %s\n",mail.subject);
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&mail_lock);
#endif
	return &mail;
}

/*  ************************************************************************************************  */

/**
 * @brief c_email:  Closes the pipe to sendmail
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param mail The struct which was used for o_email
 * @return void
 **/
void c_email(Threads *thread, MailInfo * mail) //
{
    /*  - param checking modified because we don't
       have an user sending this mail.
       Certus, 02.04.2004 */

    if (!mail || !mail->pipe) { /* removed sender check */
        return;
    }

#ifdef USE_THREADS
	my_pthread_mutex_lock(&mail_lock);
#endif
    pclose(mail->pipe);
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&mail_lock);
#endif
    return;
}
/*  ************************************************************************************************  */

/**
 * @brief s_email:  Sends a Language dependent E-Mail
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The Language in which we want to send the notice (pointer to the .so file (dl_handle))
 * @param pipe The pipe to Sendmail
 * @param message The notice message itself
 * @return void
 **/
void s_email(Threads *thread, void *language, FILE *pipe, char *message, ...){
	if(pipe==NULL || message==NULL) return;

	char tmp[PUTSERVER_BUF_LEN+1];
	void* define = NULL;
	va_list va;
	
	bzero(&tmp,sizeof(tmp));

	define = dynsym(thread,language,message);
	va_start(va, (const char*)message);
	vsnprintf(tmp, sizeof(tmp)-1, (const char*)define, va);
	va_end(va);

//#ifdef USE_THREADS
//	my_pthread_mutex_lock(&mail_lock);
//#endif
	fprintf(pipe,tmp);
//#ifdef USE_THREADS
//	my_pthread_mutex_unlock(&mail_lock);
//#endif
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief MailValidate:  Check if an Email is valid (copied from Anope)
 *
 * @param message The notice message itself
 * @return int 1 if valid, 0 if not
 **/
int MailValidate(const char *email)
{
    int i, j, has_period = 0, len;
    char copy[BUFSIZE], *domain;

    static char specials[] =
        { '(', ')', '<', '>', '@', ',', ';', ':', '\\', '\"', '[', ']',
        ' '
    };

    if (!email)
        return 0;
    strcpy(copy, email);

    domain = strchr(copy, '@');
    if (!domain)
        return 0;
    *domain = '\0';
    domain++;

    /* Don't accept NULL copy or domain. */
    if (*copy == 0 || *domain == 0)
        return 0;

    /* Check for forbidden characters in the name */
    for (i = 0; i < strlen(copy); i++) {

        if (copy[i] <= 31 || copy[i] >= 127)
            return 0;
        for (j = 0; j < 13; j++)
            if (copy[i] == specials[j])
                return 0;
    }

    /* Check for forbidden characters in the domain, and if it seems to be valid. */
    for (i = 0; i < (len = strlen(domain)); i++) {
        if (domain[i] <= 31 || domain[i] >= 127)
            return 0;
        for (j = 0; j < 13; j++)
            if (domain[i] == specials[j])
                return 0;
        if (domain[i] == '.') {
            if (i == 0 || i == len - 1)
                return 0;
            has_period = 1;
        }
    }

    if (!has_period)
        return 0;

    return 1;
}

/*  ************************************************************************************************  */
