/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 202                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-24 17:29:41 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file o_sgline.c
 *   @brief This is the Module file for O SGLINE Command.
 *
 *   It contains all functions which are neccessary to initialize and use 
 *   the O SGLINE command.
 *
 *   The O SGLINE Command allows an admin to prevent
 *   the given nick (by banning it's host / ip) from connecting the Network.
 *
 *   Syntax is: /msg O SGLINE { ADD | DEL | LIST | CLEAR } [host [< duration (2d) > < reason >]]
 *
 *    ADD   Syntax is: /msg O SGLINE ADD   < host > < duration(i.e. 2d) > < reason >
 *    DEL   Syntax is: /msg O SGLINE DEL   < host >
 *    LIST  Syntax is: /msg O SGLINE LIST
 *    CLEAR Syntax is: /msg O SGLINE CLEAR
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"

#define SERVICE "O"
#define MODUL "sgline"
#define MODUL_FILE "o_sgline.so"
#define MODUL_VERSION "1.0"

MOD_BOOL O_sgline_add(Threads *thread,void *language,bool howtosend,char *snick,char *nick,char *param[]);
MOD_BOOL O_sgline_del(Threads *thread,void *language,bool howtosend,char *snick,char *nick,char *param[]);
MOD_BOOL O_sgline_list(Threads *thread,void *language,bool howtosend,char *snick,char *nick,char *param[]);
MOD_BOOL O_sgline_clear(Threads *thread,void *language,bool howtosend,char *snick,char *nick,char *param[]);

MOD_BOOL O_sgline(Threads *thread,void *language,bool howtosend,char *snick,char *nick,char *param[]);
MOD_BOOL O_sgline_help(Threads *thread,void *language,bool howtosend,char *snick,char *nick,char *param[]);
MOD_BOOL O_sgline_help_listing(Threads *thread,void *language,bool howtosend,char *snick,char *nick,char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by 
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return void
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module;

	if(!IRC.SupportSGlines){
		Module.ModErr = MOD_ERR_CANCELLED
		return Module;
	}
	
	Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"sgline option[1]","sgline option[1]");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,4,LOGINONLY,OPER_IF_DEFCON,OPERONLY,READONLY);
	addAccess (thread,&Module,ADMIN);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"sgline add",O_sgline_add,700,INFINIT,6,0);
	addCommand(thread,&Module,"sgline del",O_sgline_del,700,INFINIT,4,7);
	addCommand(thread,&Module,"sgline list",O_sgline_list,700,INFINIT,4,4);
	addCommand(thread,&Module,"sgline clear",O_sgline_clear,700,INFINIT,4,4);

	addCommand(thread,&Module,"sgline",NULL,700,INFINIT,4,0);
	addCommand(thread,&Module,"help sgline",O_sgline_help,700,INFINIT,4,5);
	addCommand(thread,&Module,"help",O_sgline_help_listing,700,INFINIT,3,3); //100 - each Admin is able to see it.

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}

/*  ************************************************************************************************  */

/**
 * @brief O SGLINE Add function for the O SGLINE Command.
 * 
 * The O SGLINE Add Command allows an admin to
 *   add an Sgline for target host
 *  
 * Syntax is: /msg O SGLINE ADD < host > < duration(i.e. 2d) > < reason >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return int MOD_SUCCESS / MOD_STOP / MOD_CONT
 **/
MOD_BOOL O_sgline_add(Threads *thread,void *language,bool howtosend,char *snick,char *nick,char *param[]){
	/* Global Variables */
	char ident[MAX_IDENT_LEN];
	char host[MAX_HOST_LEN];
	int ret = 0;

	/* Initialise Variables to (NULL) */
	bzero(&ident,sizeof(ident));
	bzero(&host,sizeof(host));
		
	/*
	 * Sgline Add Command
	 */
	if(thread->sql.access.olevel>=Dynamic_Command_Level(thread,SERVICE,MODUL,"sgline add") && param[7]){
		if(strcor(param[6],"yMdhms")){
			if(is_valid_chars(param[5],"0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ@-_.*")){
				send_lang(thread,language,howtosend,snick,nick,"NO_HOST_INPUT",param[5]);
				return MOD_CONT;
			}
			
			ret = sscanf(param[5],"%[^@]@%s",ident,host);
			if(ret!=2){
				send_lang(thread,language,howtosend,snick,nick,"NO_HOST_INPUT",param[5]);
				return MOD_CONT;
			}

			do_gline(thread,m->servername,param[5],_dynadd("%s!%s@%s",O->nick,O->ident,O->host),expire(param[6])+time(NULL),time(NULL),_dstrjoin(' ',param,7));
			send_lang(thread,language,howtosend,snick,nick,"O_COMMAND_DONE");
		}
	}else{
	/*
	 *  Error Messages
	 */
		if(!param[7])	send_lang(thread,language,howtosend,snick,nick,"NO_REASON");
		else 			send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_SUCCESS;
}

/*  ************************************************************************************************  */

/**
 * @brief O SGLINE Del function for the O SGLINE Command.
 * 
 * The O SGLINE Del Command allows an admin to
 *   delete an Sgline for target host / ip
 *  
 * Syntax is: /msg O SGLINE DEL < host >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return int MOD_SUCCESS / MOD_STOP / MOD_CONT
 **/
MOD_BOOL O_sgline_del(Threads *thread,void *language,bool howtosend,char *snick,char *nick,char *param[]){
	/* Global Variables */

	/* Initialise Variables to (NULL) */

	/*
	 * Sgline Del Command
	 */
	if(thread->sql.access.olevel>=Dynamic_Command_Level(thread,SERVICE,MODUL,"sgline del")){
		do_ungline(thread,m->servername,param[5],_dynadd("%s!%s@%s",O->nick,O->ident,O->host),time(NULL));
		send_lang(thread,language,howtosend,snick,nick,"O_COMMAND_DONE");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_SUCCESS;
}

/*  ************************************************************************************************  */

/**
 * @brief O SGLINE List function for the O SGLINE Command.
 * 
 * The O SGLINE List Command allows an admin to
 *   dump the Sgline-List
 *  
 * Syntax is: /msg O SGLINE LIST
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return int MOD_SUCCESS / MOD_STOP / MOD_CONT
 **/
MOD_BOOL O_sgline_list(Threads *thread,void *language,bool howtosend,char *snick,char *nick,char *param[]){
	/* Global Variables */
	deepSQL gline;
	int i = 0;

	/* Initialise Variables to (NULL) */
	bzero(&gline,sizeof(gline));

	/*
	 * Sgline List Command
	 */
	if(thread->sql.access.olevel>=Dynamic_Command_Level(thread,SERVICE,MODUL,"sgline list")){
		sgline.res    = doSelectMySQL(thread,"SELECT `rng`,`line`,`gline`,`duration`,`addedby`,`reason` FROM `orion_core_glines` WHERE `rng`='0'");
		sgline.fields = mysql_num_rows(sgline.res);
		
		for(i=0;i<sgline.fields;i++){
			sgline.row = mysql_fetch_row(sgline.res);
			
			if(stol(sgline.row[3])!=0)
				send_lang(thread,language,howtosend,snick,nick,"O_GLINE_LIST_LIST_SET_GLINES_WHICH_EXPIRE_AT_A_GIVEN_TIME",sgline.row[1],sgline.row[2],get_localtime(stol(sgline.row[3])),sgline.row[4],sgline.row[5]);
			else
				send_lang(thread,language,howtosend,snick,nick,"O_GLINE_LIST_LIST_PERMANENT_GLINES",sgline.row[1],sgline.row[2],sgline.row[4],sgline.row[5]);
		}
		send_lang(thread,language,howtosend,snick,nick,"END_OF_STATS_REPORT");

		mysql_free_result_myr(sgline.res); //free MySQL result
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_SUCCESS;
}

/*  ************************************************************************************************  */

/**
 * @brief O SGLINE Clear function for the O SGLINE Command.
 * 
 * The O SGLINE Clear Command allows an admin to
 *   delete all Sglines.
 *  
 * Syntax is: /msg O SGLINE CLEAR
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return int MOD_SUCCESS / MOD_STOP / MOD_CONT
 **/
MOD_BOOL O_sgline_clear(Threads *thread,void *language,bool howtosend,char *snick,char *nick,char *param[]){
	/* Global Variables */

	/* Initialise Variables to (NULL) */

	/*
	 * Sglines Clear Command
	 */
	if(thread->sql.access.olevel>=Dynamic_Command_Level(thread,SERVICE,MODUL,"sgline clear")){
		domysql("DELETE FROM `orion_core_glines` WHERE `rng`='0'");
		send_lang(thread,language,howtosend,snick,nick,"O_COMMAND_DONE");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_SUCCESS;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the O SGLINE Command (/msg O HELP SGLINE)
 * 
 * When the user tries to get help for this command, using /msg O HELP SGLINE, 
 * this function is executed.
 * 
 * Syntax: /msg O HELP SGLINE
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return int MOD_SUCCESS / MOD_STOP / MOD_CONT
 */
MOD_BOOL O_sgline_help(Threads *thread,void *language,bool howtosend,char *snick,char *nick,char *param[]){
	/*
	 * Sgline Help
	 */
	if(thread->sql.access.olevel>=Dynamic_Command_Level(thread,SERVICE,MODUL,"help sgline")) {
		if(!param[5])
			send_lang(thread,language,howtosend,snick,nick,"O_HELP_SGLINE");
			
		else 
			send_lang(thread,language,howtosend,snick,nick,"O_NO_HELP_AVAIL");

	}else send_lang(thread,language,howtosend,snick,nick,"O_UNKNOWN_COMMAND",snick);
	return MOD_SUCCESS;
}

/*  ************************************************************************************************  */

/**

 * @brief O Help List Command - GLINE part (/msg O help)
 * 
 * When the user tries to get general O help, using /msg O help, all loaded commands with 
 * a short description are returned. This function is executed, when the help function tries to include
 * the information for the GLINE command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return int MOD_SUCCESS / MOD_STOP / MOD_CONT
 */
MOD_BOOL O_sgline_help_listing(Threads *thread,void *language,bool howtosend,char *snick,char *nick,char *param[]){
	int command_level=Dynamic_Command_Level(thread,SERVICE,MODUL,"help");
	int Users_O_Level=(int)param[5];
	/*
	 * Sgline (Help listing)
	 */
	if(Users_O_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"O_SHOWCOMMANDS_SGLINE",Dynamic_Command_Level(thread,SERVICE,MODUL,"sgline"));

	return MOD_SUCCESS;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int (MOD_SUCCESS)
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_SUCCESS;
}

/*  ************************************************************************************************  */

/*O_HELP_SGLINE[]=
		"Syntax: SGLINE ADD [+expiry] mask:reason
		"SGLINE DEL mask
		"SGLINE LIST
		"SGLINE CLEAR
		"
		"Allows Services operators to manipulate the SGLINE list. If
		"a user with a realname matching an SGLINE mask attempts to
		"connect, Services will not allow it to pursue his IRC
		"session.
		"
		"SGLINE ADD adds the given realname mask to the SGLINE
		"list for the given reason (which must be given).
		"expiry is specified as an integer followed by one of d
		"(days), h (hours), or m (minutes). Combinations (such as
		"1h30m) are permitted. If a unit specifier is not
		"included, the default is days (so +30 by itself means 30
		"days). To add an SGLINE which does not expire, use +0. If the
		"realname mask to be added starts with a +, an expiry time must
		"be given, even if it is the same as the default. The
		"current SGLINE default expiry time can be found with the
		"STATS AKILL command.
		"Note: because the realname mask may contain spaces, the
		"separator between it and the reason is a colon.
		"
		"The SGLINE DEL command removes the given mask from the
		"SGLINE list if it is present. If a list of entry numbers is
		"given, those entries are deleted. (See the example for LIST
		"below.)
		"
		"The SGLINE LIST command displays the SGLINE list.
		"If a wildcard mask is given, only those entries matching the
		"mask are displayed. If a list of entry numbers is given,
		"only those entries are shown; for example:
		"SGLINE LIST 2-5,7-9
		"Lists SGLINE entries numbered 2 through 5 and 7
		"through 9.
		"
		"SGLINE VIEW is a more verbose version of SGLINE LIST, and
		"will show who added an SGLINE, the date it was added, and when
		"it expires, as well as the realname mask and reason.
		"
		"SGLINE CLEAR clears all entries of the SGLINE list.
		"
		"Limited to Services operators.

O_SHOWCOMMANDS_SGLINE*/
