/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  Please read COPYING and README for further details.
 *  Based on the original code of Services by Andy Church. 
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 202                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-24 17:29:41 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file ptcsupport.c
 *   @brief This is the Module file to handle Protocol Events.
 *
 *   It contains all functions which are neccessary to
 *   control the Protocol-Modules.
 */
/*  ************************************************************************************************  */

#include "sysconf.h"	//ifdef USE_THREADS
#include "datatypes.h"
#include "modules.h"	//for MODULE_EXT
#include "services.h"

/**
 * @brief event_init:  Initializes a Protocol struct for the given Protocol
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param protocol The pointer to the Protocol struct (store point for the Protocol functions)
 * @param dl_handle The .so Filehandle for the loaded Protocol-Library
 * @param Author The Author of the Protocol-Module
 * @param Version The Version of the Protocol-Module
 * @return void
 **/
void event_init(Threads *thread, Protocol *protocol, void *dl_handle, char *Author, char *Version){
#ifdef USE_THREADS
	my_pthread_mutex_lock(&protocol_lock);
#endif
	dynncpy(protocol->Author,  Author,sizeof(protocol->Author));
	dynncpy(protocol->Version, Version,sizeof(protocol->Version));
	protocol->dl_handle = dl_handle;
	protocol->events = NULL;
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&protocol_lock);
#endif
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief createEvent:  Creates an Event within a given Protocol (to the Protocol struct)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param protocol The pointer to the Protocol struct (store point for the Protocol functions)
 * @param call The Protocol-Identifier on which we have to call that Event
 * @param check_param The number of the Parameter we should check for equation with the "call"
 * @return void
 **/
void createEvent(Threads *thread, Protocol *protocol,
	const char *call,
	MOD_BOOL (func) (Threads *thread, size_t ac, char **av),
	size_t check_param
	)
{
	Event *e = NULL;
	
#ifdef USE_THREADS
	my_pthread_mutex_lock(&protocol_lock);
#endif
	e = (Event*)malloc(sizeof(Event));
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&protocol_lock);
#endif
    bzero(e,sizeof(Event));

	e->check_param = check_param;	
	e->func = func;
	strcpy(e->call, call);
	
#ifdef USE_THREADS
	my_pthread_mutex_lock(&protocol_lock);
#endif
    e->next = protocol->events;
    e->prev = NULL;
    
    if (protocol->events)
        protocol->events->prev = e;
        
    protocol->events = e;
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&protocol_lock);
#endif
    return;
}

/*  ************************************************************************************************  */

/**
 * @brief LoadModule:  Loads a modul
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param protocol The protocol struct where we want to save the protocol's information
 * @param IRCd The IRCd file i.e "InspIRCd"
 * @return bool true/false
 **/
bool LoadProtocol(Threads *thread, Protocol *protocol, char *IRCd){
	char library[MAX_FILEPATH_LEN];
	void (*func) (Threads *, Protocol *, void *);
	void *dl_handle = NULL;
	bzero(&library,sizeof(library));

	snprintf(library,MAX_FILEPATH_LEN,"%s/%s%s",SERVICES_PROTOCOL_DIR,IRCd,MODULE_EXT);
	dlog(thread,0,"loaded protcol file from path %s/%s%s\n",SERVICES_PROTOCOL_DIR,IRCd,MODULE_EXT);
	
#ifdef USE_THREADS
	my_pthread_mutex_lock(&protocol_lock);
#endif
	if(protocol->Author[0]=='\0'){

		dl_handle	= dynopen(thread,library,RTLD_LAZY);
		func		= dynsym (thread,dl_handle,"CreateProtocol");
		
		if(func){
			dlog(thread,0,"LoadProtocol() - handle found, executing...\n");
			(*func) (thread,protocol,dl_handle);
			dlog(thread,0,"LoadProtocol() - execution successful, returning with SUCCESS (1)...\n");
#ifdef USE_THREADS
			my_pthread_mutex_unlock(&protocol_lock);
#endif
			return 1;
		}
		dlog(thread,0,"LoadProtocol() - symbol \"CreateProtocol\" not found, returning ERROR stat (0)...\n");
#ifdef USE_THREADS
		my_pthread_mutex_unlock(&protocol_lock);
#endif
        return 0;
	}
    else{
		dlog(thread,0,"LoadProtocol() - IRCProtocol file already found, returning ERROR stat (0)...\n");
#ifdef USE_THREADS
		my_pthread_mutex_unlock(&protocol_lock);
#endif
    	return 0;
	}
}

/*  ************************************************************************************************  */
