/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  Please read COPYING and README for further details.
 *  Based on the original code of Services by Andy Church. 
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 194                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-16 18:33:19 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file stay.c
 *   @brief This is the Module file for managing ChanServ/BotServ with Akick.
 *
 *   It contains all functions which are neccessary to
 *   force the assigned Bot to stay in a channel a bit longer.
 *   (to keep settings for a period)
 *
 *	 Stay - Manages the Bots and ChanServ / Q to stay in Channel for X Seconds, after AKICK/SUSPEND/.. events
 *   So we don't lose the Channel Bans and Modes too fast (prevent User to rejoin after KickBan, if the Channel is empty)
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "global.h"
#include "services.h"
#include "modules.h"
#include "osocket.h"

static Stays *stays = NULL;

/**
 * @brief addStayChannel:  Adds a Channel to the Stay-list
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param channel The Channel for which we like to add a Stay
 * @param botnick The Channel's Bot
 * @param bot_joined Is 1 if the Channel's Bot joined already
 * @param cs_had_to_join Is 1 if ChanServ had to join the Channel (occurs if the Channel has no bot)
 * @return void
 **/
void addStayChannel(Threads *thread, char *channel, char *botnick, int bot_joined, int cs_had_to_join)
{
	Stays *stay = NULL;
	
#ifdef USE_THREADS
	my_pthread_mutex_lock(&stays_lock);
#endif
	for (stay = stays; stay; stay = stay->next) {
		if(streqnc(stay->channel,channel)){
#ifdef USE_THREADS
			my_pthread_mutex_unlock(&stays_lock);
#endif
			return;
		}
	}
	
	stay = (Stays*)malloc(sizeof(Stays));
	bzero(stay,sizeof(Stays));

	dynncpy(stay->channel,channel,sizeof(stay->channel));
	dynncpy(stay->nick,botnick,sizeof(stay->nick));
	
	stay->botjoin = bot_joined;
	stay->csjoin  = cs_had_to_join;

	stay->addtime = time(NULL);
	
	stay->next = stays;
	stay->prev = NULL;
	if (stays)
		stays->prev = stay;
	stays = stay;
	
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&stays_lock);
#endif
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief delStayChannel:  Searches for a Channel in Stay-List, if it is Found, its deleted and '1' is returned
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param channel The Channel for which we like to get (Stay chan or not!?) - Stay for that Channel is deleted, if found
 * @return bool Returns 1 if we found the Channel in our Stay list otherwise 0 is returned
 **/
bool delStayChannel(Threads *thread, char *channel){
	Stays *stay = NULL;

#ifdef USE_THREADS
	my_pthread_mutex_lock(&stays_lock);
#endif
	for (stay = stays; stay; stay = stay->next) {
		if(streqnc(stay->channel,channel)){
			
			if (stay->prev)
				stay->prev->next = stay->next;
			else
				stays = stay->next;
			if (stay->next)
				stay->next->prev = stay->prev;
			free(stay);
			
#ifdef USE_THREADS
			my_pthread_mutex_unlock(&stays_lock);
#endif
			return 1;
		}
	}
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&stays_lock);
#endif
	return 0;
}

/*  ************************************************************************************************  */

/**
 * @brief delStayNick:  Searches for a Nick in Stay-List, if it is Found, its deleted and '1' is returned
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param Nick The Nick for which we like to get (Stay chan or not!?) - Stays for that Nick are deleted, if found
 * @return bool Returns 1 if we found the Channel in our Stay list otherwise 0 is returned
 **/
bool delStayNick(Threads *thread, char *nick){
	Stays *stay = NULL;
	bool return_val = false;

#ifdef USE_THREADS
	my_pthread_mutex_lock(&stays_lock);
#endif
	for (stay = stays; stay; stay = stay->next) {
		if(streqnc(stay->nick,nick)){

			if (stay->prev)
				stay->prev->next = stay->next;
			else
				stays = stay->next;
			if (stay->next)
				stay->next->prev = stay->prev;
			free(stay);

#ifdef USE_THREADS
			my_pthread_mutex_unlock(&stays_lock);
#endif
			return_val = true;
		}
	}
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&stays_lock);
#endif
	return return_val;
}

/*  ************************************************************************************************  */

/**
 * @brief getStayChannel:  Searches for a Channel in Stay-List, if it is Found, its returned
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param channel The Channel for which we like to get (Stay chan or not!?) - Stay for that Channel is returned, if found
 * @return Stays* Returns the pointer to the element of the Stay-List
 **/
Stays *getStayChannel(Threads *thread, char *channel){
	Stays *stay = NULL;
	
#ifdef USE_THREADS
	my_pthread_mutex_lock(&stays_lock);
#endif
	for (stay = stays; stay; stay = stay->next) {
		if(streqnc(stay->channel,channel)){
#ifdef USE_THREADS
			my_pthread_mutex_unlock(&stays_lock);
#endif
			return stay;
		}
	}
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&stays_lock);
#endif
	return NULL;
}

/*  ************************************************************************************************  */

/**
 * @brief PerformStay:  Performs all Stays (deletes those which expired)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return void
 **/
void PerformStay(Threads *thread){
	Stays *stay = NULL;
	int foundstaychans = 0;
	
	char staychannels[MAX_CHANNELNAME_LEN*250];
	bzero(&staychannels,sizeof(staychannels));
	
#ifdef USE_THREADS
	my_pthread_mutex_lock(&stays_lock);
#endif
	for (stay = stays; stay; stay = stay->next) {
		if(stay->addtime+m->CSStayChannel<=time(NULL)){
			
			if(stay->botjoin){
				orion_WRCtranslate_cmd_part(thread,stay->nick,stay->channel,NULL);
				PartChannelNick(stay->nick,stay->channel);
			}
			else if(stay->csjoin == 1){
				orion_WRCtranslate_cmd_part(thread,cs->nick,stay->channel,NULL);
				PartChannelNick(cs->nick,stay->channel);
			}
			
			if(GetUsersInChannel(stay->channel)==0){
				straddnc(staychannels,toascii(39),sizeof(staychannels));
				straddns(staychannels,mysql_quote(thread,stay->channel),sizeof(staychannels));
				straddnc(staychannels,toascii(39),sizeof(staychannels));
				straddnc(staychannels,',',sizeof(staychannels));
				foundstaychans++;
			}
			DeleteStay(stay);
		}
	}
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&stays_lock);
#endif
	
	if(foundstaychans){
		staychannels[dynlen(staychannels)-1]='\0';
		
		domysql(thread,"UPDATE `orion_cs_core` SET "
			"`datecreate`='',`mode`='',`link`='',`limit`='',`flood`='',`key`='',`last_topic`=''"
			" WHERE `channel` IN (%s)",staychannels);
	
		domysql(thread,"DELETE `orion_chan_bans`,`orion_chan_expbans`,`orion_chan_users` FROM `orion_chan_bans` LEFT OUTER JOIN `orion_chan_expbans` ON `orion_chan_expbans`.`channel`=`orion_chan_bans`.`channel` LEFT OUTER JOIN `orion_chan_users` ON `orion_chan_users`.`channel`=`orion_chan_bans`.`channel` WHERE `orion_chan_bans`.`channel` IN (%s)",staychannels);

		mysql_free_quotes(thread);
	}
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief DeleteStay:  Deletes a Stay from the List
 *
 * @param stay The pointer to the Stay-List Element we want to delete
 * @return void
 **/
void DeleteStay(Stays *stay)
{
#ifdef USE_THREADS
	my_pthread_mutex_lock(&stays_lock);
#endif

	if (stay->prev)
		stay->prev->next = stay->next;
	else
		stays = stay->next;
	if (stay->next)
		stay->next->prev = stay->prev;
	free(stay);
	
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&stays_lock);
#endif
	return;
}

/*  ************************************************************************************************  */
