/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  Please read COPYING and README for further details.
 *  Based on the original code of Services by Andy Church. 
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 194                   $:	Rivision of last commit
 *      $Author:: shock              $:	Author of last commit
 *      $Date:: 2009-01-16 18:33:19 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file uline.c
 *   @brief This is the file for handling uline exceptions
 *
 *   It contains all functions which are neccessary to
 *   control various exceptions for ulines (add/del/list ulines)
 *   in MEMORY-Lists, since we need the MEMORY Speed in some cases.
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "global.h"
#include "services.h"

static Ulines *ulines = NULL;

/**
 * @brief addUline:  Adds a Uline to Memory
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param in_server The Server we want to store
 * @return Ulines* The pointer to the list-element we stored this data to
 **/
Ulines *addUline(Threads *thread, char *in_server){
#ifdef USE_THREADS
	my_pthread_mutex_lock(&ulines_lock);
#endif
	Ulines *uline = (Ulines*)malloc(sizeof(Ulines));
	bzero(uline,sizeof(Ulines));

	dynncpy(uline->server,in_server,MAX_SERVER_LEN);

	uline->next = ulines;
	uline->prev = NULL;
	if (ulines)
		ulines->prev = uline;
	ulines = uline;
	
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&ulines_lock);
#endif
	return ulines;
}

/*  ************************************************************************************************  */

/**
 * @brief GetUline:  Searches for a Ulined Server in the Memory
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param in_server The Server we want to search for
 * @return Ulines* The pointer to the list-element we found (or NULL)
 **/
Ulines *getUline(Threads *thread, char *in_server){
	Ulines *uline = NULL;
	
#ifdef USE_THREADS
	my_pthread_mutex_lock(&ulines_lock);
#endif
	for (uline = ulines; uline; uline = uline->next) {
		if(streqnc(uline->server,in_server)){
#ifdef USE_THREADS
			my_pthread_mutex_unlock(&ulines_lock);
#endif
			return uline;
		}
	}
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&ulines_lock);
#endif
	return NULL;
}

/*  ************************************************************************************************  */

/**
 * @brief delUline:  Deletes a Ulined Server from the Memory
 *               (used for special Protocols i.e. UndernetIRCu)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param in_server The Server which's information we want to delete (SQUIT)
 * @return int 0 - ERROR (No server deleted), 1- SUCCESS (Deleted target server)
 **/
bool delUline(Threads *thread, char *in_server){
	Ulines *uline			= NULL;
	
#ifdef USE_THREADS
	my_pthread_mutex_lock(&ulines_lock);
#endif
	for (uline = ulines; uline; uline = uline->next) {
		if(streqnc(uline->server,in_server))
			break;
	}

	if (!uline){
#ifdef USE_THREADS
		my_pthread_mutex_unlock(&ulines_lock);
#endif
		return 0;
	}
	
	if (uline->prev)
		uline->prev->next = uline->next;
	else
		ulines = uline->next;
	if (uline->next)
		uline->next->prev = uline->prev;
					
	free(uline);
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&ulines_lock);
#endif
	return 1;
}

/*  ************************************************************************************************  */

/**
 * @brief delAllUlines:  Deletes all ulines read from config.
 *
 * @param thread The Server which's information we want to delete (SQUIT)
 * @return int 0 - ERROR (No server deleted), 1- SUCCESS (Deleted target server)
 **/
bool delAllUlines(Threads *thread){
	Ulines *uline			= NULL;
	
#ifdef USE_THREADS
	my_pthread_mutex_lock(&ulines_lock);
#endif
	for (uline = ulines; uline; uline = uline->next) {
		if(!delUline(thread,uline->server)){
#ifdef USE_THREADS
			my_pthread_mutex_unlock(&ulines_lock);
#endif
			return 0;
		}
	}
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&ulines_lock);
#endif
	return 1;
}

/*  ************************************************************************************************  */
